function [result] = direct_lda(options)
  % Computes feature space by linear discriminant analysis for data with many features.
  % % Options for:
  % % * 
  %
  % Dependencies
  % ============
  % 
  % % generate_superellipse_shape_set.m
  % % File Exchange: 
  % % [Finish!]
  %
  %
  % References
  % ==========
  % 
  % Yu and Yang 2004, "A Direct LDA Algorithm for High-Dimensional Data--with Application to Face Recognition" (note that this implementation assumes rows are vectors, not columns as in the paper)
  %
  %
  % Log
  % ===
  %
  % 2014-07-18 tebuck: Created.

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


  
  
  % Process options:
  default_options = struct();
  default_options.data = [];
  default_options.class_assignments = [];
  default_options.should_zscore = true;
  default_options.maximum_dimensionality = inf;
  default_options.minimum_dimensionality = 0;
  default_options.minimum_r_squared = 0.1;
  % default_options.verbose = 0;
  % % default_options.verbose = 1;
  % % default_options.verbose = 2;
  
  if ~exist('options', 'var')
    options = default_options; 
  else
    options = process_options_structure(default_options, options);
  end
  
  
  test_mode = false;
  if isempty(options.data)
    
    % For a built-in test, generate some images if options.number_images == 0 (the default value):
    
    test_mode = true;
    
    % Set random seed:
    s = RandStream('mt19937ar', 'Seed', 100);
    RandStream.setDefaultStream(s);
    
    % Generate:
    options.data = zeros(0, 2);
    options.class_assignments = zeros(0, 1);
    number_classes = 10;
    number_data_per_class = 100;
    for class_index = 1:number_classes
      current_angle = (class_index - 1) / number_classes * 2 * pi;
      options.data = [options.data; mvnrnd([cos(current_angle), sin(current_angle)] * sqrt(number_classes), [.5, .25; .25, .5], number_data_per_class)];
      options.class_assignments = [options.class_assignments; zeros(number_data_per_class, 1) + class_index];
    end
    number_embedding_dimensions = 40;
    options.data = options.data * (randn(2, number_embedding_dimensions / 2) * diag(1 ./ (1:number_embedding_dimensions / 2)));
    options.data = [options.data, zeros(size(options.data, 1), number_embedding_dimensions / 2)];
    options.data = options.data + randn(size(options.data)) * 1e-1;
    
    % options.minimum_r_squared = 0.1;
    
    options.verbose = 2;
    
  end
  
  
  data = options.data;
  [number_data, number_features] = size(data);
  class_assignments = options.class_assignments;
  classes = unique(class_assignments);
  number_classes = length(classes);
  class_assignments2 = zeros(number_data, 1);
  for class_index = 1:number_classes
    class_assignments2(class_assignments == classes(class_index)) = class_index;
  end
  class_assignments = class_assignments2;
  classes = 1:number_classes;
  
  if options.should_zscore
    data = zscore(data);
  end
  
  if test_mode
    a = [data, zeros(number_data, max(3 - size(data, 2), 0))];
    scatter3(a(:, 1), a(:, 2), a(:, 3), [], class_assignments)
    colormap('jet')
    % colormap('summer')
    axis equal
    title('data', 'Interpreter', 'none')
    dbstack, beep, pause
  end
  
  
  % current_data_sphered = zeros(number_data, 0);
  current_data_sphered = [];
  data_sphered = current_data_sphered;
  last_data_sphered = [];
  current_sphering_transform = [];
  last_sphering_transform = [];
  sphering_transform = zeros(number_features, 0);
  
  should_remove_data_sphered_range = false;
  
  % if ~false && test_mode
  if false
    dbstack, beep, keyboard
  end
  
  % While we do not have enough features and the maximum number has not been reached (I think this might fail when the data are approximately collapsed to a single point):
  while ((options.minimum_dimensionality == 0 || size(data_sphered, 2) < options.minimum_dimensionality) && (size(data_sphered, 2) < min(size(data, 2), options.maximum_dimensionality))) && (~should_remove_data_sphered_range || size(data, 2) > 0)
    
    
    last_data_sphered = current_data_sphered;
    last_sphering_transform = current_sphering_transform;
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 5: My addition, to get additional discriminative dimensions;
    
    
    if ~isempty(last_data_sphered)
      
      if should_remove_data_sphered_range
        
        % Remove by projection onto remaining bases:
        % This method might be susceptible to numerical problems in null() and introduces the need to transform each sphering_transform by all previous sphering_transform_null_space values.
        error('Method not yet correctly implemented!')
        sphering_transform_null_space = null(sphering_transform');
        data = data * sphering_transform_null_space;
        number_features = size(data, 2);
        
        if false
          % Debug info:
          % Check that this is indeed the null space (this should be close to zero):
          clf, imagesc([sphering_transform, null(sphering_transform'), sphering_transform_null_space]), colorbar, colormap('summer')
          dbstack, beep, pause
          clf, imagesc(sphering_transform' * sphering_transform_null_space), colorbar, colormap('summer')
          dbstack, beep, keyboard
        end
        
      else
        
        % Remove by projection onto the plane in the same space:
        % data = data - (last_data_sphered * pinv(sphering_transform));
        data = data - (last_data_sphered * pinv(last_sphering_transform));
        
      end
      
      if test_mode
        fprintf('Original data rank: %d, current: %d\n', rank(options.data), rank(data))
      end
      
      if false
        % Debug info:
        clf, subplot(1, 2, 1), scatter3(data(:, 1), data(:, 2), data(:, 3), [], class_assignments), subplot(1, 2, 2), scatter3(recursive_options.data(:, 1), recursive_options.data(:, 2), recursive_options.data(:, 3), [], class_assignments), colormap('summer')
        dbstack, beep, keyboard
      end
      
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 1: Diagonalize the between-class scatter matrix:
    
    
    class_member_counts = reshape(arrayfun(@(x)sum(ismember(class_assignments, x)), classes), [], 1);
    class_means = cell2mat(arrayfun(@(x)mean(data(ismember(class_assignments, x), :)), classes, 'UniformOutput', false).');
    data_mean = mean(data);
    % Phi_b':
    means_centered = repmat(sqrt(class_member_counts), [1, number_features]) .* (class_means - repmat(data_mean, [number_classes, 1]));
    
    if false
      % Use Lemma 1 to get the between-class scatter matrix's spectral decomposition using that of the inner product matrix:
      [inner_product_eigenvectors, inner_product_eigenvalues] = eig(means_centered * means_centered');
      % V:
      % between_scatter_eigenvectors = means_centered * inner_product_eigenvectors;
      % between_scatter_eigenvectors = pinv(means_centered)' * inner_product_eigenvectors;
      error
      % Lambda:
      between_scatter_eigenvalues = inner_product_eigenvalues;
    else
      % Why not just use the SVD? O(m n^2), same as for (means_centered' * means_centered). My version:
      % V, Lamda:
      [between_scatter_eigenvectors, between_scatter_eigenvalues, inner_product_eigenvectors] = svd(means_centered', 'econ');
      between_scatter_eigenvalues = between_scatter_eigenvalues.^2;
    end
    
    % Remove zero eigenvalues:
    eigenvalues_to_keep = diag(between_scatter_eigenvalues);
    eigenvalues_to_keep = (eigenvalues_to_keep ./ max(eigenvalues_to_keep)) >= eps(max(eigenvalues_to_keep)) * 1e6;
    if options.maximum_dimensionality < inf
      eigenvalues_to_keep(options.maximum_dimensionality + 1:end) = false;
    end
    between_scatter_eigenvalues = between_scatter_eigenvalues(eigenvalues_to_keep, eigenvalues_to_keep);
    % Y:
    between_scatter_eigenvectors = between_scatter_eigenvectors(:, eigenvalues_to_keep);
    
    % dbstack, beep, keyboard
    
    if false
      % Debug info:
      % Show the decomposition works:
      means_centered_between_scatter_relative_error = norm((means_centered' * means_centered) - (between_scatter_eigenvectors * between_scatter_eigenvalues * between_scatter_eigenvectors')) / norm(means_centered' * means_centered)
    end
    
    
    if test_mode
      a = data * between_scatter_eigenvectors;
      a = [a, zeros(number_data, max(3 - size(a, 2), 0))];
      scatter3(a(:, 1), a(:, 2), a(:, 3), [], class_assignments)
      colormap('jet')
      % colormap('summer')
      axis equal
      title('data * between_scatter_eigenvectors', 'Interpreter', 'none')
      
      dbstack, beep, pause
    end
    
    % error('Not yet implemented!')
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 2: Unitize decomposition's diagonal matrix Lambda to get Z, the unitizer of the between-class scatter matrix:
    
    
    % Z:
    between_scatter_unitizer = between_scatter_eigenvectors * sqrt(pinv(between_scatter_eigenvalues));
    % Phi_w':
    data_class_centered = data - class_means(class_assignments, :);
    % SVD-based memory-efficient decomposition:
    % U, D_w:
    [within_scatter_eigenvectors, within_scatter_eigenvalues] = svd(between_scatter_unitizer' * data_class_centered', 'econ');
    % [within_scatter_eigenvectors, within_scatter_eigenvalues] = svd(data_class_centered * between_scatter_unitizer, 'econ');
    within_scatter_eigenvalues = within_scatter_eigenvalues.^2;
    % whos within*
    
    % Don't remove zero eigenvalues. Instead reverse eigenvalues and eigenvectors because the most important are those with the smallest eigenvalues:
    within_scatter_eigenvalues = within_scatter_eigenvalues(end:-1:1, end:-1:1);
    within_scatter_eigenvectors = within_scatter_eigenvectors(:, end:-1:1);
    
    if false
      % Debug info:
      % Show the decomposition works:
      data_class_centered_scatter_relative_error = norm((between_scatter_unitizer' * data_class_centered' * data_class_centered * between_scatter_unitizer) - (within_scatter_eigenvectors * within_scatter_eigenvalues * within_scatter_eigenvectors')) / norm(data_class_centered' * data_class_centered)
    end
    
    % % if test_mode
      % % a = [data_class_centered_scatter_relative_error, zeros(number_data, max(3 - size(data_class_centered_scatter_relative_error, 2), 0))];
      % % scatter3(a(:, 1), a(:, 2), a(:, 3), [], class_assignments)
      % % axis equal
      % % title('data_class_centered_scatter_relative_error', 'Interpreter', 'none')
      % % pause
    % % end
    
    % error('Not yet implemented!')
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 3: Diagonalize S_w'' (describe instead of name?):
    
    
    % A = U' * Z':
    lda_matrix = within_scatter_eigenvectors' * between_scatter_unitizer';
    
    % error('Not yet implemented!')
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Step 4: Reduce dimensionality:
    
    
    % x*:
    current_sphering_transform = lda_matrix' * sqrt(pinv(within_scatter_eigenvalues));
    % Filter low-correlation features (to be consistent with LDA, maybe better to use relative mean separation? Or entropy or mutual information?):
    current_data_sphered = data * current_sphering_transform;
    % Correlation doesn't cover all predictability, try mutual information and such?
    correlations = corr(current_data_sphered, class_assignments);
    % good_current_base_indices = abs(correlations) >= options.minimum_r_squared;
    good_current_base_indices = correlations.^2 >= options.minimum_r_squared;
    % if ~false && test_mode
    if false
      correlations
      good_current_base_indices_found = find(good_current_base_indices)
      % dbstack, beep, keyboard
    end
    if sum(good_current_base_indices) == 0
      % No interesting directions left, quit:
      break
    end
    current_data_sphered(:, ~good_current_base_indices) = [];
    current_sphering_transform(:, ~good_current_base_indices) = [];
    data_sphered = [data_sphered, current_data_sphered];
    sphering_transform = [sphering_transform, current_sphering_transform];
    
    % error('Not yet implemented!')
    
    if test_mode
      a = current_data_sphered;
      a = [a, zeros(number_data, max(3 - size(a, 2), 0))];
      scatter3(a(:, 1), a(:, 2), a(:, 3), [], class_assignments)
      colormap('jet')
      % colormap('summer')
      axis equal
      title('current_data_sphered', 'Interpreter', 'none')
      
      % dbstack, beep, keyboard
      dbstack, beep, pause
    end
    
    
  end
  
  
  result = struct;
  result.data_sphered = data_sphered;
  result.sphering_transform = sphering_transform;
  
  if test_mode
    
    clf, imagesc(result.data_sphered), colorbar, colormap('summer'), title('data_sphered', 'Interpreter', 'none')
    dbstack, beep, keyboard
    
  end
  
end

