function [dcor, t_statistic, p_value] = distcorr(x, y, options)
  
  % This function calculates the distance correlation between x and y.
  % Reference: http://en.wikipedia.org/wiki/Distance_correlation 
  % Date: 18 Jan, 2013
  % Author: Shen Liu (shen.liu@hotmail.com.au)

  % Change log
  % ==========
  %
  % 2014-10-24 tebuck: Removing some computations for slight increase in efficiency.
  %
  % 2014-10-27 tebuck: Distance powers.
  %
  % 2014-11-06 tebuck: Unbiased formula from [2], t-test for significance. Default option values changed to use_affine_invariance = true, use_unbiased_formula = true.
  %
  %
  % To do
  % =====
  %
  % 2014-10-24 tebuck: Add sampling to reduce computation time.
  %
  % 2014-10-27 tebuck: Partial distance correlation.
  % 
  % 
  % Dependencies
  % ============
  %
  % tebuck's pca.m
  %
  %
  % Tests
  % =====
  %
  % distcorr([], [], struct)
  % distcorr([], [], struct('number_landmarks', 2))
  %
  %
  % References
  % ==========
  %
  % [1] Szekely and Rizzo 2009, "Measuring and testing independence by correlation of distances"
  % [2] Szekely and Rizzo 2013, "The distance correlation t-test of independence in high dimension"

  % Check if the sizes of the inputs match
  if size(x,1) ~= size(y,1);
      error('Inputs must have the same number of rows')
  end
  
  % default_options.sampling_strategy = 'all'; % {all, random, sobol}
  default_options.distance_power = 1;
  default_options.number_landmarks = inf;
  default_options.principal_component_variance = inf;
  % default_options.use_affine_invariance = false;
  default_options.use_affine_invariance = true;
  % From [2]:
  % default_options.use_unbiased_formula = false;
  default_options.use_unbiased_formula = true;
  default_options.landmark_selection_method = 'distant';
  % default_options.unbiased = false; % Should eventually implement, see original author's presentation
  % Should eventually implement partial distance correlation too, see original author's presentation
  % default_options.test_mode = false;
  default_options.verbose = 0;
  
  if ~exist('options', 'var')
    options = default_options; 
  else
    options = process_options_structure(default_options, options);
  end
  
  
  if numel(x) == 0 && numel(y) == 0
    
    % warning('This does not yet test options.use_unbiased_formula!')
    
    % Test mode:
    
    addpath('/projects/cellorganizer/xruan/tcell_project/4dcell');
    addpath('export_fig');
    addpath('pmkmp');
    
    % error('Add logging and saving!')
    % base_directory = [mfilename, '/'];
    base_directory = [mfilename('fullpath'), '/'];
    base_filename = [base_directory];
    % base_filename = [base_filename, sprintf('%.4e, %d, %.4e, %s', options.distance_power, options.principal_component_variance, options.use_affine_invariance, test_options.landmark_selection_method)];
    mkdir(base_directory)
    start_job_diary(base_filename)
    
    % Set random seed:
    s = RandStream('mt19937ar', 'Seed', 696334);
    RandStream.setDefaultStream(s);
    
    relationship_scales = 10 * 2.^(0:-1:-20);
    if options.verbose > 0
      % relationship_scales, numbers_features
      relationship_scales
    end
    x_base = randn(50, 50);
    y_noise = randn(size(x_base));
    results = zeros(length(relationship_scales), size(x_base, 2));
    
    test_options = struct;
    % test_options.test_mode = true;
    test_options_index = 0;
    for distance_power = 1:2
    test_options.distance_power = distance_power;
    for use_nystrom_extension = [false, true]
    % for use_nystrom_extension = [false]
    for principal_component_variance = [.95, inf]
    test_options.principal_component_variance = principal_component_variance;
    for use_affine_invariance = [false, true]
    test_options.use_affine_invariance = use_affine_invariance;
    landmark_selection_methods = {'distant'};
    for landmark_selection_method_index = 1:length(landmark_selection_methods)
    test_options.landmark_selection_method = landmark_selection_methods{landmark_selection_method_index};
    for use_unbiased_formula = [false, true]
    test_options.use_unbiased_formula = use_unbiased_formula;
      
      test_options_index = test_options_index + 1;
      test_random_seed = 696334 + test_options_index;
      % Set random seed:
      s = RandStream('mt19937ar', 'Seed', test_random_seed);
      RandStream.setDefaultStream(s);
      
      % error
      % test_path = [base_filename, sprintf('%.4e, %d, %.4e, %d, %s.mat', test_options.distance_power, use_nystrom_extension, test_options.principal_component_variance, test_options.use_affine_invariance, test_options.landmark_selection_method)];
      % test_path = [base_filename, sprintf('dist_pow %.4e, use_nys %d, pca_var %.4e, use_invar %d, ldmk_sel %s.mat', test_options.distance_power, use_nystrom_extension, test_options.principal_component_variance, test_options.use_affine_invariance, test_options.landmark_selection_method)];
      test_path = [base_filename, sprintf('dist_pow %.4e, use_nys %d, pca_var %.4e, use_invar %d, ldmk_sel %s, unbias %d.mat', test_options.distance_power, use_nystrom_extension, test_options.principal_component_variance, test_options.use_affine_invariance, test_options.landmark_selection_method, test_options.use_unbiased_formula)];
      
      % Compute relationships for increasing number of features and increasing relative noise:
      [test_directory, test_filename] = fileparts(test_path);
      [can_start, final_name, final_exists, temp_exists] = chunk_start(test_directory, test_filename);
      try
        if can_start
          % mean_correlation = zeros(size(results));
          test_start_time = tic;
          test_start_cputime = cputime;
          numbers_features = 1:size(x_base, 2);
          for relationship_scale_index = 1:length(relationship_scales)
            relationship_scale = relationship_scales(relationship_scale_index);
            for number_features = numbers_features
              x = x_base(:, 1:number_features);
              y = x * relationship_scale + y_noise(:, 1:number_features);
              
              % With PCA:
              % test_options.number_landmarks = number_features + 2;
              % test_options.principal_component_variance = .95;
              
              if use_nystrom_extension
                test_options.number_landmarks = number_features + 2;
              else
                test_options.number_landmarks = inf;
              end
            
              current_result = distcorr(x, y, test_options);
              results(relationship_scale_index, number_features) = current_result;
              % mean_correlation(relationship_scale_index, number_features) = mean(y, 2)
              % fprintf('relationship_scale = % 2.4f, distcorr = %.4f\n', relationship_scale, current_result)
            end
          end
          test_wall_time = toc(test_start_time);
          test_cpu_time = test_start_cputime - cputime;
          save(final_name, 'relationship_scales', 'numbers_features', 'test_options', 'results', 'test_wall_time', 'test_cpu_time', 'test_options_index', 'test_random_seed')
          chunk_finish(test_directory, test_filename);
        elseif final_exists
          load(final_name)
        else
          continue
        end
      catch err
        getReport(err, 'extended')
        continue
      end
      
      if options.verbose > 0
        results
      end
      close all
      set(gcf, 'Visible', 'off')
      % imagesc(results)
      % imagesc(real(results))
      imagesc(results.^2)
      axis image
      ylabel({'Relationship scale', '(reciprocal of noise scale)'}, 'Interpreter', 'none')
      % set(gca, 'YTickLabel', relationship_scales, 'YTick', relationship_scales)
      % drawnow, y_tick_positions = get(gca, 'YTick');
      y_tick_positions = unique([1:4:length(relationship_scales), length(relationship_scales)]);
      set(gca, 'YTickLabel', relationship_scales(round(y_tick_positions)), 'YTick', round(y_tick_positions))
      xlabel('Number of features', 'Interpreter', 'none')
      h = colorbar;
      % ylabel(h, 'Distance correlation', 'Interpreter', 'none')
      ylabel(h, 'Distance R^2', 'Interpreter', 'tex')
      % colormap(copper(1024))
      colormap(pmkmp(1024, 'CubicYF'))
      caxis([0, 1])
      % figure
      % plot(diag(1 ./ (1:length(relationship_scales))) * cumsum(results, 2))
      image_filename = [final_name(1:end - 4), ', results plot'];
      dpi = 150;
      % dpi = 300;
      set(gcf, 'Color', 'white')
      export_fig([image_filename, '.png'], '-painters', '-png', '-a1', ['-r', num2str(dpi)])
      fprintf('Wrote figure "%s"\n', [image_filename, '.png'])
      
    end
    end
    end
    end
    end
    end
    
    % Testing finished:
    return
    
  end
  
  
  % Delete rows containing unobserved values
  N = any([isnan(x) isnan(y)],2);
  x(N,:) = [];
  y(N,:) = [];
  
  [n, x_number_features] = size(x);
  [n, y_number_features] = size(y);
  
  if options.principal_component_variance <= 1
    % % Uses zscore, which is OK because distance correlation is scale-invariant (although PCA's results are not scale-invariant?):
    % % a = randn(100, 5) * diag(5:-1:1) * randn(5, 10); [c1, s1, l1] = princomp(a); [c2, s2, l2] = princomp(zscore(a)); imagesc(abs(c1' * c2)), axis image, colorbar, colormap(copper(1024))
    x = pca(x, options.principal_component_variance);
    y = pca(y, options.principal_component_variance);
    
    x_prior_number_features = x_number_features;
    y_prior_number_features = y_number_features;
    [n, x_number_features] = size(x);
    [n, y_number_features] = size(y);
    if options.verbose > 0
      % fprintf('Before PCA: x_number_features = %d, y_number_features = %d\n', x_prior_number_features, y_prior_number_features)
      % fprintf('After PCA: x_number_features = %d, y_number_features = %d\n', x_number_features, y_number_features)
      fprintf('PCA reduced number of features from %d to %d (x), %d to %d (y)\n', x_number_features, x_prior_number_features, y_number_features, y_prior_number_features)
    end
  end

  if options.use_affine_invariance
    x_covariance = cov(x);
    y_covariance = cov(y);
    if false
      x_variant = x;
      y_variant = y;
    end
    % x = x * pinv(sqrtm(x_covariance));
    % y = y * pinv(sqrtm(y_covariance));
    % Compute this manually:
    [vec, val] = eig(x_covariance);
    val = diag(val);
    val(val < 0) = 0;
    val = diag(val);
    x_covariance_square_root_inverse = vec * pinv(sqrt(val)) * vec';
    [vec, val] = eig(y_covariance);
    val = diag(val);
    val(val < 0) = 0;
    val = diag(val);
    y_covariance_square_root_inverse = vec * pinv(sqrt(val)) * vec';
    x = x * x_covariance_square_root_inverse;
    y = y * y_covariance_square_root_inverse;
  end
  
  % Calculate doubly centered distance matrices for x and y
  if options.number_landmarks >= n - 1
    
    a = pdist2(x,x);
    a = a.^options.distance_power;
    amcol = mean(a);
    abar = mean(amcol);
    ajbar = ones(n, 1)*amcol;
    akbar = amcol'*ones(1, n);
    A = a - ajbar - akbar + abar;
    
    b = pdist2(y,y);
    b = b.^options.distance_power;
    bmcol = mean(b);
    bbar = mean(bmcol);
    bjbar = ones(n, 1)*bmcol;
    bkbar = bmcol'*ones(1, n);
    B = b - bjbar - bkbar + bbar;
    
    if options.use_unbiased_formula
      
      % error('Not yet implemented!')
      
      if options.distance_power ~= 1
        error('Not yet implemented! Do not know the unbiased formulae in this case.')
      end
      
      A_star = A;
      A_star(eye(n) == 0) = A(eye(n) == 0) - a(eye(n) == 0) ./ n;
      A_star(eye(n) == 1) = amcol - abar;
      A_star = A_star * n / (n - 1);
      
      B_star = B;
      B_star(eye(n) == 0) = B(eye(n) == 0) - b(eye(n) == 0) ./ n;
      B_star(eye(n) == 1) = bmcol - bbar;
      B_star = B_star * n / (n - 1);
      
      U_star_xy = sum(A_star(eye(n) == 0) .* B_star(eye(n) == 0)) - 2 / (n - 2) * sum(A_star(eye(n) == 1) .* B_star(eye(n) == 1));
      U_star_xx = sum(A_star(eye(n) == 0) .* A_star(eye(n) == 0)) - 2 / (n - 2) * sum(A_star(eye(n) == 1) .* A_star(eye(n) == 1));
      U_star_yy = sum(B_star(eye(n) == 0) .* B_star(eye(n) == 0)) - 2 / (n - 2) * sum(B_star(eye(n) == 1) .* B_star(eye(n) == 1));
      
      V_star_xy = U_star_xy / (n * (n - 3));
      V_star_xx = U_star_xx / (n * (n - 3));
      V_star_yy = U_star_yy / (n * (n - 3));
      % Square root of (2.10) in [2] (can be imaginary as its square can be negative!):
      dcor = sqrt(V_star_xy / sqrt(V_star_xx * V_star_yy));
      
      if false && imag(dcor) ~= 0
        % Debug info:
        dbstack, beep, keyboard
      end
      
    else
      
      
      % Calculate squared sample distance covariance and variances
      dcov = mean(A(:).*B(:));
      
      dvarx = mean(A(:).*A(:));
      dvary = mean(B(:).*B(:));
      
      % Calculate the distance correlation (note that this is the square root of (2.10) in [2] to match Definition 5 in [1]):
      dcor = sqrt(dcov/sqrt(dvarx*dvary));
      
      
    end
    
  else
    
    error('Not yet implemented!')
    warning('Have not yet fixed Nystrom approximation (in general or for options.use_unbiased_formula), should use sparse matrices with landmarks in the correct rows?')
    
    if options.use_unbiased_formula
      error('Not yet implemented!')
    end
    
    % X:
    x_number_landmarks = min([options.number_landmarks, n + 2, x_number_features + 2]);
    % Get landmarks and their distance matrix columns:
    [~, x_landmark_distances] = vector_landmarks(x, setfield(options, 'number_landmarks', x_number_landmarks));
    % Power as set in options:
    x_landmark_distances = x_landmark_distances.^options.distance_power;
    % Double centering as in landmark MDS:
    x_kernel = distances_to_kernel(x_landmark_distances, options);
    % Nystrom extension:
    nystrom_extension_options = struct;
    nystrom_extension_options.method = 'classical';
    [x_extended_eigenvectors, x_eigenvalues, x_landmark_indices, x_data_order_restoration] = nystrom_extension(x_kernel, nystrom_extension_options);
    % V_a approximated from Nystrom extension in equation (3) as derived in "Taraz Efficient distance correlation 011.docx":
    dvarx = (x_eigenvalues * (x_extended_eigenvectors' * x_extended_eigenvectors)).^2;
    % dvarx = mean(dvarx(:));
    dvarx = sum(dvarx(:)) / (n^2);
    % error('Not yet implemented!')
    
    % Y:
    y_number_landmarks = min([options.number_landmarks, n + 2, y_number_features + 2]);
    % Get landmarks and their distance matrix columns:
    [~, y_landmark_distances] = vector_landmarks(y, setfield(options, 'number_landmarks', y_number_landmarks));
    % Power as set in options:
    y_landmark_distances = y_landmark_distances.^options.distance_power;
    % Double centering as in landmark MDS:
    y_kernel = distances_to_kernel(y_landmark_distances, options);
    % Nystrom extension:
    nystrom_extension_options = struct;
    nystrom_extension_options.method = 'classical';
    [y_extended_eigenvectors, y_eigenvalues, y_landmark_indices, y_data_order_restoration] = nystrom_extension(y_kernel, nystrom_extension_options);
    % V_b approximated from Nystrom extension in equation (3) as derived in "Taraz Efficient distance correlation 011.docx":
    dvary = (y_eigenvalues * (y_extended_eigenvectors' * y_extended_eigenvectors)).^2;
    % dvary = mean(dvary(:));
    dvary = sum(dvary(:)) / (n^2);
    % error('Not yet implemented!')
    
    % V_ab approximated from Nystrom extension in equation (5) as derived in "Taraz Efficient distance correlation 011.docx":
    % dcov = (((y_extended_eigenvectors' * x_extended_eigenvectors) * y_eigenvalues) .* ((x_extended_eigenvectors' * y_extended_eigenvectors) * x_eigenvalues)');
    dcov = (((x_extended_eigenvectors' * y_extended_eigenvectors) * y_eigenvalues) .* ((y_extended_eigenvectors' * x_extended_eigenvectors) * x_eigenvalues)');
    % dcov = mean(dcov(:));
    dcov = sum(dcov(:)) / (n^2);
    
    % Calculate the distance correlation
    dcor = sqrt(dcov/sqrt(dvarx*dvary));
    
  end
  
  
  if nargout > 1
  
    % error('Not yet implemented!')
    
    if options.distance_power ~= 1 || options.number_landmarks < n - 1 || options.use_unbiased_formula == false
      error('Not yet implemented! (possibly invalid)')
    end
    
    % Section 3 of [2]:
    degrees_of_freedom = n * (n - 3) / 2;
    % if n < 4
      % warning('Test is not necessarily unbiased for n < 4!')
    % end
    dcor_square = real(dcor^2);
    t_statistic = sqrt(degrees_of_freedom - 1) * dcor_square / sqrt(1 - dcor_square^2);
    p_value = 1 - tcdf(t_statistic, degrees_of_freedom);
    
  end

end


function [landmark_indices, landmark_distances] = vector_landmarks(x, options)
  
  % This function calculates landmarks for a given set of points using one of a number of heuristics. landmark_distances is a sparse full-sized distance matrix with only computed values being (possibly) non-zero.
  
  % To do:
  % 18:36 2014-10-29 tebuck: Add a method that stops adding landmarks when they span the data (i.e., have the same rank as x with very small singular values removed).
  
  landmark_indices = [];
  [number_data, number_features] = size(x);
  number_landmarks = options.number_landmarks;
  landmark_distances = spalloc(number_data, number_data, number_data * number_landmarks * 2 - number_landmarks * number_landmarks);
  
  switch options.landmark_selection_method
    case 'distant'
      
      % First one computed relative to mean:
      reference_point = mean(x, 1);
      reference_points = zeros(0, number_features);
      current_distances = pdist2(x, reference_point);
      for i = 1:number_landmarks
        [~, order] = sort(current_distances, 1, 'descend');
        % landmark_indices(end + 1) = order(1);
        for j = 1:length(order)
          if ~ismember(order(j), landmark_indices)
            landmark_indices(end + 1) = order(j);
            break
          end
        end
        reference_point = x(landmark_indices(end), :);
        reference_points(end + 1, :) = reference_point;
        current_distances = pdist2(x, reference_point);
        landmark_distances(landmark_indices(end), :) = current_distances;
      end
      
    otherwise
      error('Unrecognized landmark_selection_method')
  end
  
  landmark_distances(:, landmark_indices) = landmark_distances(landmark_indices, :)';
  
  if false
    % Debug info:
    % Plot up to first three dimensions and selected landmarks:
    x = [x, zeros(size(x, 1), max(3 - size(x, 2), 0))];
    reference_points = [reference_points, zeros(size(reference_points, 1), max(3 - size(reference_points, 2), 0))];
    scatter3(x(:, 1), x(:, 2), x(:, 3), '.')
    hold all
    scatter3(reference_points(:, 1), reference_points(:, 2), reference_points(:, 3), 'o')
    hold off
    axis equal
    dbstack, beep, keyboard
  end
  
end


function [kernel] = distances_to_kernel(distances, options)
  
  % This function calculates the Landmark MDS-style kernel distances is a sparse full-sized distance matrix with only computed values being (possibly) non-zero.
  
  column_number_observations = sum(distances > 0);
  landmark_indices = column_number_observations == max(column_number_observations);
  number_landmarks = sum(landmark_indices);
  number_data = size(distances, 1);
  landmark_distances = full(distances(landmark_indices, landmark_indices));
  remainder_distances = full(distances(landmark_indices, ~landmark_indices));
  % (1) in "Taraz Efficient distance correlation 011.docx":
  landmark_centering_matrix = eye(number_landmarks) - 1 / number_landmarks;
  landmark_kernel = -(1 / 2) * landmark_centering_matrix * landmark_distances * landmark_centering_matrix;
  % Make symmetric (due to numerical error):
  landmark_kernel = (landmark_kernel + landmark_kernel.') ./ 2;

  % (2) in "Taraz Efficient distance correlation 011.docx":
  % remainder_kernel = -(1 / 2) * (remainder_distances - (1 / number_landmarks) * landmark_distances * ones(number_data - number_landmarks, 1) * ones(1, number_landmarks)) * landmark_centering_matrix;
  remainder_kernel = -(1 / 2) * (remainder_distances - (1 / number_landmarks) * landmark_distances * ones(number_landmarks, 1) * ones(1, number_data - number_landmarks));
  
  data_order_restoration = zeros(number_data, 1);
  data_order_restoration(landmark_indices) = (1:number_landmarks)';
  data_order_restoration(~landmark_indices) = (number_landmarks + 1:number_data)';
  
  kernel = distances;
  kernel(data_order_restoration(landmark_indices), data_order_restoration(landmark_indices)) = landmark_kernel;
  kernel(data_order_restoration(landmark_indices), data_order_restoration(~landmark_indices)) = remainder_kernel;
  kernel(data_order_restoration(~landmark_indices), data_order_restoration(landmark_indices)) = remainder_kernel';
  
  % error('Not yet implemented!')
  
end


