function [final_individuals] = iga(options)
  % Interactive genetic algorithm.
  % 2012-12-05 tebuck: Created.
  % 2013-01-11 tebuck: added option to have integer-valued variables and appropriate noise terms.

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  default_options.compute_function = [];
  % default_options.initial_plot_function = [];
  default_options.plot_function = [];
  default_options.plot_function_same_as_compute_function = false;
  % Have to have bounds on all variables for now:
  default_options.bounds = [];
  default_options.noise_scale = .1;
  default_options.noise_types = 'uniform';
  default_options.save_location = '.';
  default_options.number_cpus = 8;
  default_options.choose_sequentially = false;
  default_options.initial_population = [];
  default_options.change_noise_scale_with_success = false;
  default_options.integer_variables = [];
  
  if ~exist('options', 'var')
    options = default_options;
  else
    options = process_options_structure(default_options, options);
  end
  
  number_variables = size(options.bounds, 1)
  number_individuals = number_variables + 3

  current_noise_scale = options.noise_scale;
  
  function [random_individuals] = get_random_individuals()
    % Create pool of individuals from bounds:
    random_individuals = zeros(number_individuals, number_variables);
    for variable_index = 1:number_variables
      if ismember(variable_index, options.integer_variables)
        % options.bounds(variable_index, 2) - options.bounds(variable_index, 1)
        random_individuals(:, variable_index) = randint(1, number_individuals, options.bounds(variable_index, 2) - options.bounds(variable_index, 1)) + options.bounds(variable_index, 1);
      else
        random_individuals(:, variable_index) = rand(number_individuals, 1) .* (options.bounds(variable_index, 2) - options.bounds(variable_index, 1)) + options.bounds(variable_index, 1);
      end
    end
  end
  
  function [random_individuals] = get_mutated_individuals(original_individuals)
    random_individuals = zeros(number_individuals, number_variables);
    number_chosen_individuals = size(original_individuals, 1);
    number_rejected_individuals = number_individuals - number_chosen_individuals;
    mixing_matrix = rand(number_rejected_individuals, number_chosen_individuals);
    mixing_matrix = mixing_matrix ./ repmat(sum(mixing_matrix, 2), 1, number_chosen_individuals);
    random_individuals(1:number_chosen_individuals, :) = original_individuals;
    % individuals(~chosen_plots, :) = mixing_matrix * individuals(chosen_plots, :);
    random_individuals(number_chosen_individuals + 1:end, :) = mixing_matrix * random_individuals(1:number_chosen_individuals, :);
    % Mutate with continuous noise at a small scale:
    for variable_index = 1:number_variables
      variable_bounds = options.bounds(variable_index, :);
      offset = 0;
      noise_type = options.noise_types;
      if iscell(noise_type)
        noise_type = noise_type{variable_index};
      end
      if ismember(variable_index, options.integer_variables)
        random_individuals(:, variable_index) = round(random_individuals(:, variable_index));
        % fprintf('&&&&&&&&\n')
        % Add noise to integer-valued variables ("unit/standard" geometric distribution parameter of p when noise_scale is 1 and (variable_bounds(2) - variable_bounds(1)) is 1:
        p = (-1 + sqrt(5)) / 2;
        % offset = (rand() * 2 - 1) * geornd(p^(current_noise_scale * (variable_bounds(2) - variable_bounds(1)) / 100), 1);
        offset = (randint(number_individuals, 1) * 2 - 1) .* geornd(p^(current_noise_scale * (variable_bounds(2) - variable_bounds(1))), number_individuals, 1);
        % (randint(100, 1) * 2 - 1) .* geornd(p, 100, 1), pause
        % (randint(100, 1) * 2 - 1) .* geornd(p^(variable_bounds(2) - variable_bounds(1)), 100, 1), pause
        % (randint() * 2 - 1) * geornd(p^(current_noise_scale * 1e3 * (variable_bounds(2) - variable_bounds(1))), 100, 1), pause
        % offset
      else
        switch noise_type
          case 'uniform'
            % offset = rand(number_individuals, 1) .* (variable_bounds(2) - variable_bounds(1)) .* options.noise_scale;
            offset = rand(number_individuals, 1) .* (variable_bounds(2) - variable_bounds(1)) .* current_noise_scale;
          case 'normal'
            % offset = randn(number_individuals, 1) .* (variable_bounds(2) - variable_bounds(1)) .* options.noise_scale;
            offset = randn(number_individuals, 1) .* (variable_bounds(2) - variable_bounds(1)) .* current_noise_scale;
        end
      end
      random_individuals(:, variable_index) = random_individuals(:, variable_index) + offset;
      random_individuals(:, variable_index) = min(max(random_individuals(:, variable_index), variable_bounds(1)), variable_bounds(2));
    end
  end
  
  
  initial_population = options.initial_population
  previous_chosen_population = options.initial_population;
  if numel(options.initial_population) > 0
    individuals = get_mutated_individuals(options.initial_population);
  else
    individuals = get_random_individuals();
  end
  
  chosen_plots = [];
  plot_activated = false;
  function [] = plot_chosen_function(x, v)
    % chosen_plots(x) = ~chosen_plots(x);
    chosen_plots(x) = v;
    choices_made(x) = true;
    if options.choose_sequentially
      if plot_activated
        close
        drawnow
      end
    end
  end
  
  
  try
  
    if options.number_cpus > 1
      matlabpool('open', options.number_cpus);
      matlabpool size
    end

    
    converged = false;
    last_individuals = individuals - 1;
    iteration_index = 1;
    current_random_state = [];
    while ~converged
      fprintf('==== Iteration %d\n', iteration_index)
      % individuals

      iteration_filename = [options.save_location, filesep, 'iga_iter', num2str(iteration_index, '%05d'), '.mat'];
      if exist(iteration_filename, 'file')
        % load(iteration_filename)
        % load(iteration_filename, 'iteration_index', 'individuals', 'current_random_state')
        load(iteration_filename, 'iteration_index')
        load(iteration_filename, 'individuals')
        load(iteration_filename, 'current_random_state')
        if options.change_noise_scale_with_success
          load(iteration_filename, 'current_noise_scale')
        end
        RandStream.setDefaultStream(current_random_state)
        iteration_index = iteration_index + 1;
        continue
      end
      
      % fprintf('==== Iteration %d\n', iteration_index)
      individuals
      
      % Have the user pick the individuals to keep:
      close all
      drawnow
      chosen_plots = false(number_individuals, 1);
      choices_made = false(number_individuals, 1);
      % last_plot_position = [];
      if options.number_cpus == 1
        for individual_index = 1:number_individuals
          individual_filename = [options.save_location, filesep, 'iga_iter', num2str(iteration_index, '%05d'), '_indiv', num2str(individual_index, '%05d'), '.mat'];
          % individual_filename_exists = exist(individual_filename, 'file')
          if exist(individual_filename, 'file')
            % load(individual_filename)
            % a = load(individual_filename, 'chosen_plots', 'choices_made', 'current_random_state');
            chosen_plots2 = chosen_plots;
            choices_made2 = choices_made;
            load(individual_filename, 'chosen_plots');
            load(individual_filename, 'choices_made');
            load(individual_filename, 'current_random_state');
            chosen_plots2(individual_index) = chosen_plots(individual_index);
            choices_made2(individual_index) = choices_made(individual_index);
            chosen_plots = chosen_plots2;
            choices_made = choices_made2;
            RandStream.setDefaultStream(current_random_state)
            continue
          end

          % callback = @(handle_object, event_data)plot_chosen_function(individual_index);
          callback = @(x, v)plot_chosen_function(x, v);
          % if options.choose_sequentially
            % subplot_indices = [1, 1];
          % else
            subplot_indices = [number_individuals, individual_index];
          % end
          if options.plot_function_same_as_compute_function
            options.compute_function(individuals(individual_index, :), subplot_indices, callback);
          else
            options.compute_function(individuals(individual_index, :));
            options.plot_function(individuals(individual_index, :), subplot_indices, callback);
          end
          % set(gca, 'ButtonDownFcn', callback);
          drawnow
          if options.choose_sequentially
            % if ~isempty(last_plot_position)
              % set(gcf, 'Position', last_plot_position);
            % end
            % last_plot_position = get(gcf, 'Position');
            % drawnow
            plot_activated = true;
            waitfor(gcf)
            plot_activated = false;
            if ~choices_made(individual_index)
              error('Must make a choice!')
            end
          end

          % Save work:
          current_random_state = RandStream.getDefaultStream; 
          save(individual_filename, 'options', 'iteration_index', 'individuals', 'individual_index', 'chosen_plots', 'choices_made', 'current_random_state', 'current_noise_scale');
        end
        
      else
      
        % % parfor (individual_index = 1:number_individuals, options.number_cpus)
        % parfor individual_index = 1:number_individuals
          % individual_filename = [options.save_location, filesep, 'iga_iter', num2str(iteration_index, '%05d'), '_indiv', num2str(individual_index, '%05d')];
          % if exist(iteration_filename, 'file')
            % % load(iteration_filename)
            % % a = load(iteration_filename, 'chosen_plots', 'choices_made', 'current_random_state');
            % chosen_plots2 = chosen_plots;
            % choices_made2 = choices_made;
            % load(individual_filename, 'chosen_plots');
            % load(individual_filename, 'choices_made');
            % load(individual_filename, 'current_random_state');
            % chosen_plots2(individual_index) = chosen_plots(individual_index);
            % choices_made2(individual_index) = choices_made(individual_index);
            % chosen_plots = chosen_plots2;
            % choices_made = choices_made2;
            % RandStream.setDefaultStream(current_random_state)
            % continue
          % end

          % % callback = @(handle_object, event_data)plot_chosen_function(individual_index);
          % callback = @(x, v)plot_chosen_function(x, v);
          % % if options.choose_sequentially
            % % subplot_indices = [1, 1];
          % % else
            % subplot_indices = [number_individuals, individual_index];
          % % end
          % if options.plot_function_same_as_compute_function
            % options.compute_function(individuals(individual_index, :), subplot_indices, callback);
          % else
            % options.compute_function(individuals(individual_index, :));
            % options.plot_function(individuals(individual_index, :), subplot_indices, callback);
          % end
          % % set(gca, 'ButtonDownFcn', callback);
          % drawnow
          % if options.choose_sequentially
            % % if ~isempty(last_plot_position)
              % % set(gcf, 'Position', last_plot_position);
            % % end
            % % last_plot_position = get(gcf, 'Position');
            % % drawnow
            % plot_activated = true;
            % waitfor(gcf)
            % plot_activated = false;
            % if ~choices_made(individual_index)
              % error('Must make a choice!')
            % end
          % end

          % % Save work:
          % current_random_state = RandStream.getDefaultStream; 
          % save(individual_filename, 'options', 'iteration_index', 'individuals', 'individual_index', 'chosen_plots', 'choices_made', 'current_random_state');
        % end
      end
      
      if ~options.choose_sequentially
        waitfor(gcf)
      end
      number_chosen_individuals = sum(chosen_plots)
      if options.change_noise_scale_with_success
        current_noise_scale = current_noise_scale * 2^(-(number_chosen_individuals * 2 - number_individuals) / number_individuals)
      end
      if number_chosen_individuals == 0
        % error('No choices made!')
        fprintf('No individuals chosen, regenerating random individuals\n')
        % if numel(options.initial_population) > 0
        if numel(previous_chosen_population) > 0
          individuals = get_mutated_individuals(previous_chosen_population);
        else
          individuals = get_random_individuals();
        end
      else
        % Reproduce by linear interpolation between kept individuals:
        previous_chosen_population = individuals(chosen_plots, :);
        number_rejected_individuals = number_individuals - number_chosen_individuals
        % individuals(~chosen_plots, :) = get_mutated_individuals(individuals(chosen_plots, :));
        individuals = get_mutated_individuals(individuals(chosen_plots, :));
      end
      
      % Save work:
      current_random_state = RandStream.getDefaultStream; 
      save(iteration_filename, 'options', 'iteration_index', 'individuals', 'current_random_state', 'current_noise_scale');
      iteration_index = iteration_index + 1;
    end
    
    final_individuals = individuals;
  
  catch err
    if options.number_cpus > 1
      matlabpool('close');
    end
    % getReport(err, 'extended');
    rethrow(err);
  end
  
end
  
