function [t_cell_info] = master_script(options)
  % 2013-04-17 tebuck: Copied from segment_using_snakes.m.
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh
  % 2015 -2016 xruan: reformat the code and add some new functions

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Initialize logging and make subdirectory for storing logs:

  
  start_time = tic;

  filename = mfilename
  base_filename = [filename, '/']; 

  warning('off', 'MATLAB:MKDIR:DirectoryExists');
  warning('off', 'Images:initSize:adjustingMag');
  warning('off', 'MATLAB:xlsread:Mode');
  warning('off', 'MATLAB:tifftagsread:rational:badTagValueDivisionByZero');
  mkdir(filename)

  pbs_job_id = '';
  if length(dbstack) < 2
    start_job_diary(base_filename)
  end

  function [stop] = diary_cycle()
    diary off
    diary on
  end

  function [varargout] = fprintf_diary_cycle(varargin)
    varargout = cell(1, nargout);
    [varargout{:}] = fprintf(varargin{:});
    diary_cycle();
  end

  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Set defaults and process options structure:
  
  
  default_options = master_script_get_default_options;
  
  if ~exist('options', 'var')
    options = default_options;
  else
    options_prior_to_processing = options;
    options = master_script_process_options(options);
  end
  options

  named_options_set = options.named_options_set
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Start storing important information in one big structure that can be passed around (instead of using the base namespace or stuff like that):
  
  
  t_cell_info = struct();
  t_cell_info.options = options;
  t_cell_info.log_info.diary_cycle_function = @diary_cycle;
  t_cell_info.log_info.fprintf_diary_cycle_function = @fprintf_diary_cycle;
  t_cell_info.job_info.pbs_job_id = pbs_job_id;
  
  function [result_strings] = condition_name_abbreviation_function(given_strings)
    result_strings = given_strings;
    for abbreviation_index = 1:size(options.condition_name_abbreviations, 1)
      result_strings = strrep(result_strings, options.condition_name_abbreviations{abbreviation_index, 1}, options.condition_name_abbreviations{abbreviation_index, 2});
    end
  end
  
  t_cell_info.abbreviation_info.condition_name_abbreviation_function = @condition_name_abbreviation_function;
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Paths for saving intermediate and final results:
  
  
  % Build up a prefix that allows maximal reuse of common results but separation of chains of results that use a different method in an earlier link:
  accumulated_directory_options_prefix = '';
  
  should_use_short_directory_names = false
  %should_use_short_directory_names = true
  
  temp_location = [options.results_location, accumulated_directory_options_prefix, 'temp/'];
  if should_use_short_directory_names
    cell_positions_location = [options.results_location, accumulated_directory_options_prefix, 'cellpos/'];
    regions_location = [options.results_location, accumulated_directory_options_prefix, 'reg/'];
    segmentations_location = [options.results_location, accumulated_directory_options_prefix, 'seg/'];
  else
    cell_positions_location = [options.results_location, accumulated_directory_options_prefix, 'cell-positions/'];
    regions_location = [options.results_location, accumulated_directory_options_prefix, 'regions/'];
    segmentations_location = [options.results_location, accumulated_directory_options_prefix, 'segmentations/'];
  end
  if options.use_segmentation_filtering
    segmentation_filtering_interactive_prefix = '';
    if ~options.segmentation_filtering_interactive
      if should_use_short_directory_names
        segmentation_filtering_interactive_prefix = '-man';
      else
        segmentation_filtering_interactive_prefix = '-manual';
      end
    end
    if should_use_short_directory_names
      segmentation_evaluations_location = [options.results_location, accumulated_directory_options_prefix, 'segeval', segmentation_filtering_interactive_prefix, '/'];
      segmentations_filtered_location = [options.results_location, accumulated_directory_options_prefix, 'segfiltd', segmentation_filtering_interactive_prefix, '/'];
      accumulated_directory_options_prefix = [accumulated_directory_options_prefix, 'segfiltg', segmentation_filtering_interactive_prefix, '_'];
    else
      segmentation_evaluations_location = [options.results_location, accumulated_directory_options_prefix, 'segmentation-evaluations', segmentation_filtering_interactive_prefix, '/'];
      segmentations_filtered_location = [options.results_location, accumulated_directory_options_prefix, 'segmentations-filtered', segmentation_filtering_interactive_prefix, '/'];
      accumulated_directory_options_prefix = [accumulated_directory_options_prefix, 'segmentation-filtering', segmentation_filtering_interactive_prefix, '_'];
    end
  else
    segmentations_filtered_location = segmentations_location;
  end
  
  warning('Adding two-point-related code!!!')
  % Add whether using two-point synapses to the prefix:
  if options.use_two_point_synapses
    if should_use_short_directory_names
      accumulated_directory_options_prefix = [accumulated_directory_options_prefix, '2pt_'];
    else
      accumulated_directory_options_prefix = [accumulated_directory_options_prefix, 'two-point_'];
    end
  end
  
  % Add whether using landmark smoothing to the prefix:
  if should_use_short_directory_names
    accumulated_directory_options_prefix = [accumulated_directory_options_prefix, sprintf('ldmksmmeth_%s_', options.landmark_temporal_smoothing_method)];
  else
    accumulated_directory_options_prefix = [accumulated_directory_options_prefix, sprintf('landmark-smoothing-method_%s_', options.landmark_temporal_smoothing_method)];
  end
  switch options.landmark_temporal_smoothing_method
    case 'none'
    
    case 'lowess'
      % Smooth all frames' alignments:
      if should_use_short_directory_names
        accumulated_directory_options_prefix = [accumulated_directory_options_prefix, sprintf('smprpn_%.2f_', options.landmark_temporal_smoothing_proportion)];
      else
        accumulated_directory_options_prefix = [accumulated_directory_options_prefix, sprintf('smooth-proportion_%.2f_', options.landmark_temporal_smoothing_proportion)];
      end
    case 'lowess-for-outliers'
      if should_use_short_directory_names
        accumulated_directory_options_prefix = [accumulated_directory_options_prefix, sprintf('smprpn_%.2f_', options.landmark_temporal_smoothing_proportion)];
        % % Smooth only alignments of frames with this ratio of absolute intensity error to total intensity or greater:
        % Not clear yet how to implement this filtering, probably should use training data instead of hand-tuning:
        accumulated_directory_options_prefix = [accumulated_directory_options_prefix, sprintf('outlthr_%.2f_', options.landmark_temporal_smoothing_outlier_relative_error)];
      else
        accumulated_directory_options_prefix = [accumulated_directory_options_prefix, sprintf('smooth-proportion_%.2f_', options.landmark_temporal_smoothing_proportion)];
        % % Smooth only alignments of frames with this ratio of absolute intensity error to total intensity or greater:
        % Not clear yet how to implement this filtering, probably should use training data instead of hand-tuning:
        accumulated_directory_options_prefix = [accumulated_directory_options_prefix, sprintf('outlier-threshold_%.2f_', options.landmark_temporal_smoothing_outlier_relative_error)];
      end
      error('Not yet implemented')
    otherwise
      error('Unrecognized option value landmark_temporal_smoothing_method = %s', options.landmark_temporal_smoothing_method)
  end
  
  if should_use_short_directory_names
    alignments_location = [options.results_location, accumulated_directory_options_prefix, 'algt/'];
    deformations_location = [options.results_location, accumulated_directory_options_prefix, 'dfmn/'];
  else
    alignments_location = [options.results_location, accumulated_directory_options_prefix, 'alignments/'];
    deformations_location = [options.results_location, accumulated_directory_options_prefix, 'deformations/'];
  end
  
  if should_use_short_directory_names
    accumulated_directory_options_prefix = [accumulated_directory_options_prefix, sprintf('intymrphmeth_%s_', options.intensity_morphing_inside_template_method)];
  else
    accumulated_directory_options_prefix = [accumulated_directory_options_prefix, sprintf('intensity-morphing-method_%s_', options.intensity_morphing_inside_template_method)];
  end
  switch options.intensity_morphing_inside_template_method
    case 'none'
    case 'lddmm'
      error('Not yet implemented')
    case 'ants'
      warning('Being implemented')
    otherwise
      error('Unrecognized option value intensity_morphing_inside_template_method = %s', options.intensity_morphing_inside_template_method)
  end
  
  if should_use_short_directory_names
    deformations_aligned_location = [options.results_location, accumulated_directory_options_prefix, 'dfmnalgd/'];
    models_location = [options.results_location, accumulated_directory_options_prefix, 'mdl/'];
    figures_location = [options.results_location, accumulated_directory_options_prefix, 'fig/'];
  else
    deformations_aligned_location = [options.results_location, accumulated_directory_options_prefix, 'deformations-aligned/'];
    models_location = [options.results_location, accumulated_directory_options_prefix, 'models/'];
    figures_location = [options.results_location, accumulated_directory_options_prefix, 'figures/'];
  end
  
  % Directories for storing pre-deformation alignment images:
  if should_use_short_directory_names
    alignment_illustrations_location = [figures_location, 'algtill/'];
    illustrations_location = [figures_location, 'ill/'];
    originals_location = [figures_location, 'org/'];
    originals_with_landmarks_location = [figures_location, 'orgwldmk/'];
    videos_location = [figures_location, 'vid/'];
  else
    alignment_illustrations_location = [figures_location, 'alignment-illustrations/'];
    illustrations_location = [figures_location, 'illustrations/'];
    originals_location = [figures_location, 'originals/'];
    originals_with_landmarks_location = [figures_location, 'originals-with-landmarks/'];
    videos_location = [figures_location, 'videos/'];
  end
  
  % xruan 07/18/2015
  % add the paths for some figures  
  figures_for_paper_location = [options.results_location, 'figures_for_paper/'];
  figure3a_location = [figures_for_paper_location, 'figure3a/'];
  figureS1_location = [figures_for_paper_location, 'figureS1/'];
  mkdir(temp_location)
  
  mkdir(regions_location)
  mkdir(segmentations_location)
  if options.use_segmentation_filtering
    mkdir(segmentation_evaluations_location)
  end
  mkdir(segmentations_filtered_location)
  mkdir(alignments_location)
  mkdir(deformations_location)
  mkdir(deformations_aligned_location)
  mkdir(models_location)
  mkdir(figures_location)
  mkdir(cell_positions_location)

  mkdir(alignment_illustrations_location)
  mkdir(illustrations_location)
  mkdir(originals_location)
  mkdir(originals_with_landmarks_location)
  mkdir(videos_location)
  
  % xruan 07/18/2015
  % add the paths for some figures
  mkdir(figures_for_paper_location)
  mkdir(figure3a_location)
  mkdir(figureS1_location)  
  
  t_cell_info.path_info = struct();

  t_cell_info.path_info.temp_location = temp_location;
  t_cell_info.path_info.regions_location = regions_location;
  t_cell_info.path_info.segmentations_location = segmentations_location;
  if options.use_segmentation_filtering
    t_cell_info.path_info.segmentation_evaluations_location = segmentation_evaluations_location;
  end
  t_cell_info.path_info.segmentations_filtered_location = segmentations_filtered_location;
  t_cell_info.path_info.alignments_location = alignments_location;
  t_cell_info.path_info.deformations_location = deformations_location;
  t_cell_info.path_info.deformations_aligned_location = deformations_aligned_location;
  t_cell_info.path_info.models_location = models_location;
  t_cell_info.path_info.figures_location = figures_location;
  t_cell_info.path_info.cell_positions_location = cell_positions_location;
  t_cell_info.path_info.alignment_illustrations_location = alignment_illustrations_location;
  t_cell_info.path_info.illustrations_location = illustrations_location;
  t_cell_info.path_info.originals_location = originals_location;
  t_cell_info.path_info.originals_with_landmarks_location = originals_with_landmarks_location;
  t_cell_info.path_info.videos_location = videos_location;

  % xruan 07/18/2015
  % add the paths for some figures
  t_cell_info.path_info.figures_for_paper_location = figures_for_paper_location;
  t_cell_info.path_info.figure3a_location = figure3a_location;
  t_cell_info.path_info.figureS1_location = figureS1_location;
  if options.use_WAVE2_manual_seg
     t_cell_info.path_info.WAVE2_active_root_dir  = options.WAVE2_active_root_dir;
  end
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add necessary code subdirectories to the path:

  
  % Should use help to automatically derive further dependencies from this file's dependencies...
  addpath(genpath('cellorganizer_lddmm'))
  % From the Mathworks File Exchange, possibly with modifications:
  addpath(genpath('pmkmp'))
  addpath(genpath('export_fig'))
  addpath(genpath('toolbox_graph'))
  addpath(genpath('toolbox_fast_marching'))
  % addpath(genpath('toolbox_wavelet_meshes'))
  addpath(genpath('toolbox_wavelet_meshes_compute_semiregular_sphere'));
  addpath(genpath('affine'))
  addpath(genpath('BasicSnake_version2f'))
  addpath(genpath('DataHash'))
  addpath(genpath('Mesh_voxelisation'))
  addpath(genpath('dirr'))
  addpath(genpath('ordfilt3'))
  addpath(genpath('coherencefilter_version5b'))
  addpath(genpath('inpolyhedron'))
  addpath(genpath('cellstructeq'))
  addpath(genpath('lowner'))
  addpath(genpath(options.Wrapper_root));
  % addpath(genpath('pzernfun'));
  addpath(genpath('inhull'));
  addpath(genpath('bresenham_line3d'));
  addpath(genpath('lowess'));
  % addpath(genpath('haralick'));
  % addpath(genpath('DRLSE_v0'));
  addpath(genpath('Inpaint_nans'));
  addpath(genpath('NIFTI_20121012'));
  addpath(genpath('bitmap_plot_version2'));
  addpath(genpath('rotateXLabels'));
  addpath(genpath('plotboxpos'));
  addpath(genpath('bonf_holm'));
  addpath(genpath('distinguishable_colors'));
  addpath(genpath('Uniform_Sampling_of_S2'));
  % http://www.mathworks.com/matlabcentral/fileexchange/21710-string-toolkits, because R2010a:
  addpath(genpath('strings'));
  addpath(genpath('histwc'));
  % addpath(genpath('erf_'));
  addpath(genpath('FTM'));
  % http://www.mathworks.com/matlabcentral/fileexchange/25078-html-table-writer
  % addpath(genpath('html_table'));
  % % http://www.mathworks.com/matlabcentral/fileexchange/45663-xyrotalabel-rotate-x-axis-and-y-axis-labels
  % addpath(genpath('XYrotalabel'));
  % http://www.mathworks.com/matlabcentral/fileexchange/3486-xticklabel-rotate "Updated 11 Jun 2010"
  addpath(genpath('xticklabel_rotate'));
  % http://www.mathworks.com/matlabcentral/fileexchange/41379-direct-lda-and-pca+lda "Updated 19 Apr 2013"
  % addpath(genpath('directlda'));
  
  % From Jieyue Li, possibly with modifications:
  addpath(genpath('HPA_lib'))

  % Mine, possibly including code by Ivan Cao-Berg, Gustavo Rohde, Peng Tao, Wei Wang:
%   addpath(genpath('../4dcell'));
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Figure out how large the window surrounding a synapse point should be and create a template cell shape of similar size:


  [t_cell_info] = master_script_get_template_info(t_cell_info);

  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Colormap with which to display sensor intensities:
  
  
  % illustration_colormap_to_use_name = 'imagej_16_colors'
  illustration_colormap_to_use_name = 'pmkmp_cubic'
  % illustration_colormap_to_use_name = 'pmkmp_perceptual_intensity'
  % illustration_colormap_to_use_name = 'pmkmp_cubic_fixed'
  
  
  switch illustration_colormap_to_use_name
  
    case 'imagej_16_colors'
      % This is a simplified version of "16_Colors.lut" from ImageJ, what I think Dr. Wuelfing's images use:
      custom_lookup_table = [0, 0, 0; 1, 1, 171; 1, 1, 224; 0, 110, 255; 1, 171, 254; 1, 224, 254; 1, 254, 1; 190, 255, 0; 255, 255, 0; 255, 224, 0; 255, 141, 0; 250, 94, 0; 245, 0, 0; 245, 0, 178; 222, 180, 222; 255, 255, 255] ./ 255;
      illustration_colormap_to_use = custom_lookup_table;
  
    case 'pmkmp_cubic'
      illustration_colormap_to_use = pmkmp(1024, 'CubicL');
      
    case 'pmkmp_cubic_fixed'
      illustration_colormap_to_use = pmkmp(1024, 'CubicYF');
  
    case 'lab'
      % My invention using L*a*b* space and checked for the color blind with VisCheck:
      L1 = -25; L2 = 130; a1 = 48; a2 = -48; b1 = -64; b2 = 48; n = 1024;
      illustration_colormap_to_use = applycform([linspace(L1, L2, n); linspace(a1, a2, n); linspace(b1, b2, n)]', makecform('lab2srgb'));

  % % Truncated pmkmp (5/6 is approximate highest-intensity point measured w/ImageJ and VisCheck):
  % truncation_proportion = 5/6;
  % rgb = pmkmp(ceil(1024 / truncation_proportion), 'CubicL');
  % illustration_colormap_to_use = rgb(1:ceil(end * truncation_proportion), :);
  
    case 'pmkmp_perceptual_intensity'
      % Try evening out L component of pmkmp in Lab color space (works nicely!):
      srgb_to_lab_transform = makecform('srgb2lab');
      lab_to_srgb_transform = makecform('lab2srgb');
      lab = applycform(pmkmp(1024, 'CubicL'), srgb_to_lab_transform);
      % L_exponent = sqrt(2) / 2;
      L_exponent = 1;
      % L_exponent = sqrt(2);
      lab(:, 1) = linspace(0, 1, size(lab, 1)).^L_exponent .* 100;
      illustration_colormap_to_use = applycform(lab, lab_to_srgb_transform);
      
  end
  
  
  function [result_image] = grayscale_color_mapping_function(given_image)
    result_image = ind2rgb(floor(contrast_stretch(given_image) .* (size(illustration_colormap_to_use, 1) - 1)) + 1, illustration_colormap_to_use);
  end
  
  if false
    % Debug plot:
    % imshow(reshape_contrast(template_image, -1)), pause
    imshow(contrast_stretch(mean(template_image, 3))), hold on, plot(template_synapse(1), template_synapse(2), 'rx'), plot(template_centroid(1), template_centroid(2), 'gx'), plot(empirical_template_centroid(1), empirical_template_centroid(2), 'bx'), hold off, pause
  end
  
  
  t_cell_info.plotting_info = struct();
  
  warning('IDK if this works yet!')
  env_display = getenv('DISPLAY');
  env_display = regexprep(env_display, '[\r\n]', '');
  if isempty(env_display)
    env_display = '';
  end
  t_cell_info.plotting_info.running_interactively = ~isempty(env_display);

  t_cell_info.plotting_info.illustration_colormap_to_use_name = illustration_colormap_to_use_name;
  t_cell_info.plotting_info.illustration_colormap_to_use = illustration_colormap_to_use;
  t_cell_info.plotting_info.grayscale_color_mapping_function = @grayscale_color_mapping_function;
  

  % Make plots easier to read:
  if options.set_default_plot_options
    %set(0, 'DefaultAxesFontName', 'Times')
    %set(0, 'DefaultLineLineWidth', 3, 'DefaultLineMarkerSize', 15)
    % set(0, 'DefaultLineLineWidth', 2, 'DefaultLineMarkerSize', 12)
    set(0, 'DefaultLineLineWidth', 1.5, 'DefaultLineMarkerSize', 9)
    set(0, 'DefaultAxesFontName', 'Helvetica')
    set(0, 'DefaultAxesFontSize', 11)
    % set(0, 'DefaultAxesFontSize', 12)
    %set(0, 'DefaultAxesFontSize', 14)
    % set(0, 'DefaultAxesFontSize', 18)
    %set(gcf, 'PaperPositionMode', 'auto');
    
    javax.swing.UIManager.setLookAndFeel('com.sun.java.swing.plaf.motif.MotifLookAndFeel')
    % javax.swing.UIManager.setLookAndFeel('com.sun.java.swing.plaf.windows.WindowsClassicLookAndFeel')
  end
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Profile code?
  
  
  if options.use_profiling
    profile('-memory','on'); profile reset; profile on;
  end
  

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Set random seed:
  
  
  s = RandStream('mt19937ar', 'Seed', 892734);
  % xruan 06/2015 
  % update the code, setDefaultStream is out of date.
  RandStream.setGlobalStream(s);

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Create model types and associated functions:
  

  [t_cell_info] = master_script_get_model_type_info(t_cell_info);

  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Produce lists of synapse files, images, and individual synapses, frames, and cells:
  
  
  [t_cell_info] = master_script_get_synapse_info(t_cell_info);
  
  % xruan 03/15/2016 hard code the time points so that they are consistent
  % among different groups
  if strcmp(named_options_set, 'all_conditions_only_WAVE2_one_point') || strcmp(named_options_set, 'all_conditions_only_actin_one_point')  ...
        ||  strcmp(named_options_set, 'all_conditions_only_actin_two_point') || strcmp(named_options_set, 'all_conditions_one_point')
      all_relative_times_seconds_rounded = [-40, -20, 0, 20, 40, 60, 80, 100, 120, 180, 300, 420]';
      t_cell_info.synapse_info.all_relative_times_seconds_rounded = all_relative_times_seconds_rounded;
  end
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Produce windows from images corresponding to individual synapses:
  if options.use_WAVE2_manual_seg
    [t_cell_info] = master_script_WAVE2_produce_windows_get_segmenatation(t_cell_info);
  end
  
  if false
      existing_result_dir = '/projects/cellorganizer/xruan/tcell_project/tcell_excel_results_1pt_only-actin_repro_85_m1_st/';
      system(sprintf('cp %sregions %s -r', existing_result_dir, options.results_location));
  end
      
  
  if options.skip_window_saving
  
    warning('>>>> HACK, options.skip_window_saving true!')
  
  else
  
    [t_cell_info] = master_script_produce_windows(t_cell_info);
    
  end

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Segment windows corresponding to individual synapses:

  
  % if options.skip_segmentation
  
    % warning('>>>> HACK, options.skip_segmentation true!')
  
  % else
  if ~false  
    % [t_cell_info] = master_script_segment_windows_new_test(t_cell_info);
    [t_cell_info] = master_script_segment_windows_new(t_cell_info);
    
  else
      
    [t_cell_info] = master_script_segment_windows(t_cell_info); 
    
  end  
  
  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function; 

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Train a classifier to distinguish between good and bad segmentations per frame and then filter all segmentations using it

  if options.skip_segmentation_filtering
  
    warning('>>>> HACK, options.skip_segmentation_filtering true! (intentional for now due to low accuracy)')
  
  else
    
    [t_cell_info] = master_script_filter_segmentations(t_cell_info);
    
  end

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Align frames of the times series of each cell:
 
  flag_set = {'original', 'new', 'testing', 'WAVE2_active'};
  flag = flag_set{2};
  % t_cell_info.options.align_method = 'raw_original';
  if options.use_WAVE2_manual_seg
      flag = flag_set{4};
  end
  % if options.skip_alignment
    
    % warning('>>>> HACK, options.skip_alignment true!')
    
  if strcmp(flag, 'original')    
    [t_cell_info] = master_script_align_segmentations(t_cell_info);
  elseif strcmp(flag, 'new')
      [t_cell_info] = master_script_align_segmentations_new(t_cell_info); 
  elseif strcmp(flag, 'testing')
      [t_cell_info] = master_script_align_segmentations_template_testing(t_cell_info);
  elseif strcmp(flag, 'WAVE2_active')
      [t_cell_info] = master_script_align_segmentations_WAVE2_active(t_cell_info);      
  end
  
  if options.use_WAVE2_manual_seg
      return
  end  

  % error('Implementation yet unfinished below this line!')

  if options.skip_average_shape
    warning('>>>> HACK, options.skip_average_shape true!')
    % generation figures
  else
      [t_cell_info] = master_script_average_shapes(t_cell_info);
      % [t_cell_info] = master_script_average_morphed_maps_new(t_cell_info);
  end 

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Register frames of the times series of each cell to the template:
  

  % Morph these segmentations to the template:
  
  if options.skip_morphing
    
    warning('>>>> HACK, options.skip_morphing true!')
    
  else
    
    [t_cell_info] = master_script_morph_segmentations(t_cell_info);
    
  end


  % error('Implementation yet unfinished below this line!')


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Register intensities within the template after the initial morph to the template:
  

  if options.skip_morphing_inside_template
  
    warning('>>>> HACK, options.skip_morphing_inside_template true!')
    
  else
    
    % error('Implementation yet unfinished below this line!')
    
    [t_cell_info] = master_script_morph_intensities_within_template(t_cell_info);
    
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Fit parametric distributions to the protein intensities in the template space:
  
  
  % Use chunk_start to compute per-condition models in parallel:
  [t_cell_info] = master_script_build_models(t_cell_info);
  

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Statistically compare models of sensors across treatment conditions and so on:
  
  if options.skip_statistical_testing_across_conditions
    warning('>>>> HACK, options.skip_statistical_testing_across_conditions true!')
  else
    
    [t_cell_info] = master_script_compare_models(t_cell_info);
    
  end

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Hierarchical clustering of condition-sensor combinations for each model type:
  
  % options.skip_hierarchical_clustering = false;
  if options.skip_hierarchical_clustering
  
    warning('>>>> HACK, options.skip_hierarchical_clustering true!')
    
  else

    [t_cell_info] = master_script_cluster_models(t_cell_info);
    
  end 
 
  % xruan 02/23/2016 after clustering, for differernt running, run
  % different scripts for figure generation. 
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Produce images for visual evaluation:

  if options.skip_postprocess
    warning('>>>> HACK, options.skip_figure_generation true!')
    % generation figures
  else
      if strcmp(named_options_set, 'all_conditions_one_point')
          % figure3a
          [t_cell_info] = master_script_generate_pipeline_illustrations_figure3a(t_cell_info);          
          % Figure S8B
          [t_cell_info] = master_script_average_morphed_maps_new(t_cell_info);
          % Figure 5B-G
          [t_cell_info] = master_script_generate_model_figures_new(t_cell_info);
          % supplement in the website
          [t_cell_info] = master_script_model_p_value_csv_file(t_cell_info);
          % supplement video 
          [t_cell_info] = master_script_generate_model_videos(t_cell_info);
          % figure3b & standard deviation maps
          [t_cell_info] = master_script_generate_model_spatiotemporal_maps(t_cell_info);
          % FigureS1
          % [t_cell_info] = master_script_generate_pipeline_illustrations_figureS1(t_cell_info);  
          [t_cell_info] = master_script_generate_pipeline_illustrations_figureS1_new(t_cell_info);   
          % Figure S4
          [t_cell_info] = master_script_generate_fwd_cyto_radial_raw_plots(t_cell_info);
          % model maps for all conditions in the website.  
          [t_cell_info] = master_script_generate_model_maps_and_csv_files(t_cell_info);
          
      end

      if strcmp(named_options_set, 'all_conditions_only_actin_one_point')
         % figure 6B
         [t_cell_info] = master_script_generate_model_figures_no_significance_check(t_cell_info);        
      end
      
      if strcmp(named_options_set, 'all_conditions_only_actin_two_point')
         % Figure S4
         [t_cell_info] = master_script_generate_fwd_cyto_radial_raw_plots(t_cell_info);
         % Figure S3
         [t_cell_info] = master_script_generate_model_figures_no_significance_check(t_cell_info);
      end

      if strcmp(named_options_set, 'all_conditions_only_LAT_two_point')
          
          % [t_cell_info] = master_script_generate_pipeline_illustrations(t_cell_info);
          % Figure 7D
          [t_cell_info] = master_script_generate_model_figures_defined_ROI_ns_check_noeb(t_cell_info);

          [t_cell_info] = master_script_generate_fwd_cyto_radial_raw_plots(t_cell_info);
          % Figure 7C & Figure S5
          [t_cell_info] = master_script_check_contributing_features_model_clustering(t_cell_info);
          % Figure S8B
          % [t_cell_info] = master_script_average_morphed_maps_new(t_cell_info);
      end

      if strcmp(named_options_set, 'all_conditions_only_WAVE2_one_point')
         %figure 6A
         [t_cell_info] = master_script_generate_model_figures_new_wave2(t_cell_info);

      end

      if ~false
          % counting the cell numbers and frame numbers. 
         [t_cell_info] = master_script_count_cell_pairs(t_cell_info);
      end
  end
  
  if options.use_profiling
    p = profile('info');
    profile_filename = [...
      base_filename, ...
      '_profile_results', '.mat']
    profsave(p, profile_filename);
    save([profile_filename, '.mat'], 'p');
  end

  time_so_far = toc(start_time)

  diary('off')
  
end


function [result_image] = masked_histogram_equalization(given_image, given_mask)
  relevant_values = given_image(given_mask);
  relevant_value_extrema = prctile(relevant_values, [0, 100]);
  relevant_values = histeq(contrast_stretch(relevant_values), 64);
  result_image = given_image;
  % result_image(~given_mask) = relevant_value_extrema(1);
  result_image(given_mask) = relevant_values .* diff(relevant_value_extrema) + relevant_value_extrema(1);
end  

