function [t_cell_info] = master_script_align_segmentations_new(t_cell_info, options)
  % Align frames of the times series of each cell.
  %
  % 2013-11-16 tebuck: Copied from master_script_filter_segmentations.m.
  % 2016-02-21 xruan: change the parallel computing. If the aligned mat
  % exists, then just skip it but not save as an empty file. 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  fprintf_diary_cycle_function = t_cell_info.log_info.fprintf_diary_cycle_function;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  alignments_location = t_cell_info.path_info.alignments_location;
  alignment_illustrations_location = t_cell_info.path_info.alignment_illustrations_location;
  deformations_location = t_cell_info.path_info.deformations_location;

  run_relative_path_list = t_cell_info.synapse_info.run_relative_path_list;
  run_key_list = t_cell_info.synapse_info.run_key_list;
  % run_data_list = t_cell_info.synapse_info.run_data_list;
  run_images_list = t_cell_info.synapse_info.run_images_list;
  run_brightfield_images_list = t_cell_info.synapse_info.run_brightfield_images_list;
  run_image_frame_indices_list = t_cell_info.synapse_info.run_image_frame_indices_list;
  run_number_cells = t_cell_info.synapse_info.run_number_cells;
  run_number_frames = t_cell_info.synapse_info.run_number_frames;
  run_point_tracks = t_cell_info.synapse_info.run_point_tracks;
  run_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks;
  run_relative_times = t_cell_info.synapse_info.run_relative_times;
  run_number_relative_times = t_cell_info.synapse_info.run_number_relative_times;
  all_relative_times = t_cell_info.synapse_info.all_relative_times;
  number_all_relative_times = t_cell_info.synapse_info.number_all_relative_times;
  all_relative_times_seconds_data = t_cell_info.synapse_info.all_relative_times_seconds_data;
  all_relative_times_seconds = t_cell_info.synapse_info.all_relative_times_seconds;
  number_runs = t_cell_info.synapse_info.number_runs;
  
  template_centroid = t_cell_info.template_info.template_centroid;
  template_synapse = t_cell_info.template_info.template_synapse;
  template_synapse_segment = t_cell_info.template_info.template_synapse_segment;
  template_image = t_cell_info.template_info.template_image;

  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  cropped_size = t_cell_info.preprocessing_info.cropped_size;
    
  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  
  template_volume = sum(template_image(:));
  % This should eventually be larger to eliminate degenerate segmentations, e.g., 5% of template volume and/or high surface-to-volume ratio:
  % segmentation_volume_threshold = 0;
  % Must be more than options.segmentation_volume_minimum_ratio of template_volume:
  segmentation_volume_threshold = template_volume * master_script_options.segmentation_volume_minimum_ratio;
  

  % Could use either of these possibly faster methods than used in segment_using_snakes:
  % 1. Only align frames pair-wise.
  % 2. Only align landmarks.

  
  function [result_landmark] = segmentation_2d_to_3d_landmark_function(given_segmentation_image, given_landmark)
    % Returns a segmentation-derived 3D annotation from a 2D annotation by a weighted centroid in Z.
    [x, y, z] = meshgrid(1:size(given_segmentation_image, 2), 1:size(given_segmentation_image, 1), 1:size(given_segmentation_image, 3));
    horizontal_distances = sqrt((x - given_landmark(1)).^2 + (y - given_landmark(2)).^2);
    % Normalize distances:
    median_distance = median(horizontal_distances(given_segmentation_image(:) > 0));
    distance_scale = 1;
    horizontal_weights = exp(-(horizontal_distances .* distance_scale).^2 ./ median_distance);
    % if false
      % % Debug plot:
      % imshow(contrast_stretch(mean(horizontal_weights, 3))), pause
      % imshow(contrast_stretch(mean(given_segmentation_image, 3))), pause
      % imshow(contrast_stretch(mean(horizontal_weights .* given_segmentation_image, 3))), pause
    % end
    % segmentation_volume = sum(given_segmentation_image(:));
    horizontal_weights = horizontal_weights .* given_segmentation_image;
    horizontal_weights = horizontal_weights ./ sum(horizontal_weights(:));
    synapse_z = sum(z(:) .* horizontal_weights(:));
    
    centroid = image_centroid(given_segmentation_image);
    
    result_landmark = [given_landmark, synapse_z];
  end
  
  function [result_landmark] = segmentation_2d_to_3d_landmark_function_mean(given_segmentation_image, given_landmark)
    % Returns a segmentation-derived 3D annotation from a 2D annotation by a weighted centroid in Z.
    slice_num = size(given_segmentation_image, 3);
    synapse_z = sum(reshape(sum(sum(given_segmentation_image, 1), 2), [1, slice_num]) .* [1 : slice_num]) ...
        / sum(given_segmentation_image(:), 1);
    
    centroid = image_centroid(given_segmentation_image);
    
    result_landmark = [given_landmark, synapse_z];
  end


  function [result_landmark] = segmentation_2d_to_3d_landmark_function_max(given_segmentation_image, given_landmark)
    % Returns a segmentation-derived 3D annotation from a 2D annotation by a weighted centroid in Z.
    total_intensity = sum(sum(given_segmentation_image, 1), 2);
    [~, synapse_z] = max(total_intensity); 
    result_landmark = [given_landmark, synapse_z];
  end


  function [result_landmarks] = segmentation_landmark_function(given_window_image, given_segmentation_image, given_annotations, align_method)
    % Returns synapse and segmentation-derived centroid.
    if nargin < 4
        align_method = 'original'
    end
    if master_script_options.use_two_point_synapses
      % error
      % Need to transform all annotations during window/region generation, pass in here...
      given_seg_raw_image = given_window_image(given_segmentation_image > 0);
      switch align_method
          case 'original'
            result_landmarks = [segmentation_2d_to_3d_landmark_function(given_segmentation_image, given_annotations(2, :)); ...
                               segmentation_2d_to_3d_landmark_function(given_segmentation_image, given_annotations(3, :))];
          case 'segment_centroid'
            result_landmarks = [segmentation_2d_to_3d_landmark_function_mean(given_segmentation_image, given_annotations(2, :)); ...
                               segmentation_2d_to_3d_landmark_function_mean(given_segmentation_image, given_annotations(3, :))];
          case 'segment_maximum'
            result_landmarks = [segmentation_2d_to_3d_landmark_function_max(given_segmentation_image, given_annotations(2, :)); ...
                               segmentation_2d_to_3d_landmark_function_max(given_segmentation_image, given_annotations(3, :))];
          case 'raw_original'
            result_landmarks = [segmentation_2d_to_3d_landmark_function(given_seg_raw_image, given_annotations(2, :)); ...
                               segmentation_2d_to_3d_landmark_function(given_seg_raw_image, given_annotations(3, :))];            
          case 'raw_centroid'
            result_landmarks = [segmentation_2d_to_3d_landmark_function_mean(given_seg_raw_image, given_annotations(2, :)); ...
                               segmentation_2d_to_3d_landmark_function_mean(given_seg_raw_image, given_annotations(3, :))];
          case 'raw_maximum'
            result_landmarks = [segmentation_2d_to_3d_landmark_function_max(given_seg_raw_image, given_annotations(2, :)); ...
                               segmentation_2d_to_3d_landmark_function_max(given_seg_raw_image, given_annotations(3, :))];
      end
      % keyboard
      % error
    else
      given_synapse_point = window_center_2d;
      result_landmarks = segmentation_2d_to_3d_landmark_function(given_segmentation_image, given_synapse_point);
    end
    
    centroid = image_centroid(given_segmentation_image);
    
    result_landmarks = [result_landmarks; centroid];
  end
  
  
  function [visualize_image] = visulaize_landmarks(landmarks, im_size)
    % first row: left end, using lean cross
    % second row: right end, using small cross
    % third row: centroid, using large cross
    % im_size: the size of the desirable image
    landmarks;
    rounded_landmarks = round(landmarks);
    rounded_landmarks = rounded_landmarks(:, [2, 1, 3]);

    in_image_function = @(x) x(1) > 0 & x(1) <= im_size(2) & x(2) > 0 & x(2) <= im_size(1) & x(3) > 0 & x(3) <= im_size(3);
    se =[1     1     0     0     0     1     1
         1     1     1     0     1     1     1
         0     1     1     1     1     1     0
         0     0     1     1     1     0     0
         0     1     1     1     1     1     0
         1     1     1     0     1     1     1
         1     1     0     0     0     1     1];
    left_raw_image = zeros(im_size);
    if sum(rounded_landmarks(1, :)) > 0 && in_image_function(rounded_landmarks(1, :))
        left_raw_image(rounded_landmarks(1, 1), rounded_landmarks(1, 2), rounded_landmarks(1, 3)) = 1;
    end
    left_dilate_image = imdilate(left_raw_image, se);

    se = zeros(7);
    se(3 : 5, :) = 1;
    se(:, 3 : 5) = 1;
    right_raw_image = zeros(im_size);
    if sum(rounded_landmarks(2, :)) > 0 && in_image_function(rounded_landmarks(2, :))
        right_raw_image(rounded_landmarks(2, 1), rounded_landmarks(2, 2), rounded_landmarks(2, 3)) = 1;
    end
    right_dilate_image = imdilate(right_raw_image, se);

    se = zeros(11);
    se(5 : 7, :) = 1;
    se(:, 5 : 7) = 1;
    centroid_raw_image = zeros(im_size);
    if sum(rounded_landmarks(3, :)) > 0 && in_image_function(rounded_landmarks(3, :))
        centroid_raw_image(rounded_landmarks(3, 1), rounded_landmarks(3, 2), rounded_landmarks(3, 3)) = 1;
    end
    centroid_dilate_image = imdilate(centroid_raw_image, se);   

    visualize_image = left_dilate_image + right_dilate_image + centroid_dilate_image;
 end


  template_landmarks = [template_synapse; template_centroid];
  warning('Adding two-point-related code!!!')
  if master_script_options.use_two_point_synapses
    % error
    template_landmarks = [template_synapse_segment; template_centroid];
    % % error
    % % Make the sides of the annotation 3D:
    % result_landmarks = [segmentation_2d_to_3d_landmark_function(given_segmentation_image, given_additional_annotations(1, :)); segmentation_2d_to_3d_landmark_function(given_segmentation_image, given_additional_annotations(2, :))];
  end
  
  debug_generate_figures = false;
  % debug_generate_figures = true

  % debug_print_alignment_skip_reasons = false;
  debug_print_alignment_skip_reasons = true
  
  debug_keyboard_alignment_skips = false;
  % debug_keyboard_alignment_skips = true
  
  debug_recompute_all_alignments = false;
  % debug_recompute_all_alignments = true
  
  if debug_recompute_all_alignments
    warning('>>>> HACK, debug_recompute_all_alignments true!')
  end
  
  t_cell_info.alignment_info = struct();
  t_cell_info.alignment_info.segmentation_landmark_function = @segmentation_landmark_function;
  t_cell_info.alignment_info.template_volume = template_volume;
  t_cell_info.alignment_info.segmentation_volume_threshold = segmentation_volume_threshold;
  %xruan 06/30/2015
  % t_cell_info.alignment_info.align_method = align_method;
  t_cell_info.alignment_info.align_method = master_script_options.align_method;
  
  if master_script_options.skip_alignment
    
    warning('>>>> HACK, options.skip_alignment true!')
    
  else
    
    if debug_keyboard_alignment_skips
      alignment_list = dir([alignments_location, '*.mat']); alignment_list = {alignment_list.name}.'; alignment_list(~cellfun(@isempty, regexp(alignment_list, '_finished.mat$'))) = [];
      deformation_list = dir([deformations_location, '*.mat']); deformation_list = {deformation_list.name}.';
      common = unique([alignment_list(ismember(alignment_list, deformation_list)); deformation_list(ismember(deformation_list, alignment_list))]);
      distinct = unique([alignment_list(~ismember(alignment_list, deformation_list)); deformation_list(~ismember(deformation_list, alignment_list))]);
      whos alignment_list deformation_list common distinct
      distinct
      % Remove related alignments and deformations:
      for distinct_index = 1:length(distinct)
        delete([alignments_location, distinct{distinct_index}]);
        % delete([alignments_location, distinct{distinct_index}(1:strfind(distinct{distinct_index}, '_frame') - 1), '*_finished.mat']);
        delete([deformations_location, distinct{distinct_index}]);
      end
    end

    
    % Compute transformations:
    
    
    % Keep running until everything is finished:
    % run_cell_indices_uncomputed = cellfun(@(x)true(size(x, 1), 1), run_point_tracks, 'UniformOutput', false);
    % run_cell_indices_uncomputed = cellfun(@(x)max(isempty(x), [], 2), run_point_tracks, 'UniformOutput', false);
    run_cell_indices_uncomputed = cell(number_runs, 1);
    for run_index = 1:number_runs
      number_cells = t_cell_info.synapse_info.run_number_cells(run_index);
      number_frames = run_number_frames(run_index);
      cell_synapse_point_tracks = run_point_tracks{run_index};
      run_cell_index_frame_indices_uncomputed{run_index} = false(size(cell_synapse_point_tracks, 1), number_frames);
      run_cell_indices_uncomputed{run_index} = false(number_cells, 1);
      for current_cell_index = 1:number_cells
        for frame_index = 1:number_frames
          current_synapse_center_rounded = cell_synapse_point_tracks{current_cell_index, frame_index};
          if isempty(current_synapse_center_rounded)
            continue
          end
          
          first_frame_index = find(~cellfun(@isempty, cell_synapse_point_tracks(current_cell_index, :)), 1);
          if isempty(first_frame_index)
            run_cell_indices_uncomputed{run_index}(current_cell_index) = false;
            continue
          end
          
          % run_cell_index_frame_indices_uncomputed{run_index}(current_cell_index, frame_index) = true;
          run_cell_indices_uncomputed{run_index}(current_cell_index) = true;
        end
      end
    end
    
    recently_did_any_work = false;
    did_any_work = false;
    
    % while any(~cellfun(@isempty, run_cell_indices_uncomputed))
    while any(cellfun(@(x)any(x(:)), run_cell_indices_uncomputed))
      
      recently_did_any_work = false;
      did_any_work = false;
      
      for run_index = reshape(find(~cellfun(@isempty, run_cell_indices_uncomputed)), 1, [])
        % Prepare useful variables:
        run_relative_path = run_relative_path_list{run_index};
        run_key = run_key_list{run_index};
        run_image_path = run_images_list{run_index};
        number_cells = run_number_cells(run_index);
        number_frames = run_number_frames(run_index);
        cell_synapse_point_tracks = run_point_tracks{run_index};
        cell_relative_time_tracks = run_relative_time_tracks{run_index};
        
        % fprintf_diary_cycle_function('Aligning segmentations for run "%s"\n', run_key)
        printed_run_work_message = false;
        
        % Go through frames of each cell, doing Procrustes analysis to align it to the last:
        % for current_cell_index = reshape(find(run_cell_indices_uncomputed{run_index}), 1, [])
        for current_cell_index = 1:number_cells

          if ~run_cell_indices_uncomputed{run_index}(current_cell_index)
            continue
          end

          % This takes a long time, so do not repeat work by checking for a file indicating this cell's alignments are finished:
          first_frame_index = find(~cellfun(@isempty, cell_synapse_point_tracks(current_cell_index, :)), 1);
          if isempty(first_frame_index)
            % if debug_print_alignment_skip_reasons, fprintf_diary_cycle_function('Skipping alignment %s cell %d because no first frame in cell_synapse_point_tracks\n', run_key, current_cell_index), end
            % if debug_keyboard_alignment_skips, dbstack, beep, keyboard, end
            % run_cell_indices_uncomputed{run_index}(run_cell_indices_uncomputed{run_index} == current_cell_index) = [];
            run_cell_indices_uncomputed{run_index}(current_cell_index) = false;
            continue
          end
          first_frame_synapse_center_rounded = cell_synapse_point_tracks{current_cell_index, first_frame_index}(1:2);
          current_cell_alignment_finished_filename = [run_key, sprintf('_frame%05d_synapse%05d,%05d', first_frame_index, first_frame_synapse_center_rounded), '_finished'];
          % xruan 02/24/2016
          % [finished_can_start, finished_final_name, finished_final_exists] = chunk_start(alignments_location, current_cell_alignment_finished_filename);
          [finished_can_start, finished_final_name, finished_final_exists] = chunk_start_clean(alignments_location, current_cell_alignment_finished_filename);
          
          if finished_final_exists
            % run_cell_indices_uncomputed{run_index}(run_cell_indices_uncomputed{run_index} == current_cell_index) = [];
            run_cell_indices_uncomputed{run_index}(current_cell_index) = false;
          end
          
          if debug_recompute_all_alignments && ~isempty(regexp(run_key, 'Actin Regulators - B7 Blockade_CPalpha1.xls'))
          % elseif debug_keyboard_alignment_skips && any(~cellfun(@isempty, strfind(distinct, run_key)))
          % elseif debug_keyboard_alignment_skips
            % dbstack, beep, keyboard
          elseif debug_keyboard_alignment_skips && any(~cellfun(@isempty, strfind(distinct, strrep(current_cell_alignment_finished_filename, '_finished', ''))))
            fprintf_diary_cycle_function('debug_keyboard_alignment_skips = true, so repeating ''%s''\n', current_cell_alignment_finished_filename)
          elseif ~finished_can_start
            % if debug_print_alignment_skip_reasons, fprintf_diary_cycle_function('Skipping alignment %s cell %d because ~finished_can_start\n', run_key, current_cell_index), end
            % if debug_keyboard_alignment_skips, dbstack, beep, keyboard, end
            continue
          end
          

          % Collect landmarks for all frames of this cell:
          current_cell_landmarks = cell(number_frames, 1);
          current_cell_volumes = zeros(number_frames, 1);
          for frame_index = 1:number_frames
            current_frame_image_filename = run_image_path{frame_index};
            if isempty(current_frame_image_filename)
              continue
            end
            current_synapse_center_rounded = cell_synapse_point_tracks{current_cell_index, frame_index};
            if isempty(current_synapse_center_rounded)
              continue
            end
            current_synapse_annotations = cell_synapse_point_tracks{current_cell_index, frame_index};
            current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
            
            cell_alignment_filename = [run_key, sprintf('_frame%05d_synapse%05d,%05d', frame_index, current_synapse_center_rounded)];
            cell_segmentation_filename = [segmentations_filtered_location, cell_alignment_filename];
            cell_region_filename = [regions_location, cell_alignment_filename];
            
            % Shouldn't be any additional exceptions, disabled:
            % if ~exist([cell_segmentation_filename, '.mat'], 'file')
              % % if debug_print_alignment_skip_reasons, fprintf_diary_cycle_function('Skipping alignment %s current_synapse_center_rounded %d,%d frame %d because %s does not exist!\n', run_key, current_synapse_center_rounded, frame_index, cell_segmentation_filename), end
              % % if debug_keyboard_alignment_skips, dbstack, beep, keyboard, end
              % continue
            % end
            
            a = load([cell_region_filename, '.mat']);
            current_synapse_annotations_transform = a.current_synapse_annotations_transform;            
            current_window_image = a.current_window_image;
            
            b = load([cell_segmentation_filename, '.mat']);
            segmentation_structure = b.segmentation_structure;
            window_number_slices = b.window_number_slices;
            
            current_synapse_annotations_transformed = reshape(current_synapse_annotations, 2, []).';
            current_synapse_annotations_transform_function = @(given_annotations)(given_annotations - repmat(current_synapse_annotations_transform.current_synapse_center_rounded, [size(given_annotations, 1), 1]) + current_synapse_annotations_transform.current_room) .* (current_synapse_annotations_transform.room / current_synapse_annotations_transform.current_room) + 1;
            % current_synapse_annotations_transform_function = @(given_annotations)(given_annotations - repmat(current_synapse_annotations_transform.current_synapse_center_rounded, [1, size(given_annotations, 2) / 2]) + current_synapse_annotations_transform.current_room) .* (current_synapse_annotations_transform.room / current_synapse_annotations_transform.current_room) + 1;
            current_synapse_annotations_transformed = current_synapse_annotations_transform_function(current_synapse_annotations_transformed);
            
            current_segmentation_image = segmentation_rasterization_function(segmentation_structure, [window_size_2d, window_number_slices]);
            % current_segmentation_image = segmentation_rasterization_function(segmentation_structure);
            % current_landmarks = segmentation_landmark_function(current_segmentation_image, current_synapse_annotations);
            align_method = master_script_options.align_method;
            current_landmarks = segmentation_landmark_function(current_window_image, current_segmentation_image, current_synapse_annotations_transformed, align_method);
            
            % xruan 07/15/2015 check the relative end points of synapse to
            % centroid. The original coordinate system is left-handed, so
            % if the z coordinate of the the cross product is negative, it
            % means right end point is in the relative left, and need
            % change as left point. 
            
            if master_script_options.use_two_point_synapses
                CL = current_landmarks(1, :) - current_landmarks(3, :);
                CR = current_landmarks(2, :) - current_landmarks(3, :);
                cross_product = cross(CL, CR);
                if cross_product(3) < 0
                    current_landmarks = current_landmarks([2, 1, 3], :);
                end
            end
                             
            
            if false
              current_landmarks
              imshow(reshape_contrast(current_segmentation_image, -1)), pause
            end
            
            % Cell volume:
            current_volume = sum(current_segmentation_image(:));
            is_segmentation_low_volume = current_volume <= segmentation_volume_threshold;
            
            if any(~isfinite(current_landmarks(:)))
              % If the segmentation image is blank, current_landmarks will have nans:
              if debug_print_alignment_skip_reasons, fprintf_diary_cycle_function('Skipping alignment %s current_synapse_center_rounded %d,%d frame %d, because no finite landmarks!\n', run_key, current_synapse_center_rounded, frame_index), end
              % % if debug_keyboard_alignment_skips
              % % if debug_keyboard_alignment_skips && is_segmentation_low_volume
              % if debug_keyboard_alignment_skips && ~is_segmentation_low_volume
                % current_landmarks
                % current_volume
                % dbstack, beep, keyboard
              % end
              continue
            end
            
            % At some point turn this off and filter in model building so can change this threshold to taste without recomputing:
            if is_segmentation_low_volume
              % If the segmentation image is blank, current_landmarks might not have nans?:
              if debug_print_alignment_skip_reasons, fprintf_diary_cycle_function('Skipping alignment %s current_synapse_center_rounded %d,%d frame %d, because segmentation has low volume!\n', run_key, current_synapse_center_rounded, frame_index), end
              % if debug_keyboard_alignment_skips
                % current_landmarks
                % current_volume
                % dbstack, beep, keyboard
              % end
              continue
            end

            current_cell_landmarks{frame_index} = current_landmarks;
            current_cell_volumes(frame_index) = current_volume;
          end
          
          good_cell_landmark_indices = find(~cellfun(@isempty, current_cell_landmarks));
          
          current_cell_landmarks_unsmoothed = current_cell_landmarks;
          
          
          % if master_script_options.use_landmark_temporal_smoothing
          if strcmp(master_script_options.landmark_temporal_smoothing_method, 'lowess')
            
            % Smooth all landmark points across time:
            
            current_cell_synapses = cell2mat(cellfun(@(x)x(1, :), current_cell_landmarks(good_cell_landmark_indices, :), 'UniformOutput', false));
            current_cell_centroids = cell2mat(cellfun(@(x)x(2, :), current_cell_landmarks(good_cell_landmark_indices, :), 'UniformOutput', false));
            
            % Smooth synapse directions rather than synapse points:
            current_cell_synapse_directions = current_cell_synapses - current_cell_centroids;
            current_cell_synapse_direction_lengths = sqrt(sum(current_cell_synapse_directions.^2, 2));
            current_cell_synapse_directions = current_cell_synapse_directions .* repmat(1 ./ current_cell_synapse_direction_lengths, 1, 3);
            
            columns_to_smooth = [current_cell_centroids, current_cell_synapse_directions, current_cell_synapse_direction_lengths];
            
            current_number_landmarks = size(columns_to_smooth, 1);
            if current_number_landmarks > 2
              for coordinate_index = 1:size(columns_to_smooth, 2)
                % Use at least one other point:
                [smoothed_coordinate] = lowess([good_cell_landmark_indices, columns_to_smooth(:, coordinate_index)], max(master_script_options.landmark_temporal_smoothing_proportion, 2 / current_number_landmarks));
                columns_to_smooth(:, coordinate_index) = smoothed_coordinate(:, 3);
              end
            end
            
            current_cell_centroids_smoothed = columns_to_smooth(:, 1:3);
            current_cell_synapse_directions_smoothed = columns_to_smooth(:, 4:6);
            % Normalize synapse directions:
            current_cell_synapse_directions_smoothed = current_cell_synapse_directions_smoothed .* repmat(1 ./ sqrt(sum(current_cell_synapse_directions_smoothed.^2, 2)), 1, 3);
            current_cell_synapse_direction_lengths_smoothed = columns_to_smooth(:, 7);
            
            % Reconstruct the synapse points and concatenate with the centroids:
            current_cell_landmarks_smoothed = [current_cell_centroids_smoothed + current_cell_synapse_directions_smoothed .* repmat(current_cell_synapse_direction_lengths_smoothed, 1, 3), current_cell_centroids_smoothed];
            
            if debug_generate_figures
              current_cell_synapses
              current_cell_synapses_smoothed = current_cell_landmarks_smoothed(:, 1:3)
              current_cell_synapses_smoothing_error = current_cell_synapses - current_cell_synapses_smoothed
              current_cell_centroids
              current_cell_centroids_smoothed
              current_cell_centroids_smoothing_error = current_cell_centroids - current_cell_centroids_smoothed
            end
            
            for good_frame_index = 1:length(good_cell_landmark_indices)
              current_cell_landmarks{good_cell_landmark_indices(good_frame_index)} = reshape(current_cell_landmarks_smoothed(good_frame_index, :), [3, 2])';
            end
            
            if any(~isfinite(current_cell_landmarks_smoothed(:)))
              current_cell_landmarks_unsmoothed_concatenated = cell2mat(current_cell_landmarks_unsmoothed), current_cell_landmarks_concatenated = cell2mat(current_cell_landmarks)
              % Just ignore cells where lowess fails and produces nans:
              
              % pause
              % beep
              % keyboard
              
              if debug_print_alignment_skip_reasons, fprintf_diary_cycle_function('Skipping alignment %s current_cell_index %d frame %d, because at least one non-finite landmark after smoothing!\n', run_key, current_cell_index, frame_index), end
              % if debug_keyboard_alignment_skips
                % current_cell_landmarks_smoothed
                % dbstack, beep, keyboard
              % end
              continue
            end
            
          end
          
          % beep, keyboard
          
          % Align each frame individually:
          for frame_index = 1:number_frames
            current_frame_image_filename = run_image_path{frame_index};
            if isempty(current_frame_image_filename)
              continue
            end
            current_synapse_center_rounded = cell_synapse_point_tracks{current_cell_index, frame_index};
            if isempty(current_synapse_center_rounded)
              continue
            end
            current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
            
            % This takes a long time, so do not repeat work by checking for a file indicating this synapse file's windows are finished:
            cell_alignment_filename = [run_key, sprintf('_frame%05d_synapse%05d,%05d', frame_index, current_synapse_center_rounded)];
            cell_segmentation_filename = [segmentations_filtered_location, cell_alignment_filename];
            cell_region_filename = [regions_location, cell_alignment_filename];
            
            % xruan 02/24/2016
            % [can_start, final_name, final_exists] = chunk_start(alignments_location, cell_alignment_filename);
            [can_start, final_name, final_exists] = chunk_start_clean(alignments_location, cell_alignment_filename, [], true);

            current_landmarks = current_cell_landmarks{frame_index};
            if isempty(current_landmarks)
              temp = struct; save(final_name, '-struct', 'temp'); chunk_finish(alignments_location, cell_alignment_filename);
              continue
            end
            
            % if final_exists && ~false
              % % Debug plot:
              % a = load(final_name);
              % cropped_raw_image = a.cropped_raw_image;
              % cropped_segmentation_image = a.cropped_segmentation_image;
              % cropped_raw_image = a.cropped_raw_image;
              
              % image_to_show = contrast_stretch(cropped_raw_image);
              % image_to_show = [image_to_show; cropped_segmentation_image];
              % image_to_show = [image_to_show; contrast_stretch(cropped_raw_image .* (cropped_segmentation_image >= .5) + min(cropped_raw_image(cropped_segmentation_image >= .5)) .* (cropped_segmentation_image < .5))];
              % image_to_show = reshape_2d(image_to_show, -1);
              % imshow(image_to_show), pause
            % end
 
            % xruan 02/21/2016 If the aligned file exist, then just skip it
            % rather than save an empty file.
            if final_exists
                fprintf_diary_cycle_function('Alignment exists and skip it: "%s"\n', cell_alignment_filename);
                continue;
            end      
            
            % if ~can_start
            if debug_keyboard_alignment_skips
              [~, final_name_filename, final_name_extension] = fileparts(final_name);
              % is_final_name_in_distinct = ismember(final_name, distinct);
              is_final_name_in_distinct = ismember([final_name_filename, final_name_extension], distinct);
              if false && debug_keyboard_alignment_skips
                fprintf_diary_cycle_function('is_final_name_in_distinct = %d\n', is_final_name_in_distinct)
                keyboard
              end
            end
            if debug_keyboard_alignment_skips && is_final_name_in_distinct
              fprintf_diary_cycle_function('debug_keyboard_alignment_skips = true, so repeating ''%s''\n', final_name)
            else
              if ~can_start && ~debug_generate_figures
              % if false
                temp = struct; save(final_name, '-struct', 'temp'); chunk_finish(alignments_location, cell_alignment_filename);
                continue
              end
              
              did_any_work = true;
              if ~printed_run_work_message
                fprintf_diary_cycle_function('Aligning segmentations for run "%s"\n', run_key)
                printed_run_work_message = true;
              end
            end
            
            a = load([cell_segmentation_filename, '.mat']);
            segmentation_structure = a.segmentation_structure;
            window_number_slices = a.window_number_slices;

            current_segmentation_image = segmentation_rasterization_function(segmentation_structure, [window_size_2d, window_number_slices]);
            % current_segmentation_image = segmentation_rasterization_function(segmentation_structure);
            % current_landmarks = current_cell_landmarks{frame_index};

            % Compute absolute alignments between a frame and the template:
            try
              [~, ~, current_transform] = procrustes(template_landmarks, current_landmarks, 'Reflection', false, 'Scaling', false);
            catch raised_error
              getReport(raised_error, 'extended')
              % keyboard
              % rethrow(raised_error)
              temp = struct; save(final_name, '-struct', 'temp'); chunk_finish(alignments_location, cell_alignment_filename);
              continue
            end
            current_transform_homogeneous = [diag(current_transform.b) * current_transform.T', current_transform.c(1, :)'; 0, 0, 0, 1];
            
            % Scale according to cell volume:
            current_volume = sum(current_segmentation_image(:));
            % Scale by volume about the synapse point:
            % current_transform_homogeneous = eye(4)
            % current_transform_homogeneous = translation_matrix3h(-current_landmarks(1, :)) * current_transform_homogeneous;
            % current_transform_homogeneous = translation_matrix3h(-current_landmarks(3, :)) * current_transform_homogeneous;
            current_transform_homogeneous = translation_matrix3h(-current_transform.c(1, :)') * current_transform_homogeneous;
            current_transform_homogeneous = scale_matrix3h(ones(1, 3) * nthroot(template_volume / current_volume, 3)) * current_transform_homogeneous;
            % current_transform_homogeneous = translation_matrix3h(current_landmarks(1, :)) * current_transform_homogeneous;
            % current_transform_homogeneous = translation_matrix3h(current_landmarks(3, :)) * current_transform_homogeneous;
            current_transform_homogeneous = translation_matrix3h(current_transform.c(1, :)') * current_transform_homogeneous;
            % current_transform_homogeneous = translation_matrix3h(current_landmarks(1, :)) * scale_matrix3h(ones(1, 3) * (template_volume / current_volume)) * translation_matrix3h(-current_landmarks(1, :)) * current_transform_homogeneous;

            % Produce segmentation and raw images after transformation:
            
           
            current_transform_homogeneous_function = @(lambda, x_shift)[diag(current_transform.b) * current_transform.T' * lambda, ...
                current_transform.c(1, :)' + (1 - lambda) * diag(current_transform.b) * current_transform.T' * x_shift; ...
                0, 0, 0, 1];
            current_transform_homogeneous_old_function =  @(lambda, x_shift)[diag(current_transform.b) * current_transform.T' * lambda, ...
                lambda * current_transform.c(1, :)' + (1 - lambda) * x_shift; ...
                0, 0, 0, 1];
            
            current_volume = sum(current_segmentation_image(:));
            lambda = nthroot(template_volume / current_volume, 3);
            transformation_method_set = {'original', 'rescale_origin', 'rescale_centroid', 'rescale_left_end', 'rescale_right_end',...
                'old_left_end', 'old_right_end', 'old_centroid'};
            % transformation_method = 'original';
            % 02/22/2016 xruan
            % just use the method default to keep it consistent with the
            % running for the paper. 
            if master_script_options.use_two_point_synapses
                transformation_method = transformation_method_set{3};
            else
                % for one point synapse, still use centroid shift, but it
                % is current_landmarks(2, :);
                transformation_method = transformation_method_set{4};
            end
            
            switch transformation_method
                case 'original'
                    current_transform_homogeneous = current_transform_homogeneous_function(1, [0, 0, 0]');
                case 'rescale_origin'
                    current_transform_homogeneous = current_transform_homogeneous_function(lambda, [0, 0, 0]');
                case 'rescale_centroid'
                    current_transform_homogeneous = current_transform_homogeneous_function(lambda, current_landmarks(3, :)');
                case 'rescale_left_end'
                    current_transform_homogeneous = current_transform_homogeneous_function(lambda, current_landmarks(1, :)');
                case 'rescale_right_end'
                    current_transform_homogeneous = current_transform_homogeneous_function(lambda, current_landmarks(2, :)');
                case 'old_left_end'
                    current_transform_homogeneous = current_transform_homogeneous_old_function(lambda, current_landmarks(1, :)');
                case 'old_right_end'
                    current_transform_homogeneous = current_transform_homogeneous_old_function(lambda, current_landmarks(2, :)');
                case 'old_centroid'
                    current_transform_homogeneous = current_transform_homogeneous_old_function(lambda, current_landmarks(3, :)');
            end
             
                    
            a = load([cell_region_filename, '.mat']);
            current_window_image = a.current_window_image;
            
            % Transform current_window_image:
            % xruan 06/30/2015
            % first pad the image to allow the transformed image to
            % maintain the signal
            % xruan 07/21/2015 
            % check if it is need to pad by compare the actual
            % window_number_slices with template slice numbers. if the
            % actual window number slices is larger, then pad 0. 
            padval = median(current_window_image(:));
            current_window_image_padded = padarray(current_window_image, [0, 0, max([0, cropped_size(3) - window_number_slices])], padval, 'post');
            current_window_image_transformed = affine(current_window_image_padded, current_transform_homogeneous([2, 1, 3, 4], [2, 1, 3, 4]), [], false, [], [], 'crop');
            % Transform segmentation_structure:
            segmentation_structure_transformed = segmentation_structure;
            segmentation_structure_transformed.vertices = current_transform_homogeneous * [segmentation_structure_transformed.vertices, ones(size(segmentation_structure_transformed.vertices, 1), 1)]';
            segmentation_structure_transformed.vertices = segmentation_structure_transformed.vertices(1:3, :)';

            % Crop the final aligned segmentation image:
            % cropped_segmentation_image_without_transformation = segmentation_rasterization_function(segmentation_structure, [window_size_2d, window_number_slices]);
            
            % xruan 06/30/2015
            %cropped_segmentation_image = segmentation_rasterization_function(segmentation_structure_transformed, [window_size_2d, window_number_slices]);
            cropped_segmentation_image = segmentation_rasterization_function(segmentation_structure_transformed, [window_size_2d, size(template_image, 3)]);
            cropped_segmentation_image_without_transformation = segmentation_rasterization_function(segmentation_structure, [window_size_2d,  size(template_image, 3)]);            % cropped_segmentation_image = segmentation_rasterization_function(segmentation_structure_transformed);
            
            cropped_segmentation_image_volume = sum(cropped_segmentation_image(:));
            
            % current_transform
            % fprintf_diary_cycle_function('current_volume = %f, template_volume = %f, current_volume = %f\n', current_volume, template_volume, cropped_segmentation_image_volume)
            % imshow(reshape_2d(cropped_segmentation_image, -1)), pause

            if false
              % Debug plot:
              imshow(reshape_2d([cropped_segmentation_image_without_transformation; cropped_segmentation_image], 1)), pause
            end
            
            cropped_segmentation_image = padarray(cropped_segmentation_image, max(cropped_size - size(cropped_segmentation_image), 0), 'post');
            cropped_segmentation_image = cropped_segmentation_image(1:cropped_size(1), 1:cropped_size(2), 1:cropped_size(3));
            
            % Set the edges to zero so the registration does not take forever:
            edge_width = 2;
            cropped_segmentation_image(1:edge_width, :, :) = 0;
            cropped_segmentation_image(end - edge_width + 1:end, :, :) = 0;
            cropped_segmentation_image(:, 1:edge_width, :) = 0;
            cropped_segmentation_image(:, end - edge_width + 1:end, :) = 0;
            cropped_segmentation_image(:, :, 1:edge_width) = 0;
            cropped_segmentation_image(:, :, end - edge_width + 1:end) = 0;

            % cropped_raw_image = current_window_image;
            cropped_raw_image = current_window_image_transformed;
            cropped_raw_image = padarray(cropped_raw_image, max(cropped_size - size(cropped_raw_image), 0), 'post');
            cropped_raw_image = cropped_raw_image(1:cropped_size(1), 1:cropped_size(2), 1:cropped_size(3));
            
            % if debug_keyboard_alignment_skips && current_volume ~= cropped_segmentation_image_volume
              % dbstack, beep, keyboard
            % end
            
            is_cropped_segmentation_low_volume = cropped_segmentation_image_volume <= segmentation_volume_threshold;
            if is_cropped_segmentation_low_volume
              if debug_print_alignment_skip_reasons, fprintf_diary_cycle_function('Skipping alignment %s current_synapse_center_rounded %d,%d frame %d, because segmentation has low volume!\n', run_key, current_synapse_center_rounded, frame_index), end
              temp = struct; save(final_name, '-struct', 'temp'); chunk_finish(alignments_location, cell_alignment_filename);
              continue
            end
            
            if debug_generate_figures
              % Debug plot and image saving:
              chunk_finish(alignments_location, cell_alignment_filename);
              
              % if master_script_options.use_landmark_temporal_smoothing
              if ~strcmp(master_script_options.landmark_temporal_smoothing_method, 'none')
                % Smooth landmark points across time:
                current_cell_landmarks_unsmoothed = current_cell_landmarks;
              end

              % imshow(reshape_2d([cropped_segmentation_image_without_transformation; cropped_segmentation_image], -1)), pause
              % imshow(reshape_2d(cropped_segmentation_image, -1)), pause
              
              im_size = size(template_image);

              current_landmark_image = visulaize_landmarks(current_landmarks, im_size);

              transformed_landmarks = current_transform_homogeneous * [current_landmarks, ones(3, 1)]';
              transformed_landmarks = transformed_landmarks(1 : 3, :)';

              transformed_landmark_image = visulaize_landmarks(transformed_landmarks, im_size);

              template_landmark_image = visulaize_landmarks(template_landmarks, im_size);
              
              [csy, csx, csz] = ind2sub(im_size, find(cropped_segmentation_image == 1));
              
              segmentation_transformed_centroid = [mean(csx), mean(csy), mean(csz)];
              
              transformed_segmented_centroid_image = visulaize_landmarks( [zeros(2, 3); segmentation_transformed_centroid], im_size);
              

%               image_to_show = zeros(0, size(cropped_raw_image, 2), size(cropped_raw_image, 3));
%               image_to_show = contrast_stretch(cropped_raw_image);
%               image_to_show = [image_to_show; cropped_segmentation_image];
%               image_to_show = [image_to_show; contrast_stretch(cropped_raw_image .* (cropped_segmentation_image >= .5) + min(cropped_raw_image(cropped_segmentation_image >= .5)) .* (cropped_segmentation_image < .5))];
%               % image_to_show = [image_to_show; template_image];
%               marker_image = double(template_image);
%               marker_image(:, :, 18) = marker_image(:, :, 18) * 2;
%               image_to_show = [image_to_show; contrast_stretch(marker_image)];
% 
%               image_to_show = reshape_2d(image_to_show);
%               % image_to_show = reshape_2d(image_to_show, -1);
%               % image_to_show = reshape_2d(image_to_show, 3);

              image_to_show = zeros(0, size(cropped_raw_image, 2), size(cropped_raw_image, 3));
              image_to_show_marker = zeros(0, size(cropped_raw_image, 2), size(cropped_raw_image, 3));
              image_to_show_blue = zeros(0, size(cropped_raw_image, 2), size(cropped_raw_image, 3));
              
              image_to_show = [image_to_show; contrast_stretch(current_window_image_padded)];
              image_to_show_marker = [image_to_show_marker; current_landmark_image];
              image_to_show_blue = [image_to_show_blue; zeros(im_size)];
         
              image_to_show = [image_to_show; cropped_segmentation_image_without_transformation];
              image_to_show_marker = [image_to_show_marker; current_landmark_image];
              image_to_show_blue = [image_to_show_blue; zeros(im_size)];

              image_to_show = [image_to_show; contrast_stretch(cropped_raw_image)];
              image_to_show_marker = [image_to_show_marker; transformed_landmark_image];
              image_to_show_blue = [image_to_show_blue; transformed_segmented_centroid_image];

              image_to_show = [image_to_show; cropped_segmentation_image];
              image_to_show_marker = [image_to_show_marker; transformed_landmark_image];
              image_to_show_blue = [image_to_show_blue; transformed_segmented_centroid_image];

              image_to_show = [image_to_show; contrast_stretch(cropped_raw_image .* (cropped_segmentation_image >= .5) + min(cropped_raw_image(cropped_segmentation_image >= .5)) .* (cropped_segmentation_image < .5))];
              image_to_show_marker = [image_to_show_marker; transformed_landmark_image];  
              image_to_show_blue = [image_to_show_blue; transformed_segmented_centroid_image];
              
              add_deformation_images = true;
              if add_deformation_images
                cell_deformation_filename = [deformations_location, cell_alignment_filename];
                a = {};
                if exist([cell_deformation_filename, '.mat'], 'file')                  
                    a = load([cell_deformation_filename, '.mat']);
                end
                a
                if isfield(a, 'cropped_raw_image_deformed') && isfield(a, 'cropped_segmentation_image_deformed')
                    cropped_raw_image_deformed = a.cropped_raw_image_deformed;
                    cropped_segmentation_image_deformed = a.cropped_segmentation_image_deformed;

                    image_to_show = [image_to_show; contrast_stretch(cropped_raw_image_deformed .* (cropped_segmentation_image_deformed >= .5) + min(cropped_raw_image_deformed(cropped_segmentation_image_deformed >= .5)) .* (cropped_segmentation_image_deformed < .5))];
                    image_to_show_marker = [image_to_show_marker; zeros(im_size)];  
                    image_to_show_blue = [image_to_show_blue; zeros(im_size)];
                else
                    image_to_show = [image_to_show; zeros(im_size)];
                    image_to_show_marker = [image_to_show_marker; zeros(im_size)];  
                    image_to_show_blue = [image_to_show_blue; zeros(im_size)];
                end
              end
              
              
              % image_to_show = [image_to_show; template_image];
              marker_image = double(template_image);
              % marker_image(:, :, 18) = marker_image(:, :, 18) * 2;
              image_to_show = [image_to_show; contrast_stretch(marker_image)];

              image_to_show_marker = [image_to_show_marker; template_landmark_image];
              image_to_show_blue = [image_to_show_blue; zeros(im_size)];

              image_to_show = reshape_2d(image_to_show);

              image_to_show_marker = reshape_2d(image_to_show_marker);
              
              image_to_show_blue = reshape_2d(image_to_show_blue);
              
              image_to_show_final = zeros([size(image_to_show), 3]);
              image_to_show_final(:, :, 1) = image_to_show;
              image_to_show_final(:, :, 2) = image_to_show_marker;
              image_to_show_final(:, :, 3) = image_to_show_blue;
              
              % imshow(image_to_show), pause
              
              image_filename = [alignment_illustrations_location, cell_alignment_filename];
              % if master_script_options.use_landmark_temporal_smoothing
                % image_filename = [image_filename, '_with_smoothing'];
              % end
              imwrite(image_to_show_final, [image_filename, '.png'])

              warning('Debugging, skipping alignment saving!')
            else
              % At some point save current_cell_volumes:
              save(final_name, 'current_landmarks', 'current_transform', 'current_transform_homogeneous', 'cropped_segmentation_image', 'cropped_raw_image')
              chunk_finish(alignments_location, cell_alignment_filename);
            end        
          
          end
          
          temp = struct; save(finished_final_name, '-struct', 'temp');
          chunk_finish(alignments_location, current_cell_alignment_finished_filename)
          run_cell_indices_uncomputed{run_index}(current_cell_index) = false;
        end
      end
      
      pause(1)
      % xruan 02/24/2016
      out_status = chunk_lock_clean(alignments_location);
      if out_status
          fprintf_diary_cycle_function('Delete lock files whose jobs are not running!\n')
      end       
      if recently_did_any_work && ~did_any_work
        fprintf_diary_cycle_function('Apparently no more work to do, wait for other jobs')
      end
      
    end
    
  end

  
  
  % beep, keyboard
  % error('Implementation yet unfinished below this line!')
  
end

