function [t_cell_info] = master_script_filter_segmentations(t_cell_info, options)
  % Train a classifier to distinguish between good and bad segmentations per frame and then filter all segmentations using it:
  %
  % 2013-11-15 tebuck: Copied from master_script_segment_windows.m.
  % 
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  % default_options = struct();
  % default_options.
  
  % if ~exist('options', 'var')
    % options = default_options;
  % else
    % options = process_options_structure(default_options, options);
  % end
  
  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  temp_location = t_cell_info.path_info.temp_location;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_location = t_cell_info.path_info.segmentations_location;
  use_segmentation_filtering = master_script_options.use_segmentation_filtering;
  if ~use_segmentation_filtering
    return
  end
  segmentation_evaluations_location = t_cell_info.path_info.segmentation_evaluations_location;
  segmentations_filtered_location = t_cell_info.path_info.segmentations_filtered_location;
  window_preprocessing_function = t_cell_info.segmentation_info.window_preprocessing_function;
  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  % window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  % template_cell_radius = t_cell_info.template_info.template_cell_radius;
  running_interactively = t_cell_info.plotting_info.running_interactively;
  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;



  function [segmentation_features] = compute_mesh_segmentation_evaluation_features(given_image, given_segmentation)
    % Compute features to distinguish good and bad segmentations as labeled in the evaluation CSV file:
    segmentation_image = segmentation_rasterization_function(given_segmentation, size(given_image));
    % segmentation_image = segmentation_rasterization_function(given_segmentation);
    
    show_debug_plot = false;
    % show_debug_plot = true;
    if show_debug_plot
      given_image_contrast_stretched = contrast_stretch(given_image);
      % imshow(reshape_2d([given_image_contrast_stretched; given_image_contrast_stretched .* ~segmentation_image], 4));
      imshow(reshape_2d([given_image_contrast_stretched; given_image_contrast_stretched .* (1 - segmentation_image)], 4));
      pause
    end
    
    % Preparation for feature computation:
    vertex_intensities = interp3(given_image, given_segmentation.vertices(:, 1), given_segmentation.vertices(:, 2), given_segmentation.vertices(:, 3));

    curvature_smoothing = 2;
    [minimum_curvature_direction, maximum_curvature_direction, minimum_curvature, maximum_curvature, mean_curvature, gaussian_curvature, normal] = compute_curvature(given_segmentation.vertices, given_segmentation.faces, struct('curvature_smoothing', curvature_smoothing, 'verb', false));
    % [minimum_curvature_direction, maximum_curvature_direction, minimum_curvature, maximum_curvature, mean_curvature, gaussian_curvature, normal] = compute_curvature(given_segmentation.vertices', given_segmentation.faces', struct('curvature_smoothing', curvature_smoothing, 'verb', false));
    minimum_curvature_direction = minimum_curvature_direction';
    maximum_curvature_direction = maximum_curvature_direction';
    normal = normal';
    
    neighbor_distances = build_euclidean_weight_matrix(triangulation2adjacency(given_segmentation.faces), given_segmentation.vertices, 0);
    neighbor_distances = neighbor_distances(find(neighbor_distances));

    [gradient_x, gradient_y, gradient_z] = smooth_gradient(given_image, 'scharr5', false, true);
    gradient_magnitude = sqrt(gradient_x.^2 + gradient_y.^2 + gradient_z.^2);
    vertex_gradient_magnitudes = interp3(gradient_magnitude, given_segmentation.vertices(:, 1), given_segmentation.vertices(:, 2), given_segmentation.vertices(:, 3));
    vertex_gradients = [interp3(gradient_x, given_segmentation.vertices(:, 1), given_segmentation.vertices(:, 2), given_segmentation.vertices(:, 3)), interp3(gradient_y, given_segmentation.vertices(:, 1), given_segmentation.vertices(:, 2), given_segmentation.vertices(:, 3)), interp3(gradient_z, given_segmentation.vertices(:, 1), given_segmentation.vertices(:, 2), given_segmentation.vertices(:, 3))];
    vertex_gradients(vertex_gradient_magnitudes > 0, :) = vertex_gradients(vertex_gradient_magnitudes > 0, :) ./ repmat(vertex_gradient_magnitudes(vertex_gradient_magnitudes > 0, :), 1, 3);
    
    gradient_scalar_products = sum(normal .* vertex_gradients, 2);
    
    [x, y, z] = meshgrid(1:size(given_image, 2), 1:size(given_image, 1), 1:size(given_image, 3));
    segmentation_convex_hull_image = reshape(inhull([x(:), y(:), z(:)], given_segmentation.vertices), size(given_image)); 
    
    % interesting_percentiles = 0:25:100;
    interesting_percentiles = 0:10:100;
    
    % Feature computation:
    
    % Intensity distribution at vertices:
    segmentation_features{1} = prctile(vertex_intensities, interesting_percentiles);
    % Curvature distribution at vertices:
    segmentation_features{2} = [prctile(minimum_curvature, interesting_percentiles), prctile(maximum_curvature, interesting_percentiles)];
    % Length distribution of edges:
    segmentation_features{3} = prctile(neighbor_distances, interesting_percentiles);
    % Gradient magnitude distribution at vertices:
    segmentation_features{4} = prctile(vertex_gradient_magnitudes, interesting_percentiles);
    % Scalar product of image gradient direction and mesh normal distribution at vertices:
    segmentation_features{5} = prctile(gradient_scalar_products, interesting_percentiles);
    % Segmentation image features:
    volume = sum(segmentation_image(:));
    solidity = volume / sum(segmentation_convex_hull_image(:));
    % segmentation_features{6} = [volume, solidity];
    % Do not use volume because some images are higher magnification than others:
    segmentation_features{6} = [solidity];
    % % Spherical harmonics of vertex components:
    % % :
    % % :
    % keyboard
    
    segmentation_features = cell2mat(reshape(segmentation_features, 1, []));
  end


  function [result_image] = get_evaluation_image(given_window, given_segmentation)
    % Produce visualization of image and segmentation:
    % given_window_contrast_stretched = contrast_stretch(given_window);
    [preprocessing_structure] = window_preprocessing_function(given_window);
    % given_window_contrast_stretched = contrast_stretch(preprocessing_structure.image_to_segment);
    given_window_contrast_stretched = contrast_stretch(preprocessing_structure.coherent_image_to_segment);
    segmentation_image = segmentation_rasterization_function(given_segmentation, size(given_window));
    % segmentation_image = segmentation_rasterization_function(given_segmentation);
    % slice_spacing = 1;
    slice_spacing = 2;
    % slice_spacing = 3;
    % slice_spacing = 4;
    % imagesc(reshape_2d([given_window_contrast_stretched(:, :, slice_spacing:slice_spacing:end); given_window_contrast_stretched(:, :, slice_spacing:slice_spacing:end) .* ~segmentation_image(:, :, slice_spacing:slice_spacing:end)], -1));
    result_image = reshape_2d([given_window_contrast_stretched(:, :, slice_spacing:slice_spacing:end); given_window_contrast_stretched(:, :, slice_spacing:slice_spacing:end) .* (1 - segmentation_image(:, :, slice_spacing:slice_spacing:end))], -1);
  end
  
  
  
  function [choice] = get_choice(given_window, given_segmentation, given_classwise_accuracy)
    % Present segmentation to user to get evaluation (have yes, no, no decision, and have a quit/done buttons and the current class-wise accuracy):
    good_label = 'Good';
    bad_label = 'Bad';
    indecisive_label = 'No decision';
    stop_label = 'Stop training';
    
    choice = nan;
    
    function [] = evaluation_callback(handle_object, event_data)
      % Callback function:
      switch(get(handle_object, 'string'))
        case good_label;
          choice = 1;
        case bad_label;
          choice = -1;
        case indecisive_label;
          choice = 0;
        case stop_label;
          choice = nan;
        otherwise
          choice = nan;
      end
      close
    end

    close all
    figure_handle = figure('Position', [1, 1, 4, 3] * 200);
    panel_handle = uipanel('Title', sprintf('Current weighted accuracy: %f', current_weighted_accuracy), 'Units', 'Normalized');
    
    subplot(3, 4, 1:8, 'Parent', panel_handle)
    % Plot segmentation:
    % given_window_contrast_stretched = contrast_stretch(given_window);
    [preprocessing_structure] = window_preprocessing_function(given_window);
    % given_window_contrast_stretched = contrast_stretch(preprocessing_structure.image_to_segment);
    given_window_contrast_stretched = contrast_stretch(preprocessing_structure.coherent_image_to_segment);
    segmentation_image = segmentation_rasterization_function(given_segmentation, size(given_window));
    % segmentation_image = segmentation_rasterization_function(given_segmentation);
    imagesc(get_evaluation_image(given_window, given_segmentation));
    axis image
    % set(gca, 'ActivePositionProperty', 'Position', 'Position', [0, 0, 1, 1])
    set(gca, 'ActivePositionProperty', 'Position', 'Position', [0, 0.25, 1, .75])
    % set(gca, 'Position', get(gca, 'OuterPosition'))
    % set(gca, 'ActivePositionProperty', 'Position')
    % colormap(pmkmp(1024, 'CubicL'))
    % colormap(min(max(pmkmp(1024, 'Edge'), 0), 1))
    colormap(illustration_colormap_to_use)
    
    % Make buttons:
    good_handle = uicontrol(panel_handle, 'style', 'pushbutton', 'string', good_label, 'Units', 'Normalized', 'Position', [0, 0, .25, .25]);
    bad_handle = uicontrol(panel_handle, 'style', 'pushbutton', 'string', bad_label, 'Units', 'Normalized', 'Position', [.25, 0, .25, .25]);
    indecisive_handle = uicontrol(panel_handle, 'style', 'pushbutton', 'string', indecisive_label, 'Units', 'Normalized', 'Position', [.5, 0, .25, .25]);
    stop_handle = uicontrol(panel_handle, 'style', 'pushbutton', 'string', stop_label, 'Units', 'Normalized', 'Position', [.75, 0, .25, .25]);

    % Set button callback:
    set(good_handle, 'Callback', @evaluation_callback)
    set(bad_handle, 'Callback', @evaluation_callback)
    set(indecisive_handle, 'Callback', @evaluation_callback)
    set(stop_handle, 'Callback', @evaluation_callback)
    
    % Make onClose or w/e, or just let it error out?
    
    % Wait until the user clicks something:
    waitfor(figure_handle)
  end
  
  
  
  % error('Implementation yet unfinished below this line!')
  % Process windows directly:
  % segmentation_list = dir([segmentations_location, '*_frame*_*synapse*.mat']);
  % segmentation_list = strcat(segmentations_location, {segmentation_list.name}.');
  segmentation_list = master_script_list_relevant_results(t_cell_info, struct('given_directory', segmentations_location));
  
  % whos segmentation_list

  % error('Implementation yet unfinished below this line!')

  
  
  if master_script_options.segmentation_filtering_interactive
    % Checking if work is being done elsewhere or is done:
    % segmentation_segmentation_filename = strrep(segmentation_filename, '.mat', '_segmentation.mat');
    % segmentation_classifier_filename = [options.results_location, 'segmentation_classifier'];
    segmentation_classifier_filename = ['segmentation_classifier'];
    [can_start, final_name, final_exists] = chunk_start(segmentation_evaluations_location, segmentation_classifier_filename);
    % warning('>>>> HACK erasing temp file while checking code!'), chunk_finish(segmentation_evaluations_location, segmentation_classifier_filename);
    
    if ~can_start && ~final_exists
      % Produce an error while waiting for someone to interactively train a classifier:
      error('Classifier does not exist but lock file remains, rerun after finished training classifier')
    end
    
    if can_start
  
      if ~running_interactively
        chunk_finish(segmentation_evaluations_location, segmentation_classifier_filename);
        error('The segmentation filtering training step must be run interactively!')
      end
      
      number_segmentations = length(segmentation_list);
      % Decisions are 1 (good), -1 (bad), 0 (user made no decision), nan (not yet evaluated):
      segmentation_decisions = nan(number_segmentations, 1);
      segmentation_features = cell(number_segmentations, 1);

      % Load existing decisions:
      % evaluation_list = dir([segmentation_evaluations_location, '*_frame*_*synapse*.mat']);
      evaluation_list = master_script_list_relevant_results(t_cell_info, struct('given_directory', segmentation_evaluations_location));
      % {evaluation_list.name}.';
      % keyboard
      % if length(evaluation_list) > 0
        % evaluation_list = strcat(segmentation_evaluations_location, {evaluation_list.name}.');
      % end
      for evaluation_index = 1:length(evaluation_list)
        evaluation_filename = evaluation_list{evaluation_index};
        a = load(evaluation_filename);
        % current_region_filename = a.region_filename
        % decision_index = ismember(strrep(evaluation_filename, segmentation_evaluations_location, segmentations_location), segmentation_list);
        decision_index = find(ismember(segmentation_list, strrep(evaluation_filename, segmentation_evaluations_location, segmentations_location)));
        % decision_index, keyboard
        segmentation_decisions(decision_index) = a.current_choice;

        % Compute features:
        % a = load(strrep(evaluation_filename, segmentation_evaluations_location, regions_location));
        % b = load(strrep(evaluation_filename, segmentation_evaluations_location, segmentations_location));
        % segmentation_features{decision_index} = compute_mesh_segmentation_evaluation_features(a.current_window_image, b.segmentation_structure);
        segmentation_features{decision_index} = a.current_segmentation_features;
      end

      training_running = true;
      while training_running
        anything_changed = false;
        for segmentation_index = randperm(length(segmentation_list))
          if ~isnan(segmentation_decisions(segmentation_index))
            continue
          end

          segmentation_filename = segmentation_list{segmentation_index};
          region_filename = strrep(segmentation_filename, segmentations_location, regions_location);
          evaluation_filename = strrep(segmentation_filename, segmentations_location, segmentation_evaluations_location);
          
          fprintf('Evaluating segmentation: %s\n', segmentation_filename)
          
          % Step 1: Compute features:
          a = load(strrep(evaluation_filename, segmentation_evaluations_location, regions_location));
          b = load(strrep(evaluation_filename, segmentation_evaluations_location, segmentations_location));
          current_window_image = a.current_window_image;
          current_segmentation_structure = b.segmentation_structure;

          current_segmentation_features = compute_mesh_segmentation_evaluation_features(current_window_image, current_segmentation_structure);
          
          % Step 2: Present segmentation to user to get evaluation (have yes, no, no decision, and have a quit/done buttons and the current class-wise accuracy):
          current_weighted_accuracy = nan;
          good_decisions = (segmentation_decisions == 1);
          bad_decisions = (segmentation_decisions == -1);
          usable_decisions = good_decisions | bad_decisions;
          number_good_decisions = sum(good_decisions);
          number_bad_decisions = sum(bad_decisions);
          fprintf('number_good_decisions = %d, number_bad_decisions = %d\n', number_good_decisions, number_bad_decisions)
          number_splits = 5;
          % number_splits = 10;
          if number_good_decisions >= number_splits && number_bad_decisions >= number_splits
            evaluated_segmentation_features = cell2mat(segmentation_features(usable_decisions, :));
            % Here 1 means good, 2 means bad:
            evaluated_labels = (segmentation_decisions(usable_decisions, :) - 3) / 2 * -1;
            % mkdir('temp')
            % svm_info = train_svm_with_cross_validation(evaluated_segmentation_features, evaluated_labels, [], ['temp/', num2str(now)], 'none', true);
            % svm_info = train_svm_with_cross_validation(evaluated_segmentation_features, evaluated_labels, number_splits, [temp_location, 'intermediate_svm', '_', num2str(now), '_', num2str(rand())], 'none', true);
            svm_info = train_svm_with_cross_validation(evaluated_segmentation_features, evaluated_labels, number_splits, [temp_location, 'intermediate_svm', '_', datestr(now(), 'yyyymmddHHMMSSFFF'), '_'], 'none', true);
            current_weighted_accuracy = svm_info.weighted_accuracy;
            fprintf('current_weighted_accuracy = %.3f\n', current_weighted_accuracy)
          else
            fprintf('Need %d decisions for each class to compute accuracy!\n', number_splits)
          end
          current_choice = get_choice(current_window_image, current_segmentation_structure, current_weighted_accuracy);
          
          % Step 3: Save decision, recompute classifier
          if isnan(current_choice)
            training_running = false;
            
            save(final_name, 'svm_info')
            chunk_finish(segmentation_evaluations_location, segmentation_classifier_filename);

            break
          end
          
          anything_changed = true;
          save(evaluation_filename, 'current_choice', 'current_segmentation_features', 'region_filename')

          % Store choice in memory:
          segmentation_decisions(segmentation_index) = current_choice;
          % Store features:
          segmentation_features{segmentation_index} = current_segmentation_features;
        end
        
        if ~anything_changed
          training_running = false;
        end
      end
      
      
    end
  else
      
      
    % Write segmentation evaluation files and wait for the user to sort some of them:
    
    mkdir([segmentation_evaluations_location, 'good/'])
    mkdir([segmentation_evaluations_location, 'bad/'])
    
    should_compute_features = false;
    % should_compute_features = true;

    for segmentation_index = randperm(length(segmentation_list))

      segmentation_filename = segmentation_list{segmentation_index};
      region_filename = strrep(segmentation_filename, segmentations_location, regions_location);
      evaluation_filename = strrep(segmentation_filename, segmentations_location, segmentation_evaluations_location);
      evaluation_image_filename = strrep(evaluation_filename, '.mat', '.png');
      [~, evaluation_filename_name_without_extension, evaluation_filename_extension] = fileparts(evaluation_filename);
      
      if should_compute_features
        [can_start, final_name, final_exists] = chunk_start(segmentation_evaluations_location, evaluation_filename_name_without_extension, evaluation_filename_extension);
      else
        [can_start, final_name, final_exists] = chunk_start(segmentation_evaluations_location, evaluation_filename_name_without_extension, '.png');
      end

      if ~can_start
        continue
      end

      
      % fprintf('Evaluating segmentation: %s\n', segmentation_filename)
      
      % Step 1: Compute features:
      
      a = load(region_filename);
      b = load(segmentation_filename);
      current_window_image = a.current_window_image;
      current_segmentation_structure = b.segmentation_structure;

      if should_compute_features
        current_segmentation_features = compute_mesh_segmentation_evaluation_features(current_window_image, current_segmentation_structure);
      end
      
      % Step 2: Present segmentation to user to get evaluation (have yes, no, no decision, and have a quit/done buttons and the current class-wise accuracy):
      
      current_evaluation_image = t_cell_info.plotting_info.grayscale_color_mapping_function(get_evaluation_image(current_window_image, current_segmentation_structure));
      
      % Step 3: Save decision, recompute classifier
      
      if should_compute_features
        save(final_name, 'current_segmentation_features', 'region_filename')
      end
      imwrite(current_evaluation_image, strrep(final_name, '.mat', '.png'))

      chunk_finish(segmentation_evaluations_location, evaluation_filename_name_without_extension);
      
    end

    error('Not yet implemented!')
    

    % Always rebuild the classifier?
  
  
  end

  
  
  svm_info = [];
  load(final_name, 'svm_info')
  % svm_info
  
  % t_cell_info.path_info, error
  
  % Filter using the trained classifier:
  for segmentation_index = 1:length(segmentation_list)
  
    segmentation_filename = segmentation_list{segmentation_index};
    region_filename = strrep(segmentation_filename, segmentations_location, regions_location);
    % segmentation_filtered_filename = strrep(segmentation_filename, segmentations_location, segmentations_filtered_location);
    
    [~, segmentation_filename_name_without_extension, segmentation_filename_extension] = fileparts(segmentation_filename);
    segmentation_filename_name = [segmentation_filename_name_without_extension, segmentation_filename_extension];
    
    [can_start, final_name, final_exists] = chunk_start(segmentations_filtered_location, segmentation_filename_name_without_extension);

    if false && final_exists
      % Debug plot:
      % Show the segmentation results just for filtered segmentations:
      
      a = load(window_filename);
      b = load(final_name);
      a, b
      
      segmentation_structure = b.segmentation_structure;
      window_number_slices = b.window_number_slices;

      clf
      
      show_segmentation_as_surface = false
      % show_segmentation_as_surface = true
      if show_segmentation_as_surface
        % imshow(reshape_contrast([a.current_window_image]))
        % imagesc(contrast_stretch(mean(a.current_window_image, 3)))
        image_to_plot = contrast_stretch(mean(a.current_window_image, 3));

        % imshow(image_to_plot), pause
        surface(zeros(size(image_to_plot)), image_to_plot, 'EdgeColor', 'none', 'AmbientStrength', 1, 'DiffuseStrength', 0, 'SpecularStrength', 0)
        hold on
        patch(segmentation_structure, 'EdgeColor', 'none', 'FaceColor', [1, .4, .1])
        % patch(segmentation_structure, 'EdgeColor', 'none', 'FaceColor', [1, .4, .1], 'FaceAlpha', .5)
        camlight, lighting phong
        hold off
        axis equal
        axis tight
        view(3)
      else
        segmentation_image = segmentation_rasterization_function(segmentation_structure, [window_size_2d, window_number_slices]);
        % segmentation_image = segmentation_rasterization_function(segmentation_structure);
        % imagesc(reshape_contrast([a.current_window_image; a.current_window_image .* (segmentation_image < .5) + min(a.current_window_image(:)) * (segmentation_image >= .5)], -1))
        imagesc(reshape_contrast([a.current_window_image; a.current_window_image .* (1 - segmentation_image) + min(a.current_window_image(:)) * (segmentation_image)], -1))
        axis image
      end
      colormap(pmkmp)
      % keyboard
      pause
    end

    if ~can_start
      continue
    end
    
    % fprintf('Filtering segmentation: %s\n', segmentation_filename)
    fprintf('Filtering segmentation: %s', segmentation_filename_name)
    
    % Step 1: Compute features:
    a = load(region_filename);
    b = load(segmentation_filename);
    current_window_image = a.current_window_image;
    current_segmentation_structure = b.segmentation_structure;

    current_segmentation_features = compute_mesh_segmentation_evaluation_features(current_window_image, current_segmentation_structure);
    
    % Here 1 means good, 2 means bad:
    current_decision = svm_info.classifier_info.prediction_function(current_segmentation_features);
    fprintf(', decision: %d\n', current_decision)
    if current_decision == 1
      copyfile(segmentation_filename, final_name);
      % Don't remove the lock/temp file so every run doesn't repeat these evaluations:
      chunk_finish(segmentations_filtered_location, segmentation_filename_name);
    end
    
  end
  
  % beep, keyboard
  
  % error('Implementation yet unfinished below this line!')
  % warning('Implementation of filtering yet unfinished!')

  t_cell_info.segmentation_filtering_info = struct();
  t_cell_info.segmentation_filtering_info.svm_info = svm_info;
  
  
  
end  

