function [t_cell_info] = master_script_get_synapse_info(t_cell_info, options)
  % 2013-11-06 tebuck: Copied from master_script.m.
  % 2014-03-02 tebuck: Added two-point (and more general) synapse annotation processing.
  % 
  % 
  % 
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Produce lists of synapse files, images, and individual synapses, frames, and cells:
  

  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;
  synapse_location = master_script_options.synapse_location;
  image_location = master_script_options.image_location;
  
  sensors_to_exclude = master_script_options.sensors_to_exclude;
  
  t_cell_info.synapse_info = struct();

  
  function [result_path] = clean_relative_path(given_path)
    % Remove any initial slash and reduce repeated slashes:
    result_path = given_path;
    % result_path = regexprep(result_path, '^/', '');
    % result_path = regexprep(result_path, '/+', '/');
    % result_path = regexprep([result_path, '/'], '/+$', '/');
    % result_path = regexprep([result_path, '/'], '/+', '/');
    result_path = regexprep(strcat(result_path, '/'), '/+', '/');
    result_path = regexprep(result_path, '^/', '');
  end
  
  
  function [result_path] = clean_absolute_path(given_path)
    % Remove any initial slash and reduce repeated slashes:
    result_path = given_path;
    % result_path = regexprep([result_path, '/'], '/+', '/');
    result_path = regexprep(strcat(result_path, '/'), '/+', '/');
  end
  
  
  function [result_protein_synonyms] = get_protein_synonyms(given_synonym)
    % Return the given protein name unless master_script_options.protein_name_synonyms specifies the proper name for it:
    result_protein_synonyms = {given_synonym};
    for protein_synonym_index = 1:length(master_script_options.protein_name_synonyms)
      if any(strcmp(master_script_options.protein_name_synonyms{protein_synonym_index}, given_synonym))
        result_protein_synonyms = master_script_options.protein_name_synonyms{protein_synonym_index};
        return
      end
    end
  end
  
  
  function [result_protein_name] = get_protein_name_given_synonym(given_synonym)
    % Return the given protein name unless master_script_options.protein_name_synonyms specifies the proper name for it:
    result_protein_name = get_protein_synonyms(given_synonym);
    result_protein_name = result_protein_name{1};
  end
  
  
  t_cell_info.synapse_info.clean_relative_path = @clean_relative_path;
  t_cell_info.synapse_info.clean_absolute_path = @clean_absolute_path;
  t_cell_info.synapse_info.get_protein_synonyms = @get_protein_synonyms;
  t_cell_info.synapse_info.get_protein_name_given_synonym = @get_protein_name_given_synonym;
  
  
  % 2014-01-21 tebuck: modifying so that synapse_location can now be a cell array of locations:
  
  if ~iscell(synapse_location)
    synapse_location = {synapse_location};
  end
  
  if ~iscell(image_location)
    image_location = {image_location};
  end
  
  
  % Get a list of all synapse files:
  % Read CSV files to get synapse information, and use their filenames to search for the correct image directory and images:
  
  coordinate_file_list = [];
  for synapse_file_index = 1:length(synapse_location)
    current_synapse_location = synapse_location{synapse_file_index};
    current_coordinate_file_list = dir([current_synapse_location, '*.csv']);
    coordinate_file_list = [coordinate_file_list; strcat(current_synapse_location, {current_coordinate_file_list.name}.')];
  end
  
  run_list = {};
  for coordinate_file_index = 1:length(coordinate_file_list)
    coordinate_file = coordinate_file_list{coordinate_file_index};
    run_list{end + 1, 1} = coordinate_file;
  end
  
  number_runs = length(run_list);

  
  % warning('>>>> HACK, setting number_runs = 1 for testing purposes'), number_runs = 1;
  
  run_relative_path_list = cell(number_runs, 1);
  run_key_list = cell(number_runs, 1);
  
  run_data_list = cell(number_runs, 1);
  run_data_format_list = cell(number_runs, 1);
  run_images_list = cell(number_runs, 1);
  run_brightfield_images_list = cell(number_runs, 1);
  run_image_frame_indices_list = cell(number_runs, 1);
  run_image_size_list = zeros(number_runs, 3);
  run_image_voxel_size_list = zeros(number_runs, 3);
  
  run_number_cells = zeros(number_runs, 1);
  run_number_frames = zeros(number_runs, 1);
  
  run_point_tracks = cell(number_runs, 1);
  run_relative_time_tracks = cell(number_runs, 1);
  
  run_relative_times = cell(number_runs, 1);
  run_number_relative_times = zeros(number_runs, 1);
  all_relative_times = [];
  number_all_relative_times = 0;
  all_relative_times_seconds_data = [];
  all_relative_times_seconds = [];
  
  % runs_to_keep = true(number_runs, 1);
  runs_to_keep = false(number_runs, 1);
  
  filterable_run_variables = {'run_list', 'run_relative_path_list', 'run_key_list', 'run_data_list', 'run_data_format_list', 'run_images_list', 'run_brightfield_images_list', 'run_image_frame_indices_list', 'run_image_size_list', 'run_image_voxel_size_list', 'run_number_cells', 'run_number_frames', 'run_point_tracks', 'run_relative_time_tracks', 'run_relative_times', 'run_number_relative_times'};
  
  
  function [result] = conditional_str2num(given_string)
    % Convert to numeric value if it is numeric, otherwise keep the string value:
    result = given_string;
    if ~isnumeric(result)
      result = str2num(result);
    end
    if isempty(result)
      result = given_string;
    end
  end
  
  function [result] = conditional_str2double(given_string)
    % Convert to numeric value if it is numeric, otherwise keep the string value (but using str2double):
    if isempty(given_string)
      result = nan;
    elseif ~isnumeric(given_string)
      result = str2double(given_string);
      if isnan(result) && ~strcmpi(given_string, 'nan')
        result = given_string;
      end
    else
      result = given_string;
    end
  end
  
  
  % Collect synapse info:
  % fprintf('\nCollecting synapse locations from Excel files in "%s"\n', master_script_options.annotation_location)
  % fprintf('\nCollecting synapse locations from Excel files in "%s"\n', master_script_options.synapse_location)
  fprintf('\nCollecting synapse locations from Excel files in the following locations:\n');
  for synapse_file_index = 1:length(synapse_location)
    fprintf('  %s\n', synapse_location{synapse_file_index})
  end
  for run_index = 1:number_runs
    % Filename of file specifying synapse locations in 2D and time:
    run_file = run_list{run_index};

    % fprintf('Collect synapse info: %s\n', run_file)
    
    [~, run_file_name, run_file_extension] = fileparts(run_file);
    run_file_name_tokens = regexp(run_file_name, '(.+)_(.+)_(.+)', 'tokens');
    run_base_subdirectory = run_file_name_tokens{1}{1};
    run_protein = run_file_name_tokens{1}{2};
    run_name = run_file_name_tokens{1}{3};
    
    % Relative paths used as unique identifiers for later naming of other intermediate and results files:
    run_relative_path = [run_file_name, run_file_extension];
    run_relative_path = clean_relative_path(run_relative_path);
    run_relative_path = regexprep(run_relative_path, '/+$', '');
    run_relative_path_list{run_index} = run_relative_path;
    run_key = strrep(run_relative_path, '/', '_');
    run_key_list{run_index} = run_key;

    % Some worksheets/run names are date, protein, run:
    % Accomodate spaces in the date:
    run_tokens = regexpi(run_file, '(([0-9]{2} ?){3}) *[^ ]* *run *([0-9]+) *\.csv', 'tokens');
    run_date = run_tokens{1}{1};
    run_date = strrep(run_date, ' ', '');
    run_number = run_tokens{1}{2};
    if master_script_options.debug_synapse_file_processing_verbose
      fprintf('Worksheet key "%s"\n', run_key)
    end

    if length(run_date) ~= 6
      warning('    run_date "%s" is not six digits, ignoring run', run_date)
      continue
    end
          
    % Check for run images in directories named after the protein or its synonyms:
    
    run_protein_name_synonyms_protein_index = find(cellfun(@(given_protein_name_synonyms)any(strcmp(given_protein_name_synonyms, run_protein)), master_script_options.protein_name_synonyms));
    if isempty(run_protein_name_synonyms_protein_index)
      run_protein_name_synonyms = {run_protein};
    else
      run_protein_name_synonyms = master_script_options.protein_name_synonyms{run_protein_name_synonyms_protein_index};
    end
    run_protein_name_number_synonyms = length(run_protein_name_synonyms);
    run_protein_name_synonym_index = 1;
    done = false;
    for image_location_index = 1:length(image_location)
      current_image_location = image_location{image_location_index};
      for run_protein_name_synonym_index = 1:run_protein_name_number_synonyms
        current_protein_synonym = run_protein_name_synonyms{run_protein_name_synonym_index};
        run_base_directory = clean_absolute_path([current_image_location, run_base_subdirectory, '/', current_protein_synonym, '/']);
        % done = exist(run_base_directory, 'file') > 0;
        if ~exist(run_base_directory, 'file')
          if master_script_options.debug_synapse_file_processing_verbose
            fprintf('    "%s" does not exist!\n', run_base_directory)
          end
          continue
        end
        
        [~, ~, protein_file_paths] = dirr(run_base_directory, 'name', '');
        protein_file_paths = protein_file_paths';
        protein_file_paths = strrep(protein_file_paths, run_base_directory, '');
        run_file_paths = protein_file_paths;
        run_paths_good = true(size(run_file_paths));
        % run_path_pattern = ['/[^/]*', run_date, '.*Run *', run_number, '[^/]*/'];
        run_path_pattern = sprintf('/[^/]*%s ?%s ?%s.*run *%s[^/]*/', run_date(1:2), run_date(3:4), run_date(5:6), run_number);
        run_paths_good = run_paths_good & ~cellfun(@isempty, regexpi(run_file_paths, run_path_pattern, 'start'));
        % run_file_paths
        run_file_paths = run_file_paths(run_paths_good);
        run_paths_good = true(size(run_file_paths));
        % run_file_paths
        
        % Find all fluorescence images of sensors:
        
        % image_path_patterns = {'/[^/]*T([0-9]{5})C[0-9]{2}Z\.[^/]*$', 'EGFP.*T([0-9]+)[^/]*\.[^/]+$', 'GFP.*\(Timepoint\W*([0-9]+)\)[^/]*\.[^/]+$'};
        image_path_patterns = {'/[^/]*T([0-9]{5})C[0-9]{2}Z\.[^/]*$', 'EGFP.*T([0-9]+)[^/]*\.[^/]+$', 'GFP.*\(Timepoint\W*([0-9]+)\)[^/]*\.[^/]+$'};

        for image_path_pattern_index = 1:length(image_path_patterns)
          image_path_pattern = image_path_patterns{image_path_pattern_index};
          run_image_paths_good = run_paths_good;
          run_image_paths_good = run_image_paths_good & ~cellfun(@isempty, regexp(run_file_paths, image_path_pattern, 'start'));
          run_image_paths = run_file_paths(run_image_paths_good);
          if isempty(run_image_paths)
            % Try another pattern:
            % warning('    pattern "%s" unmatched', image_path_pattern)
            continue
          else
            % Found images matching this pattern:
            run_image_paths = strcat(run_base_directory, run_image_paths);
          end
          % run_images_list{run_index} = run_image_paths;
          
          run_image_frame_indices = cellfun(@(x)str2num(x{1}{1}), regexp(run_image_paths, image_path_pattern, 'tokens'));
          
          if run_image_frame_indices(1) == 0
            if master_script_options.debug_synapse_file_processing_verbose
              fprintf('    run_key "%s": First frame is frame zero, adding one to the frame index for each image\n', run_key)
            end
            run_image_frame_indices = run_image_frame_indices + 1;
          end
          
          % Correct indices in run_images_list{run_index} so that indices corresponds to the frame number in the filename (now an empty image filename will appear where the image is missing):
          % Note: run_image_frame_indices_list is currently not used in other files, so everywhere it is assumed that all frames are present!
          run_image_frame_indices_list{run_index} = run_image_frame_indices;
          listed_run_image_paths = run_image_paths;
          run_image_paths = cell(max(run_image_frame_indices), 1);
          run_image_paths(run_image_frame_indices) = listed_run_image_paths;
          run_images_list{run_index} = run_image_paths;
          
        end
        
        if isempty(run_images_list{run_index})
          continue
        end

        % Find all brightfield images:
        
        % brightfield_image_path_pattern = [run_date, '.+Run *', run_number, '.*DIC.*/[^/]*\.[^/]*$'];
        % brightfield_image_path_pattern = [run_date, '.+Run *', run_number, '.*DIC.*/[^/]*DIC[^/]*\.[^/]*$'];
        % brightfield_image_path_pattern = [run_date, '.+Run *', run_number, '.*(DIC.*/)?[^/]*DIC\.[^/]*$'];
        brightfield_image_path_pattern = ['.*(DIC.*/)?[^/]*DIC\.[^/]*$'];
        run_brightfield_image_paths_good = run_paths_good;
        run_brightfield_image_paths_good = run_brightfield_image_paths_good & ~cellfun(@isempty, regexpi(run_file_paths, brightfield_image_path_pattern, 'start'));
        run_brightfield_image_paths = run_file_paths(run_brightfield_image_paths_good);
        if ~isempty(run_brightfield_image_paths)
          run_brightfield_image_paths = strcat(run_base_directory, run_brightfield_image_paths);
        end
        run_brightfield_images_list{run_index} = run_brightfield_image_paths;
        
        % Get the synapse locations in 2D and time:
        try
          [~, ~, run_file_extension] = fileparts(run_file);
          % [~, ~, run_data] = xlsread(run_file.file, run_file.worksheet, '', 'basic');
          % run_data = importdata(run_file, ',');
          % run_data = run_data.textdata;
          % run_data = cellfun(@conditional_num2str, run_data, 'UniformOutput', false);
          % run_data_number_columns_inference_data = importdata(run_file, ',');
          % if isfield(run_data_number_columns_inference_data, 'colheaders')
            % run_data_number_columns_inference_data = getfield(run_data_number_columns_inference_data, 'colheaders');
          % else
            % run_data_number_columns_inference_data = getfield(run_data_number_columns_inference_data, 'textdata');
          % end
          % run_data_number_columns = size(run_data_number_columns_inference_data, 2);
          run_file_handle = fopen(run_file);
          run_data_first_line = fgetl(run_file_handle);
          % Find number of columns as number of non-quoted commas plus one:
          run_data_first_line_commas = regexprep(run_data_first_line, '"[^"]*"', '');
          % run_data_first_line_commas = regexprep(run_data_first_line, '("[^"]*")|(''[^"]*'')', '');
          % run_data_first_line_commas = regexprep(run_data_first_line_commas, '[^,]+', '');
          % beep, keyboard
          % run_data_number_columns = length(run_data_first_line_commas) + 1;
          run_data_number_columns = sum(run_data_first_line_commas == ',') + 1;
          fseek(run_file_handle, 0, 'bof');
          % run_data = textscan(run_file_handle, [repmat('%q,', 1, run_data_number_columns - 1), '%s']);
          % run_data = textscan(run_file_handle, '%s', 'Delimiter', ',');
          % run_data = textscan(run_file_handle, [repmat('%[^,"\r\n]', 1, run_data_number_columns - 1), '%[^,"\r\n]'], inf);
          run_data = {};
          field_pattern = '([^,"\r\n]*|("[^"\r\n]*"))';
          while ~feof(run_file_handle)
            % run_data{end + 1} = textscan(run_file_handle, '%q', 'Delimiter', ',');
            current_line = fgetl(run_file_handle);
            % run_data{end + 1, 1} = textscan(current_line, [repmat('%[^,"\r\n],', 1, run_data_number_columns - 1), '%[^,"\r\n]']);
            run_data{end + 1, 1} = regexp(current_line, [repmat([field_pattern, ','], 1, run_data_number_columns - 1), field_pattern], 'tokens');
            run_data{end} = run_data{end}{1};
            % current_line
            % pause
            % beep, keyboard
          end
          fclose(run_file_handle);
          run_data = cat(1, run_data{:});
          % run_data = run_data.textdata;
          run_data_empty = cellfun(@isempty, run_data);
          % run_data(run_data_empty) = {{nan}};
          run_data(run_data_empty) = {nan};
          % run_data = cellfun(@(x)x{1}, run_data, 'UniformOutput', false);
          run_data = cellfun(@conditional_str2double, run_data, 'UniformOutput', false);
          % whos run_data, beep, keyboard
        catch err
          fprintf('    Error reading coordinates!\n')
          rethrow(err)
        end
        [run_data, run_data_format] = get_excel_file_coordinates(run_data);
        
        ignore_runs_without_brightfield_images = false;
        % ignore_runs_without_brightfield_images = true;
        if isempty(run_brightfield_images_list{run_index})
          if ignore_runs_without_brightfield_images
            warning('    No brightfield stack detected, ignoring run')
            continue
          else
            if master_script_options.debug_synapse_file_processing_verbose
              fprintf('    No brightfield stack detected\n')
            end
          end
        end

        
        done = true;
        
        
        if false && strcmpi(run_protein, 'Actin')
          % Debug info:
          % run_base_directory
          run_file.file, run_file.worksheet, run_base_directory, done
          keyboard
        end
        
        % % Used to break once we found the directory, but now we might have multiple entries in image_location, so do not:
        if done, break, end
      end
      % % Used to break once we found the directory, but now we might have multiple entries in image_location, so do not:
      if done, break, end
    end

    if done
      runs_to_keep(run_index) = true;
    else
      if master_script_options.debug_synapse_file_processing_verbose
        warning('    No appropriate images found, ignoring run')
      end
    end
    
    % error('Implementation yet unfinished below this line!')
    
    
    run_data_list{run_index} = run_data;
    run_data_format_list{run_index} = run_data_format;
    run_data_as_array = cell2mat(reshape(run_data, [], 1));
    if isempty(run_data_as_array)
      if master_script_options.debug_synapse_file_processing_verbose
        % In case there are no cell couples, produce a warning:
        fprintf('    No cell couples specified, ignoring run\n')
      end
      runs_to_keep(run_index) = false;
      continue
    end
    
    % Filter cells with one frame:
    if master_script_options.minimum_number_frames_per_cell > 1
      number_frames_per_cell = hist(run_data_as_array(:, end), unique(run_data_as_array(:, end)));
      cells_to_remove = reshape(find(number_frames_per_cell < master_script_options.minimum_number_frames_per_cell), 1, []);
      for cell_to_remove = cells_to_remove
        for frame_index = 1:length(run_data)
          if numel(run_data{frame_index}) == 0
            continue
          end
          indices_to_remove = run_data{frame_index}(:, end) == cell_to_remove;
          run_data{frame_index}(indices_to_remove, :) = [];
        end
      end
      run_data_list{run_index} = run_data;
      run_data_as_array = cell2mat(reshape(run_data, [], 1));
    end
    
    if isempty(run_data_as_array)
      % In case there are no cell couples left, produce a warning:
      warning('    No cell couples specified after filtering cells with fewer than %d frames, ignoring run', master_script_options.minimum_number_frames_per_cell)
      runs_to_keep(run_index) = false;
      continue
    end
    
    number_cells = max(run_data_as_array(:, end));
    number_frames = size(run_data, 1);
    run_number_cells(run_index) = number_cells;
    run_number_frames(run_index) = number_frames;
    
    % Make sure the image list has a blank entry for every frame entry in run_data:
    if length(run_images_list{run_index}) < number_frames
      run_images_list{run_index} = [run_images_list{run_index}; cell(number_frames - size(run_images_list{run_index}, 1), 1)];
    end

    
    % This assumes the files are named with numbers 1 to number_frames:
    cell_synapse_point_tracks = cell(number_cells, number_frames);
    cell_relative_time_tracks = cell(number_cells, number_frames);
    if false
      % Debug info:
      % keyboard
      % dbstop if warning
      number_cells, number_frames
    end
    
    % Produce a text file listing the position of each cell in its first frame:
    cell_indices_seen = [];
    % X, Y, frame number
    cell_positions = zeros(0, 3);
    
    
    % Loop through all frames with marked synapses and assign them to one time series per cell:
    for frame_index = 1:number_frames
      frame_synapse_locations = run_data{frame_index};
      for frame_cell_index = 1:size(frame_synapse_locations, 1)
      
        % For any method listing points, the last two are current_cell_relative_time and current_cell_index, the rest being the landmarks, with the first two assumed to be integer-valued 2D coordinates suitable for use with the old one-point annotation pipeline:
        current_cell_relative_time = frame_synapse_locations(frame_cell_index, end - 1);
        current_cell_index = frame_synapse_locations(frame_cell_index, end);
        % Add segmentation for this frame to the appropriate cell:
        current_synapse_center_rounded = frame_synapse_locations(frame_cell_index, 1:2);
        if master_script_options.use_two_point_synapses
          cell_synapse_point_tracks{current_cell_index, frame_index} = frame_synapse_locations(frame_cell_index, 1:end - 2);
        else
          cell_synapse_point_tracks{current_cell_index, frame_index} = current_synapse_center_rounded;
        end
        cell_relative_time_tracks{current_cell_index, frame_index} = current_cell_relative_time;

        % Produce a text file listing the position of each cell in its first frame:
        if master_script_options.create_cell_position_text_file
          if ~ismember(current_cell_index, cell_indices_seen)
            cell_indices_seen = [cell_indices_seen; current_cell_index];
            cell_positions(end + 1, :) = [current_synapse_center_rounded, frame_index];
          end
        end
        
      end
    end

    if master_script_options.create_cell_position_text_file
      cell_positions_filename = [t_cell_info.path_info.cell_positions_location, run_key_list{run_index}, '.csv'];
      file_handle = fopen(cell_positions_filename, 'wt');
      fprintf(file_handle, 'X, Y, frame number\n');
      fclose(file_handle);
      dlmwrite(cell_positions_filename, cell_positions, '-append');
    end
    
    run_point_tracks{run_index} = cell_synapse_point_tracks;
    run_relative_time_tracks{run_index} = cell_relative_time_tracks;
    
    % How many cells are available for each relative time?
    run_relative_times{run_index} = unique(cell2mat(cell_relative_time_tracks(:)));
    if any(~ismember(run_relative_times{run_index}(:), [-2:9]))
      warning('run_relative_times{%d} (run_key "%s") contains non-standard relative times!', run_index, run_key_list{run_index})
      if false
        % Debug info:
        current_run_relative_times = run_relative_times{run_index}
        beep, keyboard
      end
    end
    run_number_relative_times(run_index) = length(run_relative_times{run_index});
    
    % What relative times do we have overall?
    all_relative_times = unique([all_relative_times; reshape(run_relative_times{run_index}, [], 1)]);
    number_all_relative_times = length(all_relative_times);
    
    % Collect relative times to determine approximately how many seconds each frame is from synapse formation:
    for relative_times_row_index = 1:size(cell_relative_time_tracks, 1)
      current_relative_times = cell2mat(cell_relative_time_tracks(relative_times_row_index, :));
      current_relative_times_zero_index = find(current_relative_times == 0);
      if isempty(current_relative_times_zero_index)
        continue
      end
      current_frames = find(~cellfun(@isempty, cell_relative_time_tracks(relative_times_row_index, :)));
      current_frames = (current_frames - current_frames(current_relative_times_zero_index)) * 20;

      all_relative_times_seconds_data = [all_relative_times_seconds_data; current_relative_times', current_frames'];
      
    end
    
  end
  
  all_relative_times_seconds = zeros(size(all_relative_times));
  for relative_time_index = 1:number_all_relative_times
    all_relative_times_seconds(relative_time_index) = mean(all_relative_times_seconds_data(all_relative_times_seconds_data(:, 1) == all_relative_times(relative_time_index), 2));
  end
  all_relative_times_seconds
  all_relative_times_seconds_rounded = round(all_relative_times_seconds)
  
  
  function filter_runs(given_runs_to_keep)
    % Remove runs from all relevant variables compactly:
    for filterable_run_variable_index = 1:length(filterable_run_variables)
      filtering_command = sprintf('%s = %s(given_runs_to_keep, :);', filterable_run_variables{filterable_run_variable_index * [1, 1]});
      eval(filtering_command);
    end
  end
  
  
  % Remove runs without images:
  
  filter_runs(runs_to_keep);
  
  number_runs = length(run_relative_path_list);
  
  
  % Filename and run_key parsing:
  
  condition_retrieval_function = @(given_strings)cellfun(@(x)x{1}{1}, regexp(given_strings, '[^_/]+ - ([^_/]+)_[^_/]+_[^_/]+\.csv', 'tokens'), 'UniformOutput', false);
  sensor_retrieval_function = @(given_strings)cellfun(@(x)x{1}{1}, regexp(given_strings, '[^_/]+ - [^_/]+_([^_/]+)_[^_/]+\.csv', 'tokens'), 'UniformOutput', false);
  % Assume everything was taken this century, and reorder so that datenum operates directly on this output after running cell2mat:
  % run_date_retrieval_function = @(given_strings)cellfun(@(x)cellfun(@(y)str2num(y), x{1}(1:3)) + [0, 0, 2000], regexp(given_strings, '[^_/]+ - [^_/]+_[^_/]+_[^_/]*([0-9]{2}) *([0-9]{2}) *([0-9]{2}) *[^_/]* *[Rr]un *[0-9]+ *\.csv', 'tokens'), 'UniformOutput', false);
  run_date_retrieval_function = @(given_strings)cellfun(@(x)cellfun(@(y)str2num(y), x{1}([3, 1, 2])) + [2000, 0, 0], regexp(given_strings, '[^_/]+ - [^_/]+_[^_/]+_[^_/]*([0-9]{2}) *([0-9]{2}) *([0-9]{2}) *[^_/]* *[Rr]un *[0-9]+ *\.csv', 'tokens'), 'UniformOutput', false);
  
  % Observed combinations, i.e.:
  % condition_sensor_combinations = unique(deformations_aligned_condition_sensor_list)
  % This doesn't work for some stupid reason:
  % condition_sensor_combinations = unique([condition_retrieval_function(run_key_list), sensor_retrieval_function(run_key_list)], 'rows')
  
  function [result_combinations] = unique_condition_sensor_combination_function(given_combinations)
    % Find unique condition-sensor combinations as represented by a two-column cell array of strings (first column condition, second sensor):
    % Explicit two-column version:
    result_combinations = strcat(given_combinations(:, 1), '|', given_combinations(:, 2));
    result_combinations = unique(result_combinations);
    [result_conditions, result_sensors] = strtok(result_combinations, '|');
    result_sensors = strrep(result_sensors, '|', '');
    result_combinations = [result_conditions, result_sensors];
  end
  
  function [result_combination_pairs] = unique_condition_sensor_combination_pair_function(given_combination_pairs)
    % % Find unique condition-sensor combinations as represented by a two-column cell array of strings (first column condition, second sensor):
    % Explicit two-by-two version:
    result_combination_pairs = cellfun(@(x)reshape(x.', 1, []), given_combination_pairs, 'UniformOutput', false);
    result_combination_pairs = cellfun(@(x)strjoin(x, '|'), result_combination_pairs, 'UniformOutput', false);
    result_combination_pairs = unique(result_combination_pairs);
    result_combination_pairs = cellfun(@(x)strsplit(x, '|'), result_combination_pairs, 'UniformOutput', false);
    result_combination_pairs = cellfun(@(x)reshape(x, 2, 2).', result_combination_pairs, 'UniformOutput', false);
  end
  
  if false
    % Debug info for unique_condition_sensor_combination_pair_function:
    test_combination_pairs = {{'full', 'actin'; 'b7', 'actin'}; {'full', 'actin'; 'b7block', 'actin'}; {'full', 'arp3'; 'b7', 'arp3'}; {'full', 'actin'; 'b7', 'actin'}}
    test_combination_pairs{:}
    unique_test_combination_pairs = unique_condition_sensor_combination_pair_function(test_combination_pairs)
    unique_test_combination_pairs{:}
    dbstack, beep, keyboard
  end
  
  function [result_memberships] = condition_sensor_combination_ismember(given_combinations1, given_combinations2)
    % Check if each combination from given_combinations1 is in given_combinations2 (assumes both are cell arrays with two columns):
    % % If the first entry is just a pair, assume everything else is:
    conditions1 = given_combinations1(:, 1);
    sensors1 = given_combinations1(:, 2);
    conditions2 = given_combinations2(:, 1);
    sensors2 = given_combinations2(:, 2);
    result_memberships = ismember(conditions1, conditions2) & ismember(sensors1, sensors2);
  end
  
  function [result_memberships] = condition_sensor_combination_pair_ismember(given_combination_pairs1, given_combination_pairs2)
    % Check if each pair of combinations from given_combination_pairs1 is in given_combination_pairs2 (assumes both are cell arrays of cell arrays):
    % Explicit two-by-two version:
    % % If the first entry is just a pair, assume everything else is:
    former_conditions1 = cellfun(@(x)x{1, 1}, given_combination_pairs1, 'UniformOutput', false);
    former_sensors1 = cellfun(@(x)x{1, 2}, given_combination_pairs1, 'UniformOutput', false);
    latter_conditions1 = cellfun(@(x)x{2, 1}, given_combination_pairs1, 'UniformOutput', false);
    latter_sensors1 = cellfun(@(x)x{2, 2}, given_combination_pairs1, 'UniformOutput', false);
    former_conditions2 = cellfun(@(x)x{1, 1}, given_combination_pairs2, 'UniformOutput', false);
    former_sensors2 = cellfun(@(x)x{1, 2}, given_combination_pairs2, 'UniformOutput', false);
    latter_conditions2 = cellfun(@(x)x{2, 1}, given_combination_pairs2, 'UniformOutput', false);
    latter_sensors2 = cellfun(@(x)x{2, 2}, given_combination_pairs2, 'UniformOutput', false);
    result_memberships = ismember(former_conditions1, former_conditions2) & ismember(former_sensors1, former_sensors2) & ismember(latter_conditions1, latter_conditions2) & ismember(latter_sensors1, latter_sensors2);
  end
  
  if false
    % Debug info for unique_condition_sensor_combination_pair_function:
    test_combination_pairs = {{'full', 'arp3'; 'b7', 'arp3'}; {'full', 'wave2'; 'b7', 'wave2'}}
    test_combination_pairs{:}
    test_combination_pairs2 = {{'full', 'actin'; 'b7', 'actin'}; {'full', 'actin'; 'b7block', 'actin'}; {'full', 'arp3'; 'b7', 'arp3'}; {'full', 'actin'; 'b7', 'actin'}}
    test_combination_pairs2{:}
    test_combination_pairs_memberships = condition_sensor_combination_pair_ismember(test_combination_pairs, test_combination_pairs2)
    dbstack, beep, keyboard
  end
  
  if false
    condition_sensor_combinations
    % dbstack, beep, keyboard
  end
  condition_sensor_combinations = unique_condition_sensor_combination_function([condition_retrieval_function(run_key_list), sensor_retrieval_function(run_key_list)]);
  % error

  if false
    condition_sensor_combinations2 = condition_sensor_combinations
    all_same = all(strcmp(condition_sensor_combinations(:), condition_sensor_combinations2(:)))
    dbstack, beep, keyboard
  end
  
  % Remove runs for specific sensors:
  
  run_keys_to_exclude = master_script_options.run_keys_to_exclude;
  for sensors_to_exclude_index = 1:length(sensors_to_exclude)
    fprintf('Removing sensor %s!\n', sensors_to_exclude{sensors_to_exclude_index})
    run_keys_to_exclude = [run_keys_to_exclude; run_key_list(strcmp(sensor_retrieval_function(run_key_list), sensors_to_exclude{sensors_to_exclude_index}))];
    condition_sensor_combinations(strcmp(condition_sensor_combinations(:, 2), sensors_to_exclude{sensors_to_exclude_index}), :) = [];
  end
  
  % Remove specific runs:
  
  runs_to_keep = true(number_runs, 1);
  for run_keys_to_exclude_index = 1:length(run_keys_to_exclude)
    fprintf('Removing run_key %s!\n', run_keys_to_exclude{run_keys_to_exclude_index})
    runs_to_keep(strcmp(run_key_list, run_keys_to_exclude{run_keys_to_exclude_index})) = false;
  end
  
  % Remove runs in specific date ranges:
  
  date_ranges_to_exclude = master_script_options.date_ranges_to_exclude;
  current_run_dates = run_date_retrieval_function(run_key_list);
  
  if false
    % Debug info:
    current_run_dates
    error
  end
  
  if any(cellfun(@isempty, current_run_dates))
    current_run_dates
    error('current_run_dates empty for at least one run!')
  end
  current_run_dates = cell2mat(current_run_dates);
  current_run_datenums = datenum(current_run_dates);
  date_range_runs_to_keep = true(number_runs, 1);
  for date_ranges_to_exclude_index = 1:size(date_ranges_to_exclude, 1)
    fprintf('Removing date_range %04d-%02d-%02d to %04d-%02d-%02d!\n', date_ranges_to_exclude(date_ranges_to_exclude_index, :))
    % runs_to_keep((current_run_datenums >= datenum(date_ranges_to_exclude(date_ranges_to_exclude_index, 1:3))) & (current_run_datenums <= datenum(date_ranges_to_exclude(date_ranges_to_exclude_index, 4:6)))) = false;
    % runs_to_keep((current_run_datenums >= datenum(date_ranges_to_exclude(date_ranges_to_exclude_index, 1:3))) & (current_run_datenums <= datenum(date_ranges_to_exclude(date_ranges_to_exclude_index, 4:6)))) = false;
    date_range_runs_to_keep((current_run_datenums >= datenum(date_ranges_to_exclude(date_ranges_to_exclude_index, 1:3))) & (current_run_datenums <= datenum(date_ranges_to_exclude(date_ranges_to_exclude_index, 4:6)))) = false;
  end
  runs_to_keep = runs_to_keep & date_range_runs_to_keep;
  
  if false
    % Debug info:
    current_run_dates_novel_date_range_runs_to_keep = [current_run_dates, date_range_runs_to_keep]
    error
  end
  
  if false
    % Debug info:
    runs_with_to_keep = [regexprep(run_key_list, '^.* - ', ''), num2cell(runs_to_keep)]
    error
  end
  
  
  number_condition_sensors = size(condition_sensor_combinations, 1);
  condition_sensor_combinations, number_condition_sensors
  
  % Remove runs from all relevant variables compactly:
  number_runs, number_runs_to_keep = sum(runs_to_keep)
  filter_runs(runs_to_keep);
  number_runs = number_runs_to_keep;
  
  
  conditions = unique(condition_sensor_combinations(:, 1));
  sensors = unique(condition_sensor_combinations(:, 2));
  number_conditions = length(conditions)
  number_sensors = length(sensors)

  
  
  % % Correct indices in run_images_list{run_index} so that indices corresponds to the frame number in the filename:
  
  fprintf('\n')
  
  
  % Get fluorescent image frame and voxel sizes:
  % 02/27/2016 xruan
  % warning('>>>> HACK, hard-coding images'' voxel sizes using the date of CW''s move to University of Bristol!')
  for run_index = 1:number_runs
  
    current_run_key = run_key_list{run_index};
    current_run_number_frames = run_number_frames(run_index);
    current_run_images_list = run_images_list{run_index};
    current_run_images_list = current_run_images_list(1:current_run_number_frames);
    current_run_brightfield_images_list = run_brightfield_images_list{run_index};
    current_run_brightfield_images_list = current_run_brightfield_images_list(1:min(end, current_run_number_frames));
    
    frames_available = ~cellfun(@isempty, current_run_images_list);
    fluorescent_image_frame = robust_read_stack(current_run_images_list{find(frames_available, 1)});
    fluorescent_image_frame_info = imfinfo(current_run_images_list{find(frames_available, 1)});
    % xruan 07/26/2015
    % disable the image frame description because in some downsampled
    % images, there is no such description ImageDescription. Maybe we can
    % add it later if possible. 
    % fluorescent_image_frame_description = strrep(fluorescent_image_frame_info(1).ImageDescription, char(13), '  ');
    fluorescent_image_frame_size = size(fluorescent_image_frame);
    
    run_image_size_list(run_index, :) = fluorescent_image_frame_size;
    if false
      % Debug info:
      % fprintf('"%s" description: %s\n', current_run_key, fluorescent_image_frame_description)
      fprintf('"%s" description length: %d\n', current_run_key, length(fluorescent_image_frame_description))
      % beep, keyboard
      % error
    end
    current_run_date = cell2mat(run_date_retrieval_function({current_run_key}));
    % current_run_date(current_run_date(:, 3) < 100, 3) = current_run_date(current_run_date(:, 3) < 100, 3) + 2000;
    % All worksheet names' dates should be month day year (American style) as with old data:
    % current_run_date_number = datenum(current_run_date(3), current_run_date(1), current_run_date(2));
    current_run_date_number = datenum(current_run_date);
    % fprintf('current_run_date_number: %04d-%02d-%02d %d <= 2012-08-15 %d?\n', current_run_date([3, 1, 2]), current_run_date_number, datenum(2012, 8, 15))
    if current_run_date_number < datenum(2012, 8, 15) || current_run_date_number >= datenum(2014, 3, 9)
      run_image_voxel_size_list(run_index, :) = [0.406, 0.406, 0.4];
    else
      run_image_voxel_size_list(run_index, :) = [0.34, 0.34, 1];
    end
    % xruan 02/27/2016
    % warning('2014-11-19: Add rule for HS1 images! Or delete associated .csv files!')
    
  end
  
  % Segmentation and such were designed for this size, scale everything accordingly:
  design_voxel_size = [0.406, 0.406, 0.4];
  % % We want to rescale images to have maximum resolution and cubical voxels, so take the minimum voxel dimension:
  % desired_voxel_size = min(run_image_voxel_size_list(:)) .* ones(1, 3);
  desired_voxel_size = design_voxel_size;
  
  
  % Filter synapse centers outside of the image frame:
  
  runs_to_keep = true(number_runs, 1);

  for run_index = 1:number_runs
    
    current_run_key = run_key_list{run_index};
    current_run_number_frames = run_number_frames(run_index);
    current_run_images_list = run_images_list{run_index};
    current_run_images_list = current_run_images_list(1:current_run_number_frames);
    current_run_brightfield_images_list = run_brightfield_images_list{run_index};
    current_run_brightfield_images_list = current_run_brightfield_images_list(1:min(end, current_run_number_frames));
    current_run_point_tracks = run_point_tracks{run_index};
    
    frames_available = ~cellfun(@isempty, current_run_images_list);

    frame_size = run_image_size_list(run_index, :);
    
    frames_annotated = any(~cellfun(@isempty, current_run_point_tracks), 1).';
    cells_annotated = any(~cellfun(@isempty, current_run_point_tracks), 2);
    frame_cells_within_bounds = cellfun(@(x)(~isempty(x)) && (x(1, 1) >= 1 && x(1, 2) >= 1 && x(1, 1) <= frame_size(2) && x(1, 2) <= frame_size(1)), current_run_point_tracks);
    if false
      % Debug info:
      % current_run_point_tracks
      % frame_cells_within_bounds
      % frame_cells_within_bounds ~= ~cellfun(@isempty, current_run_point_tracks)
      any_out_of_bounds = any(any(frame_cells_within_bounds ~= ~cellfun(@isempty, current_run_point_tracks)))
      beep, keyboard
    end
    current_run_point_tracks(~frame_cells_within_bounds) = {[]};
    run_point_tracks{run_index} = current_run_point_tracks;
    
  end
  
  if false
    % Debug info:
    runs_to_keep
  end
  
  % error
  
  filter_runs(runs_to_keep);
  number_runs = number_runs_to_keep;
  
  
  % if master_script_options.debug_synapse_file_processing_verbose
  if false
    
    % List image sizes:
    
    for run_index = 1:number_runs
      current_run_key = run_key_list{run_index};
      current_run_number_frames = run_number_frames(run_index);
      current_run_images_list = run_images_list{run_index};
      current_run_images_list = current_run_images_list(1:current_run_number_frames);
      current_run_brightfield_images_list = run_brightfield_images_list{run_index};
      current_run_brightfield_images_list = current_run_brightfield_images_list(1:min(end, current_run_number_frames));
      
      frames_available = ~cellfun(@isempty, current_run_images_list);

      fluorescent_image_frame_size = size(robust_read_stack(current_run_images_list{find(frames_available, 1)}));
      fprintf('%s fluorescent_image_frame_size: [%d, %d, %d]\n', current_run_key, fluorescent_image_frame_size)
    end
    
    error
    
  end
  
  

  if master_script_options.debug_synapse_file_processing_verbose
    
    
    % Show random images with annotations:
    
    % First collect all runs by condition_sensor_combinations:
    
    run_key_list_viewing_order = [];
    % run_key_list_remaining = run_key_list;
    run_key_list_condition_sensors = [condition_retrieval_function(run_key_list), sensor_retrieval_function(run_key_list)];
    run_key_list_remaining_condition_sensor_indices = zeros(length(run_key_list), 1);
    
    for condition_sensor_combination_index = 1:number_condition_sensors
      condition_sensor_combination_run_indices = all(ismember(run_key_list_condition_sensors, condition_sensor_combinations(condition_sensor_combination_index, :)), 2);
      run_key_list_remaining_condition_sensor_indices(condition_sensor_combination_run_indices) = condition_sensor_combination_index;
    end
    
    % Now order by randomly selecting one run from each condition_sensor_combinations and repeating until no runs left:
    
    while length(run_key_list_viewing_order) < length(run_key_list)
      for condition_sensor_combination_index = 1:number_condition_sensors
        condition_sensor_combination_remaining_run_indices = find(run_key_list_remaining_condition_sensor_indices == condition_sensor_combination_index);
        if length(condition_sensor_combination_remaining_run_indices) > 0
          run_key_list_viewing_order(end + 1) = condition_sensor_combination_remaining_run_indices(randi(length(condition_sensor_combination_remaining_run_indices)));
          run_key_list_remaining_condition_sensor_indices(run_key_list_viewing_order(end)) = 0;
        end
      end
    end
    
    % % run_key_list_viewing_order
    % run_key_list_viewing_ordered = run_key_list(run_key_list_viewing_order);
    
    use_custom_run_key_list = false;
    % use_custom_run_key_list = true
    if use_custom_run_key_list
      % beep, dbstack, keyboard
      % Look at runs that had problems viewing just single frames:
      run_key_list_viewing_order = find(strcmp(sensor_retrieval_function(run_key_list), 'Akt')).';
      % run_brightfield_images_list = cellfun(@(x)run_brightfield_images_list(strcmp(x, run_key_list)), custom_run_key_list);
      % run_key_list = custom_run_key_list;
      % run_key_list_viewing_order = 1:length(run_key_list);
      % run_key_list_viewing_order = 1:length(run_key_list);
    end
    
    show_all_frames = false;
    % show_all_frames = true;
    % show_fluorescence_image = false;
    show_fluorescence_image = true;
    
    
    % For each runs in this partially random order:
    
    for run_index = reshape(run_key_list_viewing_order, 1, [])
      current_run_key = run_key_list{run_index};
      current_run_number_frames = run_number_frames(run_index);
      current_run_images_list = run_images_list{run_index};
      current_run_images_list = current_run_images_list(1:current_run_number_frames);
      current_run_brightfield_images_list = run_brightfield_images_list{run_index};
      current_run_brightfield_images_list = current_run_brightfield_images_list(1:min(end, current_run_number_frames));
      if show_all_frames
        frame_indices_to_show = 1:current_run_number_frames;
      else
        frame_indices_to_show = randi(current_run_number_frames);
      end
      current_run_point_tracks = run_point_tracks{run_index};
      
      frames_available = ~cellfun(@isempty, current_run_images_list);

      % frame_size = size(robust_read_stack(current_run_images_list{find(frames_available, 1)}));
      frame_size = [];
      current_brightfield_image = [];
      if ~isempty(current_run_brightfield_images_list)
        try
          current_brightfield_image = robust_read_stack(current_run_brightfield_images_list);
          frame_size = size(current_brightfield_image);
        catch raised_error
          % getReport(raised_error, 'extended')
          warning('Cannot read DIC time series!')
        end
      end
      if isempty(frame_size)
        frame_size = size(robust_read_stack(current_run_images_list{find(frames_available, 1)}));
      end
      % fluorescent_image_frame_size
      
      frames_annotated = any(~cellfun(@isempty, current_run_point_tracks), 1).';
      cells_annotated = any(~cellfun(@isempty, current_run_point_tracks), 2);
      frame_cells_within_bounds = cellfun(@(x)(~isempty(x)) && (x(1) >= 1 && x(2) >= 1 && x(1) <= frame_size(2) && x(2) <= frame_size(1)), current_run_point_tracks);
      frames_with_cells_within_bounds = any(frame_cells_within_bounds, 1).';
      frames_to_show = frames_annotated & frames_available & frames_with_cells_within_bounds;
      frame_indices_to_show = frame_indices_to_show(frames_to_show(frame_indices_to_show));
      
      if false
        % Debug info:
        frames_available_annotated_within_bounds_to_show = [frames_available, frames_annotated, frames_with_cells_within_bounds, frames_to_show]
        frame_size
        beep, keyboard
      end
      
      % if false
        % % Debug info:
        % current_run_key, current_run_first_image = current_run_images_list{frame_indices_to_show(1)}
        % beep, keyboard
      % end
      
      fprintf('Currently viewing "%s"\n', current_run_key)
      
      % if length(current_run_brightfield_images_list) == 1
        % current_run_brightfield_image_filename = current_run_brightfield_images_list{1};
        % current_brightfield_image = robust_read_stack(current_run_brightfield_image_filename);
      % end
      % current_brightfield_image = robust_read_stack(current_run_brightfield_images_list);
      
      % For each frame:
      
      for frame_index = frame_indices_to_show
        if show_fluorescence_image || isempty(current_run_brightfield_images_list) || isempty(current_brightfield_image)
          current_frame_image_filename = current_run_images_list{frame_index};
          current_frame_image = robust_read_stack(current_frame_image_filename);
          fprintf('  Frame %+3d, file "%s"\n', frame_index, current_frame_image_filename)
        else
          % if length(current_run_brightfield_images_list) > 1
            % current_run_brightfield_image_filename = current_run_brightfield_images_list{frame_index};
            % current_frame_image = robust_read_stack(current_run_brightfield_image_filename);
          % else
            current_frame_image = current_brightfield_image(:, :, frame_index);
          % end
          fprintf('  Frame %+3d\n', frame_index)
        end
        
        
        % For each cell in this frame:
        
        clf
        imagesc(mean(current_frame_image, 3))
        % imagesc(histeq(contrast_stretch(mean(current_frame_image, 3)), 64))
        axis image
        % colormap(t_cell_info.plotting_info.illustration_colormap_to_use)
        colormap(copper(1024))
        colorbar
        
        hold on
        
        % for current_cell_index = 1:size(current_run_point_tracks, 1)
        for current_cell_index = reshape(find(frame_cells_within_bounds(:, frame_index)), 1, [])
        
          % Check if this cell appears in this frame:
          
          current_synapse_annotations = current_run_point_tracks{current_cell_index, frame_index};
          current_synapse_annotations = reshape(current_synapse_annotations, 2, []).';
          current_synapse_center_rounded = current_synapse_annotations(1, :);

          % Continue if there is no synapse or it is outside of the image:
          % if isempty(current_synapse_center_rounded) || current_synapse_center_rounded(1) < 1 || current_synapse_center_rounded(2) < 1 || current_synapse_center_rounded(1) > size(current_frame_image, 2) || current_synapse_center_rounded(2) > size(current_frame_image, 1)
            % continue
          % end
          
          % current_window_image = imtransform(current_frame_image, maketform('affine', eye(3)), 'UData', [1, size(current_frame_image, 2)], 'VData', [1, size(current_frame_image, 1)], 'Size', [window_size_2d, size(current_frame_image, 3)], 'XData', current_synapse_center_rounded(1) + room * [-1, 1], 'YData', current_synapse_center_rounded(2) + room * [-1, 1]);
          
          % scatter(current_synapse_center_rounded(1), current_synapse_center_rounded(2), 'bx', 'LineWidth', 2)
          plot(current_synapse_annotations(:, 1), current_synapse_annotations(:, 2), 'b-o', 'LineWidth', 2)
          
        end
        hold off
        
        pause
        
      end
    end
    
    keyboard
    
    
  end
  
  
  if ~false
    fprintf('\n\n========================================\nStatistics prior to any processing:\n')
    % run_number_frames_mean = mean(run_number_frames)
    run_number_synapses_marked = cellfun(@(x)sum(sum(~cellfun(@isempty, x))), run_relative_time_tracks);
    run_number_synapses_marked_mean = mean(run_number_synapses_marked)
    run_number_synapses_marked_quartiles = prctile(run_number_frames, [0:25:100])
    run_number_cells_mean = mean(run_number_cells)
    run_number_cells_quartiles = prctile(run_number_cells, [0:25:100])
    number_runs
    run_number_synapses_marked_sum = sum(run_number_synapses_marked)
    run_number_cells_sum = sum(run_number_cells)
    % error
  end
  
  
  
  t_cell_info.synapse_info.run_relative_path_list = run_relative_path_list;
  t_cell_info.synapse_info.run_key_list = run_key_list;
  t_cell_info.synapse_info.run_data_list = run_data_list;
  t_cell_info.synapse_info.run_images_list = run_images_list;
  t_cell_info.synapse_info.run_brightfield_images_list = run_brightfield_images_list;
  t_cell_info.synapse_info.run_image_frame_indices_list = run_image_frame_indices_list;
  t_cell_info.synapse_info.run_image_size_list = run_image_size_list;
  t_cell_info.synapse_info.run_image_voxel_size_list = run_image_voxel_size_list;
  t_cell_info.synapse_info.desired_voxel_size = desired_voxel_size;
  t_cell_info.synapse_info.design_voxel_size = design_voxel_size;
  
  t_cell_info.synapse_info.run_number_cells = run_number_cells;
  t_cell_info.synapse_info.run_number_frames = run_number_frames;
  t_cell_info.synapse_info.run_point_tracks = run_point_tracks;
  t_cell_info.synapse_info.run_relative_time_tracks = run_relative_time_tracks;
  t_cell_info.synapse_info.run_relative_times = run_relative_times;
  t_cell_info.synapse_info.run_number_relative_times = run_number_relative_times;
  t_cell_info.synapse_info.all_relative_times = all_relative_times;
  t_cell_info.synapse_info.number_all_relative_times = number_all_relative_times;
  t_cell_info.synapse_info.all_relative_times_seconds_data = all_relative_times_seconds_data;
  t_cell_info.synapse_info.all_relative_times_seconds = all_relative_times_seconds;
  t_cell_info.synapse_info.all_relative_times_seconds_rounded = all_relative_times_seconds_rounded;
  t_cell_info.synapse_info.number_runs = number_runs;
  
  t_cell_info.synapse_info.condition_sensor_combinations = condition_sensor_combinations;
  t_cell_info.synapse_info.number_condition_sensors = number_condition_sensors;
  t_cell_info.synapse_info.conditions = conditions;
  t_cell_info.synapse_info.sensors = sensors;
  t_cell_info.synapse_info.number_conditions = number_conditions;
  t_cell_info.synapse_info.number_sensors = number_sensors;
  t_cell_info.synapse_info.condition_retrieval_function = condition_retrieval_function;
  t_cell_info.synapse_info.sensor_retrieval_function = sensor_retrieval_function;
  t_cell_info.synapse_info.run_date_retrieval_function = run_date_retrieval_function;
  t_cell_info.synapse_info.unique_condition_sensor_combination_function = @unique_condition_sensor_combination_function;
  t_cell_info.synapse_info.unique_condition_sensor_combination_pair_function = @unique_condition_sensor_combination_pair_function;
  t_cell_info.synapse_info.condition_sensor_combination_ismember = @condition_sensor_combination_ismember;
  t_cell_info.synapse_info.condition_sensor_combination_pair_ismember = @condition_sensor_combination_pair_ismember;
  
  
  if false
    % Debug info:
    % run_data
    run_data_list
    run_data_format_list
    run_point_tracks
    run_relative_time_tracks
    beep, keyboard
  end
  
  if false
    % Debug info:
    run_condition_sensors = [condition_retrieval_function(run_key_list), sensor_retrieval_function(run_key_list)];
    akt_run_indices = strcmpi(run_condition_sensors(:, 2), 'akt');
    akt_run_key_list = run_key_list(akt_run_indices);
    akt_run_data_list = run_data_list(akt_run_indices);
    akt_run_data_list
    % % Wrong:
    % akt_run_cell_counts = cellfun(@(x)max(cellfun(@(y)size(y, 1), x)), akt_run_data_list)
    dbstack, beep, keyboard
  end
  
end  

