function [file_list] = master_script_list_relevant_results(t_cell_info, options)
  % List all the cell-related files in a results directory that the analysis is currently willing to use (so we can exclude or include proteins without deleting files and rerunning the pipeline).
  % 
  % 2013-11-08 tebuck: Copied from master_script_produce_windows.m.
  % 
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  default_options = struct();
  default_options.given_directory = t_cell_info.path_info.regions_location;
  
    
  if ~exist('options', 'var')
    options = default_options; 
  else
    % options = process_options_structure(default_options, options);
    options = process_options_structure(default_options, options, [], true);
  end
  % options


  
  % Variables referenced more than once can be copied to local variables:
  master_script_options = t_cell_info.options;

  
  % Filter a listing rather than checking for the existence of each file:
  % file_list = dir([master_script_options.results_location, options.results_subdirectory, '*.mat']);
  file_list = dir([options.given_directory, '*.mat']);
  file_list = {file_list.name}.';
  % file_list = strcat(regions_location, file_list);
  
  file_list_relevant = false(size(file_list));
  % relevant_file_list = {};
  
  % file_list_tokens = regexp(file_list, ['^[^-]+ - (.+)_(.+)\.xls_frame([0-9]+)_synapse([0-9]+),([0-9]+)\.mat$'], 'tokens');
  file_list_tokens = regexp(file_list, ['^(.+)_frame([0-9]+)_synapse([0-9]+),([0-9]+)\.mat$'], 'tokens');
  file_list_keys = repmat({''}, [length(file_list_tokens), 1]);
  file_list_frames_synapses = nan(length(file_list_tokens), 3);
  matching_files = ~cellfun(@(x)isempty(x), file_list_tokens);
  
  file_list_keys(matching_files) = cellfun(@(x)x{1}{1}, file_list_tokens(matching_files), 'UniformOutput', false);
  file_list_frames_synapses(matching_files, :) = cell2mat(cellfun(@(x)[str2num(x{1}{2}), str2num(x{1}{3}), str2num(x{1}{4})], file_list_tokens(matching_files), 'UniformOutput', false));
  
  if false
    % Debug info:
    % file_list_tokens
    % file_list_keys
    % file_list_frames_synapses
    non_matching_files = file_list(~matching_files, :)
    whos file_list*
    % number_actin_files = length(dir('/projects/cellorganizer/xruan/tcell_project/tcell_excel_results/regions/*_Actin.xls*.mat'))
    % number_actin_files = length(dir([options.given_directory, '*_Actin.xls*.mat']))
    number_actin_files = length(dir([options.given_directory, '*_Actin_*.mat']))
    % number_actin_files_in_list = sum(~cellfun(@isempty, strfind(file_list_keys, '_Actin.xls')))
    number_actin_files_in_list = sum(~cellfun(@isempty, strfind(file_list_keys, '_Actin_')))
    beep, keyboard
  end
  
  
  for run_index = 1:t_cell_info.synapse_info.number_runs
    % Prepare useful variables:
    run_relative_path = t_cell_info.synapse_info.run_relative_path_list{run_index};
    run_key = t_cell_info.synapse_info.run_key_list{run_index};
    run_image_path = t_cell_info.synapse_info.run_images_list{run_index};
    run_brightfield_images = t_cell_info.synapse_info.run_brightfield_images_list{run_index};
    number_cells = t_cell_info.synapse_info.run_number_cells(run_index);
    number_frames = t_cell_info.synapse_info.run_number_frames(run_index);
    cell_synapse_point_tracks = t_cell_info.synapse_info.run_point_tracks{run_index};
    cell_relative_time_tracks = t_cell_info.synapse_info.run_relative_time_tracks{run_index};
    
    files_revelant_to_run_key_indices = find(ismember(file_list_keys, run_key));
    files_revelant_to_run_key = file_list_keys(files_revelant_to_run_key_indices);
    file_list_frames_synapses_revelant_to_run_key = file_list_frames_synapses(files_revelant_to_run_key_indices, :);
    
    for frame_index = 1:number_frames
      current_frame_image_filename = run_image_path{frame_index};
      if isempty(current_frame_image_filename)
        continue
      end
      
      % frame_synapse_locations = run_data{frame_index};
      cell_indices_to_output = 1:size(cell_synapse_point_tracks, 1);
      % Produce windows for only one cell for each run (so maximum number of windows processed is (number_runs * number_all_relative_times):
      if master_script_options.debug_produce_windows_one_cell_per_run
        % cell_indices_to_output = 1;
        cell_indices_to_output = cell_indices_to_output(1:min(master_script_options.debug_produce_windows_number_cells_per_run, end));
      end
      
      for current_cell_index = cell_indices_to_output
      
        % Check if this cell appears in this frame:
        current_synapse_center_rounded = cell_synapse_point_tracks{current_cell_index, frame_index};
        if isempty(current_synapse_center_rounded)
          continue
        end
        current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
        % Add the pattern to the list:
        current_filename_pattern = [sprintf('%s_frame%05d_synapse%05d,%05d', run_key, frame_index, current_synapse_center_rounded), '.*\.mat'];
        
        relevant_indices = find(ismember(file_list_frames_synapses_revelant_to_run_key, [frame_index, current_synapse_center_rounded], 'rows'));
        relevant_indices = files_revelant_to_run_key_indices(relevant_indices);
        file_list_relevant(relevant_indices) = true;
        % % More efficient?
        % file_list_relevant(~file_list_relevant) = file_list_relevant(~file_list_relevant) | (~cellfun(@isempty, regexp(file_list(~file_list_relevant), current_filename_pattern, 'start')));
        
      end
    end
  end
  
  
  file_list = file_list(file_list_relevant);
  file_list = strcat(options.given_directory, file_list);
  
  
  if false
    % Debug info:
    file_list
  end
  
  
end  

