function [final_two_stage_segmentation_options] = master_script_display_novel_segmentations(options)
  % Produces segmentation parameters currently hard-coded in other master_script*.m files. For now just returns the manually optimized parameters for non-Akt cells and automatically optimized for Akt.
  %
  % 2014-05-13 tebuck: Copied from master_script_display_novel_segmentations.m.
  % % Dependencies:
  % % From the File Exchange: export_fig, pmkmp, toolbox_graph, toolbox_fast_marching, affine, Snake3D, Mesh_voxelisation, dirr, inhull
  % % From tebuck: modified version of Jieyue Li's HPA_lib, rasterize_mesh

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu



  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Initialize logging and make subdirectory for storing logs:

  
  start_time = tic;

  filename = mfilename
  base_filename = [filename, '/']; 

  warning('off', 'MATLAB:MKDIR:DirectoryExists');
  warning('off', 'Images:initSize:adjustingMag');
  warning('off', 'MATLAB:xlsread:Mode');
  mkdir(filename)

  pbs_job_id = getenv('PBS_JOBID');
  pbs_job_id = regexprep(pbs_job_id, '[\r\n]', '');
  if isempty(pbs_job_id)
    pbs_job_id = '';
  end
  date_text = datestr(now(), 'yyyymmddHHMMSSFFF')
  if length(dbstack) < 2
    % This might be called by other functions, so don't start a diary in such cases in case the calling functions have already done so:
    diary([base_filename, 'log', date_text, '_', pbs_job_id, '.txt']);
  end

  
  % Profile code?
  if options.use_profiling
    profile('-memory','on'); profile reset; profile on;
  end
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Set defaults and process options structure:
  
  
  default_options = master_script_get_default_options;
  default_options.number_samples_per_round = 20;
  % default_options.number_samples_per_round = 100;
  % default_options.parameter_step_sizes = 20;
  default_options.maximum_parallel_cpus = 8;
  default_options.use_profiling = false;
  
  if ~exist('options', 'var')
    options = default_options;
  else
    options_prior_to_processing = options;
    options = master_script_process_options(options);
    options = process_options_structure(default_options, options);
  end
  
  number_samples_per_round = options.number_samples_per_round;

  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Run master_script part of the way so we don't have to duplicate its setup code here:
  
  
  options_for_master_script = options;
  % Skip everything:
  options_for_master_script.skip_window_saving = true;
  options_for_master_script.skip_segmentation = true;
  options_for_master_script.skip_alignment = true;
  options_for_master_script.skip_morphing = true;
  options_for_master_script.skip_morphing_inside_template = true;
  options_for_master_script.skip_model_building = true;
  options_for_master_script.skip_hierarchical_clustering = true;
  options_for_master_script.skip_model_figure_generation = true;
  options_for_master_script.skip_model_video_saving = true;
  options_for_master_script.skip_illustration_saving = true;
  options_for_master_script.skip_statistical_testing_across_conditions = true;
  options_for_master_script.skip_create_blender_figures = true;
  options_for_master_script.use_profiling = false;

  t_cell_info = master_script(options_for_master_script);
  master_script_options = t_cell_info.options;
  % t_cell_info, dbstack, beep, keyboard, error
  
  fprintf_diary_cycle_function = t_cell_info.log_info.fprintf_diary_cycle_function;
  regions_location = t_cell_info.path_info.regions_location;
  segmentations_location = t_cell_info.path_info.segmentations_location;
  
  % illustration_colormap_to_use_name = t_cell_info.plotting_info.illustration_colormap_to_use_name;
  illustration_colormap_to_use = t_cell_info.plotting_info.illustration_colormap_to_use;
  
  number_runs = t_cell_info.synapse_info.number_runs;
  
  window_center_2d = t_cell_info.preprocessing_info.window_center_2d;
  window_size_2d = t_cell_info.preprocessing_info.window_size_2d;
  background_subtraction_function = t_cell_info.preprocessing_info.background_subtraction_function;
  
  segmentation_rasterization_function = t_cell_info.segmentation_info.segmentation_rasterization_function;
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Segmentation-specific options:
  
  
  two_stage_segmentation_options = struct;
  two_stage_segmentation_options.window_center_2d = window_center_2d;
  two_stage_segmentation_options.seed_radius = master_script_options.seed_radius;
  two_stage_segmentation_options.cell_diameter = master_script_options.cell_diameter;
  two_stage_segmentation_options.background_subtraction_function = background_subtraction_function;
  two_stage_segmentation_options.coarse_snake_options = master_script_options.segmentation_coarse_snake_options;
  two_stage_segmentation_options.fine_snake_options = master_script_options.segmentation_fine_snake_options;
  
  snake_options_limits = struct(...
    'Gamma', [.1, 10] ...
    , 'Sigma1', [.5, 5] ...
    , 'Wline', [-20, 0] ...
    , 'Wedge', [-5, 5] ...
    , 'Sigma2', [.5, 5] ...
    , 'Sigma3', [.5, 5] ...
    , 'Alpha', [0, 5] ...
    , 'Beta', [0, 5] ...
    , 'Delta', [0, 5] ...
    , 'Kappa', [0.5, 5] ...
    , 'Iterations', [100, 500] ...
    , 'GIterations', [0, ceil(master_script_options.cell_diameter)] ...
    , 'Lambda', [.5, .99] ...
    );
    % , 'GIterations', [0, ceil(cell_radius * .5)] ...
  
  
  number_samples_per_round = options.number_samples_per_round;
  
  
  % error('Not yet implemented!')
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Add necessary code subdirectories to the path:

  
  % Should use help to automatically derive further dependencies from this file's dependencies...
  
  % From the Mathworks File Exchange, possibly with modifications:
  addpath(genpath('discretesample'))
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Find all work relevant to given options:
  
  
  % Work is divided up by cell and frame:
  run_relevant_cell_indices = cell(number_runs, 1);
  for run_index = 1:number_runs
    run_image_path = t_cell_info.synapse_info.run_images_list{run_index};
    number_cells = t_cell_info.synapse_info.run_number_cells(run_index);
    number_frames = t_cell_info.synapse_info.run_number_frames(run_index);
    cell_synapse_point_tracks = t_cell_info.synapse_info.run_point_tracks{run_index};
    run_relevant_cell_indices{run_index} = false(size(cell_synapse_point_tracks, 1), number_frames);
    for frame_index = 1:number_frames
      current_frame_image_filename = run_image_path{frame_index};
      if isempty(current_frame_image_filename)
        continue
      end
      for current_cell_index = 1:number_cells
        current_synapse_center_rounded = cell_synapse_point_tracks{current_cell_index, frame_index};
        if isempty(current_synapse_center_rounded)
          continue
        end
        
        first_frame_index = find(~cellfun(@isempty, cell_synapse_point_tracks(current_cell_index, :)), 1);
        if isempty(first_frame_index)
          run_cell_indices_uncomputed{run_index}(current_cell_index) = false;
          continue
        end
        
        run_relevant_cell_indices{run_index}(current_cell_index, frame_index) = true;
      end
    end
  end

  % Keep track of options that have been evaluated and how well they did:
  segmentation_options_list = struct();
  segmentation_options_list.original = two_stage_segmentation_options;
  segmentation_options_list.best = two_stage_segmentation_options;
  segmentation_options_list.tested = {};
  segmentation_options_list.test_cells = {};
  segmentation_options_list.test_cell_evaluations = {};
  segmentation_options_list.test_evaluations = [];
  finished = false;
  iteration_index = 0;
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Keep track of which cells are being displayed to evaluate segmentations:
  
  
  selected_run_cell_frame_indices = nan(0, 3);
  run_uncomputed_image_counts = cellfun(@(x)sum(x(:)), run_relevant_cell_indices);
  run_cell_uncomputed_image_counts = cellfun(@(x)sum(x, 2), run_relevant_cell_indices, 'UniformOutput', false);
  % Should selected_run_cell_frame_indices be different every iteration?
  while size(selected_run_cell_frame_indices, 1) < number_samples_per_round
    selected_run_cell_frame_indices(end + 1, :) = nan;
    % Randomly sample weighted by the number of segmentations per run:
    selected_run = discretesample(run_uncomputed_image_counts ./ sum(run_uncomputed_image_counts), 1);
    % Randomly sample weighted by the number of segmentations per cell:
    selected_cell = discretesample(run_cell_uncomputed_image_counts{selected_run} ./ sum(run_cell_uncomputed_image_counts{selected_run}), 1);
    % Randomly sample a frame:
    selected_frame = discretesample(run_relevant_cell_indices{selected_run}(selected_cell, :) ./ sum(run_relevant_cell_indices{selected_run}(selected_cell, :)), 1);
    selected_run_cell_frame_indices(end, :) = [selected_run, selected_cell, selected_frame];
    selected_run_cell_frame_indices = unique(selected_run_cell_frame_indices, 'rows');
  end


  if false
    % Debug info:
    selected_run_cell_frame_indices
    % % Check if sampled cell/frame index pairs are all valid:
    % [selected_run_cell_frame_indices, cell_index_uncomputed_run_counts(selected_run_cell_frame_indices(:, 1)), selected_run_cell_frame_indices(:, 2) <= cell_index_uncomputed_run_counts(selected_run_cell_frame_indices(:, 1))]
    % Check if sampled cell/frame index pairs are all unique:
    size(selected_run_cell_frame_indices, 1) == size(unique(selected_run_cell_frame_indices, 'rows'), 1)
    dbstack, beep, keyboard
  end
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Write out images to annotate:
  
  
  manual_segmentations_location = regexprep(segmentations_location, '/$', '_manual/');
  mkdir(manual_segmentations_location)
  fprintf_diary_cycle_function('manual_segmentations_location = %s\n', manual_segmentations_location)
  
  annotated_image_filenames = {};
  selected_cells_annotated = false(number_samples_per_round, 1);
  image_annotations = {};
  annotated_image_images = {};
  annotated_image_numbers_slices = [];
  
  for image_index = 1:number_samples_per_round
    run_index = selected_run_cell_frame_indices(image_index, 1);
    cell_index = selected_run_cell_frame_indices(image_index, 2);
    frame_index = selected_run_cell_frame_indices(image_index, 3);
    run_key = t_cell_info.synapse_info.run_key_list{run_index};
    
    current_synapse_center_rounded = t_cell_info.synapse_info.run_point_tracks{run_index}{cell_index, frame_index};
    current_synapse_center_rounded = current_synapse_center_rounded(1, 1:2);
    
    % Check for work done:
    cell_filename = [run_key, sprintf('_frame%05d_synapse%05d,%05d', frame_index, current_synapse_center_rounded)];
    cell_region_filename = [regions_location, cell_filename];
    cell_segmentation_filename = [segmentations_location, cell_filename];
    
    
    % Write the image that will contain the segmentation in the blanked/changed region only if it does not exist:
    image_filename2 = [cell_filename, '_segmented'];
    % manual_segmentations_location, image_filename2
    [can_start, final_name, final_exists] = chunk_start(manual_segmentations_location, image_filename2, '.png');
    if can_start
      fprintf_diary_cycle_function('Writing image to annotate for cell_filename "%s"\n', cell_filename)
      
      a = load([cell_region_filename, '.mat']);
      current_window_image = a.current_window_image;
      preprocessing_structure = two_stage_snake_segmentation_window_preprocessing(current_window_image, two_stage_segmentation_options);
      image_to_segment = preprocessing_structure.image_to_segment;
      
      % Recolor the fluorescent image:
      image_to_show = ind2rgb(floor(reshape_contrast(image_to_segment) .* (size(illustration_colormap_to_use, 1) - 1)) + 1, illustration_colormap_to_use);
      
      % Always write the reference image:
      image_filename = [manual_segmentations_location, cell_filename];
      imwrite(image_to_show, [image_filename, '.png'])
      
      imwrite(image_to_show, final_name)
      chunk_finish(manual_segmentations_location, image_filename2);
    elseif final_exists
      fprintf_diary_cycle_function('Reading for possible annotation cell_filename "%s"\n', cell_filename)
      
      possibly_annotated_image = imread(final_name);
      % dbstack, beep, keyboard
      % Annotated regions are black:
      image_annotation = reshape(ismember(reshape(possibly_annotated_image, [], 3), [0, 0, 0], 'rows'), size(possibly_annotated_image) .* [1, 1, 0] + [0, 0, 1]);
      image_is_annotated = any(image_annotation(:));
      if image_is_annotated
        a = load([cell_region_filename, '.mat']);
        current_window_image = a.current_window_image;
        preprocessing_structure = two_stage_snake_segmentation_window_preprocessing(current_window_image, two_stage_segmentation_options);
        image_to_segment = preprocessing_structure.image_to_segment;
        
        image_annotation = reshape(image_annotation, size(image_to_segment));
        annotated_slices = any(any(image_annotation, 1), 2);
        % Slices without annotation are filled with nan:
        image_annotation = single(image_annotation);
        image_annotation(:, :, ~annotated_slices) = nan;
        annotated_image_filenames{end + 1} = final_name;
        image_annotations{end + 1} = image_annotation;
        annotated_image_images{end + 1} = image_to_segment;
        selected_cells_annotated(image_index) = true;
        
        annotated_image_numbers_slices(end + 1) = size(current_window_image, 3);  
      end
    end
  end
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Optimize segmentation parameters:
  
  % Create functions that will convert between parameter vector and the structure expected by two_stage_snake_segmentation:
  
  segmentation_coarse_parameter_vector_fields = fieldnames(two_stage_segmentation_options.coarse_snake_options);
  segmentation_fine_parameter_vector_fields = fieldnames(two_stage_segmentation_options.fine_snake_options);
  
  function [result_segmentation_parameter_structure] = segmentation_parameter_vector_to_structure(given_segmentation_parameter_vector)
    result_segmentation_parameter_structure = two_stage_segmentation_options;
    for field_index = 1:length(segmentation_coarse_parameter_vector_fields)
      result_segmentation_parameter_structure.coarse_snake_options.(segmentation_coarse_parameter_vector_fields{field_index}) = given_segmentation_parameter_vector(field_index);
    end
    for field_index = (1:length(segmentation_fine_parameter_vector_fields)) + length(segmentation_coarse_parameter_vector_fields)
      result_segmentation_parameter_structure.fine_snake_options.(segmentation_fine_parameter_vector_fields{field_index}) = given_segmentation_parameter_vector(field_index);
    end
  end
  
  function [result_segmentation_parameter_vector] = segmentation_parameter_structure_to_vector(given_segmentation_parameter_structure)
    result_segmentation_parameter_vector = nan(length(segmentation_coarse_parameter_vector_fields) + length(segmentation_fine_parameter_vector_fields), 1);
    for field_index = 1:length(segmentation_coarse_parameter_vector_fields)
      result_segmentation_parameter_vector(field_index) = given_segmentation_parameter_structure.coarse_snake_options.(segmentation_coarse_parameter_vector_fields{field_index});
    end
    for field_index = (1:length(segmentation_fine_parameter_vector_fields)) + length(segmentation_coarse_parameter_vector_fields)
      result_segmentation_parameter_vector(field_index) = given_segmentation_parameter_structure.fine_snake_options.(segmentation_fine_parameter_vector_fields{field_index});
    end
  end

  
  % Create functions that will convert between structures with and without the substructures expected by two_stage_snake_segmentation:

  function [result_structure] = segmentation_parameter_single_to_dual_structure(given_structure)
    result_structure = two_stage_segmentation_options;
    for field_index = 1:length(segmentation_coarse_parameter_vector_fields)
      % result_structure.coarse_snake_options.(segmentation_coarse_parameter_vector_fields{field_index}) = given_structure.(segmentation_coarse_parameter_vector_fields{field_index}(1));
      % result_structure.fine_snake_options.(segmentation_coarse_parameter_vector_fields{field_index}) = given_structure.(segmentation_coarse_parameter_vector_fields{field_index}(2));
      result_structure.coarse_snake_options.(segmentation_coarse_parameter_vector_fields{field_index}) = given_structure.(segmentation_coarse_parameter_vector_fields{field_index});
      result_structure.fine_snake_options.(segmentation_coarse_parameter_vector_fields{field_index}) = given_structure.([segmentation_coarse_parameter_vector_fields{field_index}, '_2']);
      % error
    end
  end
  
  function [result_structure] = segmentation_parameter_dual_to_single_structure(given_structure)
    result_structure = cell2struct(num2cell(nan(length(segmentation_coarse_parameter_vector_fields), 1)), segmentation_coarse_parameter_vector_fields);
    for field_index = 1:length(segmentation_coarse_parameter_vector_fields)
      % result_structure.(segmentation_coarse_parameter_vector_fields{field_index})(1) = given_structure.coarse_snake_options.(segmentation_coarse_parameter_vector_fields{field_index});
      % result_structure.(segmentation_fine_parameter_vector_fields{field_index})(1) = given_structure.fine_snake_options.(segmentation_coarse_parameter_vector_fields{field_index});
      result_structure.(segmentation_coarse_parameter_vector_fields{field_index}) = given_structure.coarse_snake_options.(segmentation_coarse_parameter_vector_fields{field_index});
      result_structure.([segmentation_coarse_parameter_vector_fields{field_index}, '_2']) = given_structure.fine_snake_options.(segmentation_coarse_parameter_vector_fields{field_index});
    end
  end
  
  % Run the optimization:
  
  two_stage_segmentation_options_single = segmentation_parameter_dual_to_single_structure(two_stage_segmentation_options)
  
  evaluation_options = struct;
  evaluation_options.number_images = length(annotated_image_images);
  % warning('>>>> HACK, setting evaluation_options.number_images = 1!'), evaluation_options.number_images = 1;
  evaluation_options.image_function = @(given_index)annotated_image_images{given_index};
  evaluation_options.extra_info_function = @(given_index)struct('window_number_slices', annotated_image_numbers_slices(given_index));
  % evaluation_options.segmentation_function = @(given_image, given_parameters)two_stage_snake_segmentation(two_stage_snake_segmentation_window_preprocessing(given_image, segmentation_parameter_vector_to_structure(given_parameters)), segmentation_parameter_vector_to_structure(given_parameters));
  % evaluation_options.segmentation_function = @(given_image, given_parameters)two_stage_snake_segmentation(two_stage_snake_segmentation_window_preprocessing(given_image, given_parameters), given_parameters);
  % evaluation_options.segmentation_function = @(given_image, given_parameters)two_stage_snake_segmentation(two_stage_snake_segmentation_window_preprocessing(given_image, process_options_structure(two_stage_segmentation_options, given_parameters)), process_options_structure(two_stage_segmentation_options, given_parameters));
  % evaluation_options.segmentation_function = @(given_image, given_parameters, given_extra_info)segmentation_rasterization_function(two_stage_snake_segmentation(two_stage_snake_segmentation_window_preprocessing(given_image, process_options_structure(two_stage_segmentation_options, given_parameters)), process_options_structure(two_stage_segmentation_options, given_parameters)), [window_size_2d, given_extra_info.window_number_slices]);
  evaluation_options.segmentation_function = @(given_image, given_parameters, given_extra_info)segmentation_rasterization_function(two_stage_snake_segmentation(two_stage_snake_segmentation_window_preprocessing(given_image, segmentation_parameter_single_to_dual_structure(process_options_structure(two_stage_segmentation_options_single, given_parameters))), segmentation_parameter_single_to_dual_structure(process_options_structure(two_stage_segmentation_options_single, given_parameters))), [window_size_2d, given_extra_info.window_number_slices]);
  % evaluation_options.evaluation_function = @;
  evaluation_options.label_image_function = @(given_index)image_annotations{given_index};
  
  evaluation_options.parameter_information = struct;
  for field_index = 1:length(segmentation_coarse_parameter_vector_fields)
    current_field_name = segmentation_coarse_parameter_vector_fields{field_index};
    evaluation_options.parameter_information.(current_field_name) = struct;
    evaluation_options.parameter_information.(current_field_name).limits = snake_options_limits.(current_field_name);
    evaluation_options.parameter_information.(current_field_name).initial_value = two_stage_segmentation_options_single.(current_field_name);
    evaluation_options.parameter_information.([current_field_name, '_2']) = struct;
    evaluation_options.parameter_information.([current_field_name, '_2']).limits = snake_options_limits.(current_field_name);
    evaluation_options.parameter_information.([current_field_name, '_2']).initial_value = two_stage_segmentation_options_single.([current_field_name, '_2']);
  end
  
  evaluation_options.finite_difference_step_scale = 1e-1;
  evaluation_options.verbose = 2;
  evaluation_options.number_random_parameter_initializations = 1;
  evaluation_options.maximum_parallel_cpus = options.maximum_parallel_cpus;
  optimization_directory = sprintf('%s%s_%s/', base_filename, date_text, pbs_job_id);
  % optimization_directory = [base_filename, 'optimization_iteration_states/'];
  mkdir(optimization_directory)
  evaluation_options.save_filename = [optimization_directory, 'optimization_state'];
  
  evaluation_options, evaluation_options.parameter_information, evaluation_options.parameter_information.(current_field_name)
  % error
  
  % evaluation_options.parameter_information. = struct;
  
  try
    
    segmentation_result_evaluation = evaluate_segmentation_method_performance(evaluation_options);
    
    best_parameter_set = segmentation_result_evaluation
    
    warning('Did we get here?')
    fprintf_diary_cycle_function('Diary cycle to check...')
  
  catch err
    
    getReport(err, 'extended')
    
  end
  
  % error('Not yet implemented!')

  
  
  % final_two_stage_segmentation_options = best_two_stage_segmentation_options;
  
  
  
  if false
    fprintf('Finished segmenting windows, now check window_sensor_index_list!\n')
    % window_sensor_index_list(window_index) = sensor_retrieval_function(window_segmentation_filename);
    sensors_with_counts = [sensors, num2cell(hist(window_sensor_index_list, 1:number_sensors)).']
    beep, keyboard
  end


  
  
  
  
  
  if options.use_profiling
    p = profile('info');
    profile_filename = [optimization_directory, 'profile_results', '.mat']
    profsave(p, profile_filename);
    save([profile_filename, '.mat'], 'p');
  end
  
  
  time_so_far = toc(start_time)
  
  diary('off')
  
end
