function [result] = remove_confounding_subspace(options)
  % Computes feature space by linear discriminant analysis for data with many features.
  % % Options for:
  % % - 
  %
  % Dependencies
  % ============
  % 
  % File Exchange
  % -------------
  % 
  % * My version of distcorr (original at http://www.mathworks.com/matlabcentral/fileexchange/39905-distance-correlation)
  % 
  %
  % Dependencies for tests
  % ======================
  % 
  % File Exchange
  % -------------
  % 
  % * pmkmp http://www.mathworks.com/matlabcentral/fileexchange/28982-perceptually-improved-colormaps
  % * export_fig http://www.mathworks.com/matlabcentral/fileexchange/23629-exportfig
  %
  %
  % References
  % ==========
  % 
  % * Yu and Yang 2004, "A Direct LDA Algorithm for High-Dimensional Data--with Application to Face Recognition" (note that this implementation assumes rows are vectors, not columns as in the paper)
  %
  %
  % Log
  % ===
  %
  % 2014-08-09 tebuck: Copied from direct_lda.m.

%
% Author: Taraz E. Buck and Xiongtao Ruan
%
% Copyright (C) 2012-2016 Murphy Lab
% Computational Biology Department
% School of Computer Science
% Carnegie Mellon University
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published
% by the Free Software Foundation; either version 2 of the License,
% or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
% 02110-1301, USA.
%
% For additional information visit http://murphylab.web.cmu.edu or
% send email to murphy@cmu.edu


  
  
  % Process options:
  default_options = struct();
  default_options.data = [];
  % First level of grouping:
  default_options.class_assignments = [];
  % Second level of grouping (within those groups):
  default_options.subclass_assignments = [];
  default_options.should_zscore = true;
  default_options.minimum_r_squared = 0.1;
  default_options.verbose = 0;
  % default_options.verbose = 1;
  % default_options.verbose = 2;
  
  default_options.test_name = 'linear';
  
  % default_options.method = 'none';
  default_options.method = 'lda';
  % default_options.method = 'mutual_information'; % Mutual information
  % default_options.method = 'distance_correlation'; % Distance correlation
  
  default_options.distance_correlation_options = struct;
  
  % Shouldn't these affect test names, too?
  default_options.use_cda = false;
  default_options.use_permutation_test = true;
  
  default_options.confidence_level = 0.95;
  
  if ~exist('options', 'var')
    options = default_options; 
  else
    % options = process_options_structure(default_options, options);
    options = process_options_structure(default_options, options, true);
  end
  
  
  if options.use_permutation_test
    % Should this be done here?
    % addpath(genpath('FTM'));
  end
  
  
  test_mode = isempty(options.data);
  
  if test_mode
    
    % For a built-in test, generate some images if options.number_images == 0 (the default value):
    
    set(0, 'DefaultLineLineWidth', 1.5, 'DefaultLineMarkerSize', 9)
    set(0, 'DefaultAxesFontName', 'Helvetica')
    set(0, 'DefaultAxesFontSize', 11)
    
    % options.method = 'distance_correlation';
    
    addpath(genpath('distcorr'))
    addpath(genpath('pmkmp'))
    addpath(genpath('export_fig'))
    addpath(genpath('FTM'));
    base_directory = [mfilename, '/'];
    base_filename = [base_directory];
    % base_filename = [base_filename, sprintf('test %s, method %s, use_cda %s, use_permutation_test %s', options.test_name, options.method, options.use_cda, options.use_permutation_test)];
    base_filename = [base_filename, sprintf('%s, %s, ucda %d, uperm %d', options.test_name, options.method, options.use_cda, options.use_permutation_test)];
    mkdir(base_directory)
    start_job_diary(base_filename)
    
    % Set random seed:
    s = RandStream('mt19937ar', 'Seed', 100);
    RandStream.setDefaultStream(s);
    
    
    if strcmp(options.test_name, 'compute_minimum_r_squared_rules')
      % For method distance_correlation for now, taken from below:
      % error('Make this loop through important variables!')
      warning('Obsolete with unbiased distance correlation method...')
      
      switch options.method
        case 'distance_correlation'
          numbers_data_per_class = ceil(10.^((10:30) / 10.));
          number_numbers_data_per_class = length(numbers_data_per_class);
          distance_correlation_number_samples = 1000;
          distribution_functions = struct;
          distribution_functions.normal = @randn;
          distribution_functions.normal_small = @(varargin)randn(varargin{:}) * 1e-1;
          distribution_functions.normal_large = @(varargin)randn(varargin{:}) * 1e1;
          distribution_functions.uniform = @rand;
          distribution_functions.uniform_small = @(varargin)rand(varargin{:}) * 1e-1;
          distribution_functions.uniform_large = @(varargin)rand(varargin{:}) * 1e1;
          distribution_functions.beta_u = @(varargin)betarnd(.5, .5, varargin{:});
          distribution_functions.beta_left_peak = @(varargin)betarnd(2, 5, varargin{:});
          distribution_functions.beta_right_rising = @(varargin)betarnd(5, 1, varargin{:});
          distribution_function_names = fieldnames(distribution_functions);
          number_distribution_functions = length(distribution_function_names);
          distance_correlation_samples = zeros(number_distribution_functions, number_numbers_data_per_class, distance_correlation_number_samples);
          current_distance_correlation_percentiles = [];
          
          for k = 1:number_distribution_functions
            current_distribution_name = distribution_function_names{k};
            current_distance_correlation_samples = zeros(number_numbers_data_per_class, distance_correlation_number_samples);
            
            [rule_directory, rule_filename] = fileparts([base_filename, sprintf('%s', current_distribution_name)]);
            [can_start, final_name, final_exists, temp_exists] = chunk_start(rule_directory, rule_filename);
            if can_start
              % Figure out a good threshold for random correlation:
              current_sampling_start_time = tic
              fprintf('Computing samples for distribution function %s...\n', current_distribution_name)
              current_distribution_function = distribution_functions.(distribution_function_names{k})
              
              % Set random seed for each chunk:
              s = RandStream('mt19937ar', 'Seed', 1000 + k);
              RandStream.setDefaultStream(s);

              for j = 1:number_numbers_data_per_class
                fprintf('  Computing samples for size %d...\n', numbers_data_per_class(j))
                for i = 1:distance_correlation_number_samples
                  current_distance_correlation_samples(j, i) = distcorr(current_distribution_function(numbers_data_per_class(j), 1), current_distribution_function(numbers_data_per_class(j), 1));
                end
              end
              current_sampling_total_time = toc(current_sampling_start_time)
              current_distance_correlation_percentiles = prctile(current_distance_correlation_samples, 0:10:100, 2);
              current_distance_correlation_percentiles
              diary off, diary on
              save(final_name, 'numbers_data_per_class', 'distance_correlation_number_samples', 'number_distribution_functions', 'current_distance_correlation_samples', 'current_distance_correlation_percentiles', 'current_sampling_total_time')
              chunk_finish(rule_directory, rule_filename);
            elseif final_exists
              load(final_name)
            else
              continue
              % error('Temporary file already exists!')
            end
            
            % distance_correlation_samples(k, :, :) = current_distance_correlation_samples;
            
            % continue
            
            % dbstack, beep, keyboard
            
            % Plot to visually justify rules for distance_correlation thresholds:
            % plot(0:10:100, distance_correlation_percentiles)
            loglog(numbers_data_per_class, current_distance_correlation_percentiles)
            xlabel('numbers_data_per_class', 'Interpreter', 'none')
            ylabel('distance_correlation_percentiles', 'Interpreter', 'none')
            image_filename = [final_name(1:end - 3), ', log num_data vs log distcorr_prctiles'];
            dpi = 150;
            % dpi = 300;
            set(gcf, 'Color', 'white')
            export_fig([image_filename, '.png'], '-painters', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])
            fprintf('Wrote figure "%s"\n', [image_filename, '.png'])
            
            
            % Plot CDFs:
            clf, hold all
            legend_entries = {};
            % plot_colors = pmkmp(number_numbers_data_per_class, 'CubicYF');
            % plot_colors = pmkmp(number_numbers_data_per_class, 'LinearL');
            % plot_colors = pmkmp(number_numbers_data_per_class, 'LinLhot');
            plot_colors = pmkmp(number_numbers_data_per_class, 'IsoL');
            % plot_colors = plot_colors * .9;
            for i = 1:number_numbers_data_per_class
              h = cdfplot(current_distance_correlation_samples(i, :));
              set(h, 'Color', plot_colors(i, :))
              % legend_entries{end + 1} = sprintf('Sample size %d', numbers_data_per_class(i));
              legend_entries{end + 1} = sprintf('%d', numbers_data_per_class(i));
            end
            hold off
            xlabel('Distance correlation', 'Interpreter', 'none')
            ylabel('Cumulative probability', 'Interpreter', 'none')
            % h = legend(legend_entries, 'Location', 'EastOutside'); ylabel(h, 'Sample size'), title(h, 'Sample size')
            
            % caxis(prctile(numbers_data_per_class, [0, 100])), colormap(pmkmp(1024, 'IsoL')), h = colorbar; set(h, 'YScale', 'log', 'YTick', numbers_data_per_class), ylabel(h, 'Sample size')
            % caxis(prctile(numbers_data_per_class, [0, 100])), colormap(plot_colors), h = colorbar; set(h, 'YScale', 'log', 'YTick', numbers_data_per_class), ylabel(h, 'Sample size')
            caxis(.5 + [0, number_numbers_data_per_class]), colormap(plot_colors), h = colorbar; set(h, 'YTick', 1:number_numbers_data_per_class, 'YTickLabel', numbers_data_per_class), ylabel(h, 'Sample size')
            image_filename = [final_name(1:end - 3), ', distance correlation CDFs'];
            dpi = 150;
            % dpi = 300;
            set(gcf, 'Color', 'white')
            export_fig([image_filename, '.png'], '-painters', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])
            fprintf('Wrote figure "%s"\n', [image_filename, '.png'])
            
            
            % distance_r_squared_99th_percentile = prctile(distance_correlation_samples, 99).^2
            % minimum_distance_r_squared = 1 - (1 - distance_r_squared_99th_percentile) * .9
            distance_r_squared_max = max(distance_correlation_samples).^2;
            % minimum_distance_r_squared = 1 - (1 - distance_r_squared_max) * .9
            minimum_distance_r_squared = 1 - (1 - distance_r_squared_max) * (1 - 2 / distance_correlation_number_samples);
            fprintf('distance_r_squared_max %f, minimum_distance_r_squared %f\n', distance_r_squared_max, minimum_distance_r_squared)
            options.minimum_r_squared = minimum_distance_r_squared;
            % Do this automatically upon request? Put it into a separate function? Can we call this file/function defined here separately?
            
          end
          
          error('Finished?')
        otherwise
          error('Not yet implemented!')
      end
      
    else
      
      % Run statistical significance tests:
      options.data = zeros(0, 2);
      options.class_assignments = zeros(0, 1);
      options.subclass_assignments = zeros(0, 1);
      number_classes = 10;
      number_subclasses = 3;
      number_data_per_class = 30;
      % number_data_per_class = 90;
      % number_data_per_class = 300;
      switch lower(options.test_name)
        case 'linear'
          % Do nothing else
        case 'dependence'
          number_subclasses = 20;
          number_data_per_class = 60;
        case 'data-like'
          number_subclasses = 5;
          number_data_per_class = 50;
      end
      number_embedding_dimensions = 20;
      number_noise_dimensions = 20;
      % number_embedding_dimensions = 1;
      % number_noise_dimensions = 1;
      
      number_confounding_dimensions = number_embedding_dimensions;
      if ismember(options.test_name, {'linear-no-confounded', 'linear-no-variation'})
        % Debug info:
        % Use this to check if statistical significance is present without any confounding features:
        % warning('>>>>HACK, setting number_confounding_dimensions = 0!')
        number_confounding_dimensions = 0;
      end
      
      % class_noise_scale = 0;
      class_noise_scale = 1e-2;
      % class_noise_scale = 1e-1;
      class_radius = sqrt(number_classes);
      if ismember(options.test_name, {'linear-no-informed', 'linear-no-variation'})
        % Debug info:
        % Use this to check if statistical significance is present without any legitimately separating features:
        % warning('>>>>HACK, setting class_radius = 0!')
        class_radius = 0;
      end
      % noise_size = 1e-2;
      noise_size = 5e-2;
      % noise_size = 1e-1;
      % confounding_noise_size = 0;
      % confounding_noise_size = 1e-2;
      % confounding_noise_size = 5e-2;
      % confounding_noise_size = 1e-1;
      confounding_noise_size = 2e0;
      % confounding_noise_size = 1e1;
      % confounding_noise_size = 1e2;
      if ismember(options.test_name, {'linear-no-confounded', 'linear-no-variation'})
        % Debug info:
        % Use this to check if statistical significance is present without any confounding features:
        % warning('>>>>HACK, setting confounding_noise_size = 0!')
        confounding_noise_size = 0;
      end
      confounding_noise_size
      
      % class_repetition = 1
      % class_repetition = 2
      class_repetition = 3
      
      for class_index = 1:number_classes
        % current_angle = (class_index - 1) / number_classes * 2 * pi;
        % Make every two classes the same so we can check for statistically significant differences before but not after correction:
        current_angle = (class_index - mod(class_index - 1, class_repetition) - 1) / (number_classes) * 2 * pi;
        % current_angle, pause
        options.data = [options.data; mvnrnd([cos(current_angle), sin(current_angle)] * class_radius, [.5, .25; .25, .5] * class_noise_scale, number_data_per_class)];
        options.class_assignments = [options.class_assignments; zeros(number_data_per_class, 1) + class_index];
        % options.subclass_assignments = [options.subclass_assignments; ceil((1:number_data_per_class)' / number_data_per_class * number_subclasses)];
        options.subclass_assignments = [options.subclass_assignments; ceil((1:number_data_per_class)' / number_data_per_class * number_subclasses) + (class_index - 1) * number_subclasses];
        % dbstack, beep, keyboard
      end
      
      % Data structure is [informative, noise, confounded]
      
      % Embedding features (informative):
      options.data = options.data * (randn(2, number_embedding_dimensions) .* (1 / number_embedding_dimensions));
      % Noise features:
      options.data = [options.data, zeros(size(options.data, 1), number_noise_dimensions)];
      % Confounding features:
      options.data = [options.data, options.data(:, 1:number_confounding_dimensions)];
      
      % Modify some features based on subclass to confound:
      subclasses = unique(options.subclass_assignments);
      for subclass_index = 1:length(subclasses)
        current_datum_indices = find(options.subclass_assignments == subclasses(subclass_index));
        current_feature_indices = size(options.data, 2) - (number_confounding_dimensions - 1:-1:0);
        current_perturbation = repmat(randn(1, length(current_feature_indices)), length(current_datum_indices), 1) * confounding_noise_size;
        current_perturbation_proportion = repmat(rand(1, length(current_feature_indices)), length(current_datum_indices), 1) * confounding_noise_size;
        if false
          % dbstack, beep, keyboard
          error('Not updated')
          clf, subplot(1, 2, 1), imagesc(options.data(current_datum_indices, current_feature_indices)), subplot(1, 2, 2), imagesc(options.data(current_datum_indices, current_feature_indices) + current_perturbation), colorbar, colormap('jet')
          dbstack, beep, pause
        end
        options.data(current_datum_indices, current_feature_indices) = options.data(current_datum_indices, current_feature_indices) .* (1 - current_perturbation_proportion) + current_perturbation .* current_perturbation_proportion;
        % error('modify some feats based on subclass_assignments!')
      end
      % Add noise:
      options.data = options.data + randn(size(options.data)) * noise_size;
      
      % options.method = 'lda';
      % % options.minimum_r_squared = 1e-2;
      % options.minimum_r_squared = 1e-4;
      
      switch options.method
        case 'distance_correlation'
          if false
            warning('Obsolete with unbiased distance correlation method...')
            % Figure out a good threshold for random correlation:
            distance_correlation_number_samples = 1000;
            distance_correlation_samples = arrayfun(@(x)distcorr(randn(number_data_per_class, 1), randn(number_data_per_class, 1)), 1:distance_correlation_number_samples);
            distance_correlation_percentiles = prctile(distance_correlation_samples, 0:10:100);
            fprintf('distance_correlation_number_samples %d\n', distance_correlation_number_samples)
            fprintf('distance_correlation_percentiles %s\n', mat2str(distance_correlation_percentiles, 4))
            % distance_r_squared_99th_percentile = prctile(distance_correlation_samples, 99).^2
            % minimum_distance_r_squared = 1 - (1 - distance_r_squared_99th_percentile) * .9
            distance_r_squared_max = max(distance_correlation_samples).^2;
            % minimum_distance_r_squared = 1 - (1 - distance_r_squared_max) * .9
            minimum_distance_r_squared = 1 - (1 - distance_r_squared_max) * (1 - 2 / distance_correlation_number_samples);
            fprintf('distance_r_squared_max %f, minimum_distance_r_squared %f\n', distance_r_squared_max, minimum_distance_r_squared)
            options.minimum_r_squared = minimum_distance_r_squared;
            % Do this automatically upon request? Put it into a separate function? Can we call this file/function defined here separately?
          end
      end
      options
      
      close all
      set(gcf, 'Visible', 'off')
      
    end
      
  end
  
  
  data = options.data;
  class_assignments = options.class_assignments;
  classes = unique(class_assignments);
  number_classes = length(classes);
  [number_data, number_features] = size(data);
  subclass_assignments = options.subclass_assignments;
  subclasses_per_class = arrayfun(@(x)unique(subclass_assignments(class_assignments == x)), classes, 'UniformOutput', false);
  should_zscore = options.should_zscore;
  
  if options.should_zscore
    data = zscore(data);
  end
  
  
  if test_mode
    
    if false
      % Show plots of distributions per class per feature:
      for feature_index = 1:size(options.data, 2)
        support = prctile(options.data(:, feature_index), [0, 100]);
        support_samples = (0:1000) ./ 1000 .* abs(diff(support)) + min(support);
        clf
        % hold on
        hold all
        legend_entries = {};
        for class_index = 1:number_classes
          kde_fit = fitdist(options.data(class_assignments == classes(class_index), feature_index), 'kernel');
          % support = prctile(options.data(class_assignments == classes(class_index), feature_index), [0, 100]);
          % support_samples = (0:1000) ./ 1000 .* abs(diff(support)) + min(support);
          plot(support_samples, kde_fit.pdf(support_samples))
          legend_entries{end + 1} = sprintf('Class %d', class_index);
        end
        title(sprintf('Fits of feature %d', feature_index))
        legend(legend_entries)
        hold off
        % beep
        dbstack
        pause
      end
    end
    


    clf
    % % plot_to_show = 'one from each'
    % plot_to_show = 'embedding'
    % % plot_to_show = 'noise'
    % % plot_to_show = 'confounding'
    if number_confounding_dimensions > 0
      plots_to_show = {'one from each', 'embedding', 'noise', 'confounding'};
    else
      plots_to_show = {'embedding', 'noise'};
    end
    for plot_to_show_index = 1:length(plots_to_show)
      plot_to_show = plots_to_show{plot_to_show_index};
      fprintf('plot_to_show %s\n', plot_to_show)
      label_x = '';
      label_y = '';
      label_z = '';
      a = [];
      switch plot_to_show
        case 'one from each'
          a = data(:, [1, number_embedding_dimensions + 1, number_embedding_dimensions + number_noise_dimensions + 1]);
          label_x = 'Embedding';
          label_y = 'Embedding';
          label_z = 'Noise';
        case 'embedding'
          a = data(:, 1:number_embedding_dimensions);
        case 'noise'
          a = data(:, number_embedding_dimensions + (1:number_noise_dimensions));
        case 'confounding'
          a = data(:, number_embedding_dimensions + number_noise_dimensions + (1:number_confounding_dimensions));
      end
      a = [a, zeros(number_data, max(3 - size(data, 2), 0))];
      % scatter(a(:, 1), a(:, 2), [], class_assignments)
      scatter3(a(:, 1), a(:, 2), a(:, 3), [], class_assignments, '.')
      xlabel(label_x)
      ylabel(label_y)
      zlabel(label_z)
      axis equal
      title(sprintf('plot_to_show %s', plot_to_show), 'Interpreter', 'none')
      colormap('jet')
      % dbstack, beep, pause
      
      image_filename = [base_filename, sprintf(', scatter %s', plot_to_show)];
      dpi = 150;
      % dpi = 300;
      set(gcf, 'Color', 'white')
      export_fig([image_filename, '.png'], '-painters', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])
      fprintf('Wrote figure "%s"\n', [image_filename, '.png'])
    end
  end
  
  
  function display_pairwise_tests(given_data_sets, given_title)
    
    clf
    
    if ~iscell(given_data_sets)
      given_data_sets = {given_data_sets};
    end
    
    if ~iscell(given_title)
      given_title = repmat({given_title}, size(given_data_sets));
    end
    
    comparison_matrices = {};
    bonferroni_correction = (number_classes * (number_classes - 1) / 2) * length(given_data_sets);
    if options.use_permutation_test
      number_permutations_scale = 10;
      % warning('>>>HACK, number_permutations_scale = 1'), number_permutations_scale = 1;
      number_permutations_scale
      number_permutations = ceil(bonferroni_correction / (1 - options.confidence_level) * number_permutations_scale);
    else
      number_permutations = 0;
      number_permutations_scale = 0;
    end
    fprintf('number_permutations %d, number_permutations_scale %d\n', number_permutations, number_permutations_scale)
    
    for data_set_index = 1:length(given_data_sets)
      
      
      current_filename = [strrep(base_filename, base_directory, ''), sprintf(', title %s, set %d', given_title{data_set_index}, data_set_index)];
      [can_start, final_name, final_exists, temp_exists] = chunk_start(base_directory, current_filename, []);
      if can_start
        
        p_values = nan(number_classes);
        for class_index1 = 1:number_classes
          current_class1 = classes(class_index1);
          current_class_indices1 = class_assignments == current_class1;
          current_class_data1 = given_data_sets{data_set_index}(current_class_indices1, :);
          for class_index2 = class_index1 + 1:number_classes
            current_class2 = classes(class_index2);
            current_class_indices2 = class_assignments == current_class2;
            current_class_data2 = given_data_sets{data_set_index}(current_class_indices2, :);
            
            if options.use_cda
              p_values(class_index1, class_index2) = cda_manova_p_value_computation_function(current_class_data1, current_class_data2, options.confidence_level, number_permutations);
            else
              p_values(class_index1, class_index2) = manova_p_value_computation_function(current_class_data1, current_class_data2, options.confidence_level, number_permutations);
            end
            p_values(class_index2, class_index1) = p_values(class_index1, class_index2);
          end
        end
        
        save(final_name, 'p_values', 'bonferroni_correction', 'number_permutations', 'number_permutations_scale')
        chunk_finish(base_directory, current_filename);
      elseif final_exists
        load(final_name, 'p_values')
      else
        error('Test being computed elsewhere')
      end
      
      % Correct roughly using conservative Bonferroni correction:
      p_values = p_values .* bonferroni_correction;
      % p_values
      % negative_log10_p_values = -log10(p_values)
      comparison_matrices{data_set_index} = p_values;
    end
    
    % smallest_p_value = min(cellfun(@(x)min(x(:)), comparison_matrices));
    % smallest_p_value = min(cellfun(@(x)min(x(x > 0)), comparison_matrices));
    smallest_p_value = min(cellfun(@(x)min(x(:)), comparison_matrices));
    largest_p_value = max(cellfun(@(x)max(x(:)), comparison_matrices));
    if smallest_p_value == largest_p_value
      smallest_p_value = smallest_p_value + .5;
      largest_p_value = largest_p_value - .5;
    end
    
    for data_set_index = 1:length(given_data_sets)
      % if length(given_data_sets) == 1
      % Draw the matrix itself:
      matrix_to_plot = -log10(comparison_matrices{data_set_index});
      matrix_to_plot(~isfinite(matrix_to_plot)) = nan;
      imagesc(matrix_to_plot)
      axis image
      colormap(pmkmp(1024, 'CubicYF'))
      % colormap(copper(1024))
      % colormap(summer(1024))
      colorbar_handle = colorbar;
      ylabel(colorbar_handle, '-log10(p-value)', 'Interpreter', 'none')
      % color_limits = [0, 3];
      % color_limits = [0, 10];
      % color_limits = [0, -log10(smallest_p_value)];
      % color_limits = [-log10(largest_p_value), -log10(smallest_p_value)];
      color_limits = [-log10((1 - options.confidence_level) * 1e1), -log10((1 - options.confidence_level) * 1e-5)];
      color_limits = sort(color_limits);
      caxis(color_limits)
      title(given_title{data_set_index})
      image_filename = [base_filename, sprintf(', title %s, set %d', given_title{data_set_index}, data_set_index)];
      dpi = 150;
      % dpi = 300;
      set(gcf, 'Color', 'white')
      export_fig([image_filename, '.png'], '-painters', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])
      fprintf('Wrote figure "%s"\n', [image_filename, '.png'])
      % end
      
      if data_set_index > 1
        % And compare to the first matrix if more than one:
        ratio = comparison_matrices{data_set_index} ./ comparison_matrices{1};
        % ratio = comparison_matrices{1} ./ comparison_matrices{data_set_index}
        matrix_to_plot = -log10(ratio);
        % matrix_to_plot = ratio;
        matrix_to_plot(~isfinite(matrix_to_plot)) = nan;
        imagesc(matrix_to_plot)
        axis image
        colormap(pmkmp(1024, 'CubicYF'))
        % colormap(copper(1024))
        % colormap(summer(1024))
        colorbar_handle = colorbar;
        ylabel(colorbar_handle, sprintf('-log10(ratio of %dth to first p-values)', data_set_index), 'Interpreter', 'none')
        % ylabel(colorbar_handle, sprintf('ratio of %dth to first p-values', data_set_index), 'Interpreter', 'none')
        % ylabel(colorbar_handle, sprintf('-log10(ratio of first to %dth p-values)', data_set_index), 'Interpreter', 'none')
        % % ylabel(colorbar_handle, sprintf('ratio of first to %dth p-values', data_set_index), 'Interpreter', 'none')
        % caxis([0, 3])
        % caxis([0, 10])
        title(given_title{data_set_index})
        image_filename = [base_filename, sprintf(', title %s, set %d ratio', given_title{data_set_index}, data_set_index)];
        dpi = 150;
        % dpi = 300;
        set(gcf, 'Color', 'white')
        export_fig([image_filename, '.png'], '-painters', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])
        fprintf('Wrote figure "%s"\n', [image_filename, '.png'])
      end

    end
    
  end
  
  
  if false && test_mode
    % Show that there are statistically significant differences between some pairs of classes even though the means for every pair of classes are the same:
    display_pairwise_tests(data, 'Before correction');
  end
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Step 1: For each class, transform by LDA and concatenate the resulting bases:
  
  
  function [result_value] = mutual_information_cost_function(given_data, given_direction)
    % % Decrease magnitude at radii other than 1?
    
    % Looks like "Faivishevsky and Goldberger 2012 - Dimensionality reduction based on non-parametric mutual information.pdf" used kNN and so naive implementation will have quadratic costs. :( kd-tree that works in high dimension?
    % result_value = 
    
    % Try KDE for smaller-time approximation? Still quadratic time...:
    % Project onto given_direction:
    given_data_projected = given_data * given_direction;
    given_data_projected_fit = fitdist(given_data_projected, 'kernel', 'frequency', current_weights);
    % kde_fit = fitdist(given_data_projected, 'kernel', 'frequency', current_weights, 'width', .025);
    
    % result_value = ;
  end
  
  
  function [result_value] = distance_correlation_cost_function(given_data, given_variable_of_interest, given_direction)
    % % Decrease magnitude at radii other than 1?
    
    % Project onto given_direction:
    given_data_projected = given_data * given_direction;
    
    if false && test_mode
      clf
      scatter(given_data_projected, given_variable_of_interest, '.')
      axis equal
      xlabel('given_data_projected', 'Interpreter', 'none')
      ylabel('given_variable_of_interest', 'Interpreter', 'none')
      % colormap(pmkmp(1024, 'CubicYF'))
      colormap(jet(1024))
      % Data structure is [informative, noise, confounded]
      number_embedding_dimensions, number_noise_dimensions, number_confounding_dimensions
      dbstack, beep
      keyboard
      % pause
    end
    
    number_data = size(given_data, 1);
    number_data_to_use = min(number_data, 2000);
    result_value = -distcorr(given_data_projected(1:number_data_to_use, :), given_variable_of_interest(1:number_data_to_use, :));
  end
  
  
  % These should be functions taking a matrix and a vector and returning linear correlation-like values with absolute value between zero and 1:
  feature_evaluation_functions = struct;
  % feature_evaluation_functions.mutual_information = [];
  % Version of mutual information with values between zero and 1:
  feature_evaluation_functions.normalized_redundancy = [];
  feature_evaluation_functions.distance_correlation = @distcorr;
  function [result] = distance_correlation_p_value_function(varargin)
    [~, ~, result] = distcorr(varargin);
  end
  feature_evaluation_functions.distance_correlation_p_value = @distance_correlation_p_value_function;
  
  
  lda_options = struct;
  lda_options.should_zscore = false;
  lda_options.maximum_dimensionality = inf;
  lda_options.minimum_dimensionality = inf;
  % confounding_range = [];
  confounding_range = zeros(number_features, 0);
  for class_index = 1:number_classes
    current_class = classes(class_index);
    current_class_indices = class_assignments == current_class;
    lda_options.data = data(current_class_indices, :);
    lda_options.class_assignments = subclass_assignments(current_class_indices, :);
    switch options.method
      case 'none'
        % Do nothing
      case 'lda'
        current_class_data_lda = direct_lda(lda_options);
        fprintf('Just called direct_lda\n')
        current_class_data_transform = current_class_data_lda.sphering_transform;
        confounding_range = [confounding_range, current_class_data_transform];
      case fieldnames(feature_evaluation_functions)
        if true
          current_handle = feature_evaluation_functions.(options.method);
          if strcmp(func2str(current_handle), 'distcorr') && class_index == 1 && size(lda_options.data, 1) >= 500
            warning('Plan on using Nystrom extension for distcorr!')
          end
          basis_evaluation_start_time = tic;
          basis_evaluation_last_message_time = basis_evaluation_start_time;
          for basis_index = 1:number_features
            current_basis = zeros(number_features, 1);
            current_basis(basis_index) = 1;
            current_correlation = current_handle(lda_options.data(:, basis_index), lda_options.class_assignments);
            if current_correlation.^2 >= options.minimum_r_squared
              confounding_range = [confounding_range, current_basis];
            end
            if false && test_mode
              fprintf('Basis %d current_correlation %0.4f\n', basis_index, current_correlation)
            end
            if options.verbose > 0 && toc(basis_evaluation_last_message_time) >= 30
              fprintf('Correlation computation for class %d took %3.2f seconds so far, done in %3.2f seconds\n', class_index, toc(basis_evaluation_start_time), toc(basis_evaluation_start_time) / basis_index * (number_features - basis_index))
              basis_evaluation_last_message_time = tic;
            end
          end
        else
          % Old way based on optimization, might return to it sometime:
          % error('Not yet implemented!')
          current_class_data_lda = direct_lda(lda_options);
          fprintf('Just called direct_lda\n')
          current_class_data_transform = current_class_data_lda.sphering_transform;
          for basis_index = 1:size(current_class_data_transform, 2)
            current_basis = current_class_data_transform(:, basis_index);
            % 5 times as much as default:
            % current_basis = fminunc(@(x)distance_correlation_cost_function(lda_options.data, lda_options.class_assignments, x), current_basis, optimset('MaxFunEvals', 500 * size(lda_options.data, 2), 'MaxIter', 1000, 'FinDiffRelStep', 1e-3, 'DiffMinChange', 1e-5));
            % current_basis = fminunc(@(x)distance_correlation_cost_function(lda_options.data, lda_options.class_assignments, x), current_basis, optimset('Algorithm', 'lm-line-search', 'MaxFunEvals', 500 * size(lda_options.data, 2), 'MaxIter', 1000, 'FinDiffRelStep', 1e-5, 'DiffMinChange', 1e-5, 'Display', 'notify'));
            % current_basis = fminunc(@(x)distance_correlation_cost_function(lda_options.data, lda_options.class_assignments, x), current_basis, optimset('LargeScale', 'on', 'MaxFunEvals', 500 * size(lda_options.data, 2), 'MaxIter', 1000, 'FinDiffRelStep', 1e-5, 'DiffMinChange', 1e-5, 'Display', 'notify'));
            % current_basis = fminunc(@(x)distance_correlation_cost_function(lda_options.data, lda_options.class_assignments, x), current_basis, optimset('Algorithm', 'levenberg-marquardt', 'MaxFunEvals', 500 * size(lda_options.data, 2), 'MaxIter', 1000, 'FinDiffRelStep', 1e-5, 'DiffMinChange', 1e-5, 'Display', 'notify'));
            current_basis = fminunc(@(x)distance_correlation_cost_function(lda_options.data, lda_options.class_assignments, x), current_basis, optimset('LargeScale', 'on', 'Algorithm', 'interior-point', 'MaxFunEvals', 500 * size(lda_options.data, 2), 'MaxIter', 1000, 'FinDiffRelStep', 1e-5, 'DiffMinChange', 1e-5, 'TolX', 1e-4, 'TypicalX', current_basis * 0 + sqrt(1 / length(current_basis)), 'Display', 'notify'));
            confounding_range = [confounding_range, current_basis];
          end
        end
    end
  end
  
  if false && test_mode
    dbstack, beep, keyboard
  end
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Step 2: Project all data into a non-confounding space:
  
  
  switch options.method
    case 'none'
      data_transform = eye(number_features);
    case 'lda'
      data_transform = null(confounding_range');
    case fieldnames(feature_evaluation_functions)
      data_transform = eye(number_features);
      bad_features = max(confounding_range, [], 2) > 0;
      data_transform(:, bad_features) = [];
      if test_mode
        fprintf('Allegedly confounding features found %s\n', mat2str(find(bad_features), 4))
      end
  end
  data_corrected = data * data_transform;
  
  
  if test_mode
    clf
    subplot(1, 4, 1), imagesc(confounding_range), axis image, title('confounding_range', 'Interpreter', 'none')
    colorbar
    subplot(1, 4, 2), imagesc(data_transform), axis image, title('data_transform', 'Interpreter', 'none')
    subplot(1, 4, 3), imagesc(data), axis image, title('data', 'Interpreter', 'none')
    subplot(1, 4, 4), imagesc(data_corrected), axis image, title('data_corrected', 'Interpreter', 'none')
    colormap(pmkmp(1024, 'CubicYF'))
    % Data structure is [informative, noise, confounded]
    % number_embedding_dimensions, number_noise_dimensions, number_confounding_dimensions
    fprintf('number_embedding_dimensions %d, number_noise_dimensions %d, number_confounding_dimensions %d\n', number_embedding_dimensions, number_noise_dimensions, number_confounding_dimensions)
    if false
      dbstack, beep, keyboard
    end
    image_filename = [base_filename, sprintf(', transforms')];
    dpi = 150;
    % dpi = 300;
    set(gcf, 'Color', 'white')
    export_fig([image_filename, '.png'], '-painters', '-png', '-a1', '-nocrop', ['-r', num2str(dpi)])
    fprintf('Wrote figure "%s"\n', [image_filename, '.png'])
  end
  
  
  if test_mode
    % Show that there are statistically significant differences between some pairs of classes even though the means for every pair of classes are the same:
    display_pairwise_tests({data, data_corrected}, {'Before correction', 'After correction'});
  end
  
  
  result = struct;
  % result.data = data;
  result.data_corrected = data_corrected;
  result.data_transform = data_transform;
  % error('Not yet implemented!')
  
  
  if test_mode
    diary off
  end
  
  
end


function [result_p_value] = manova_p_value_computation_function(given_data1, given_data2, confidence_level, number_permutations)
  if ~exist('confidence_level', 'var')
    confidence_level = 0.95;
  end
  if ~exist('number_permutations', 'var')
    number_permutations = 0;
  end
  if number_permutations > 0
    data_distances = squareform(pdist([given_data1; given_data2]));
    result = f_npManova(data_distances, [zeros(size(given_data1, 1), 1) + 1; zeros(size(given_data2, 1), 1) + 2], number_permutations, 0, []);
    result_p_value = result(1).p;
  else
    [~, result_p_value, ~] = manova1([given_data1; given_data2], [zeros(size(given_data1, 1), 1) + 1; zeros(size(given_data2, 1), 1) + 2], confidence_level);
    result_p_value = result_p_value(1);
  end
  
  if false
    % Debug info:
    scatter([given_data1(:, 1); given_data2(:, 1)], rand(size(given_data1, 1) + size(given_data2, 1), 1), [], [zeros(size(given_data1, 1), 1) + 1; zeros(size(given_data2, 1), 1) + 2], '.')
    title('Inside manova_p_value_computation_function', 'Interpreter', 'none')
    dbstack, beep
    % keyboard
    pause
  end
end

function [result_p_value] = cda_manova_p_value_computation_function(given_data1, given_data2, confidence_level, number_permutations)
  if ~exist('confidence_level', 'var')
    confidence_level = 0.95;
  end
  if ~exist('number_permutations', 'var')
    number_permutations = 0;
  end
  
  combined_data = [given_data1; given_data2];
  combined_classes = [zeros(size(given_data1, 1), 1) + 1; zeros(size(given_data2, 1), 1) + 2];
  
  % My implementation of Yu and Yang 2004, "A Direct LDA Algorithm for High-Dimensional Data--with Application to Face Recognition:"
  combined_data_cda = direct_lda(struct('data', combined_data, 'class_assignments', combined_classes, 'maximum_dimensionality', 1, 'minimum_r_squared', 0));
  combined_data_cda = combined_data_cda.data_sphered;
  
  if false
    % Debug info:
    a = combined_data_cda;
    a = [a, zeros(size(a, 1), max(3 - size(a, 2), 0))];
    scatter3(a(:, 1), a(:, 2), a(:, 3), [], combined_classes, '.')
    axis equal
    title('Inside cda_manova_p_value_computation_function', 'Interpreter', 'none')
    
    dbstack, beep
    % keyboard
    pause
  end
  
  result_p_value = manova_p_value_computation_function(combined_data_cda(1:size(given_data1, 1), :), combined_data_cda(size(given_data1, 1) + 1:end, :), confidence_level, number_permutations);
  
end
